import UIKit

class AppTabBarController: UITabBarController, UITabBarControllerDelegate, WalkthroughViewControllerDelegate {
    var window: UIWindow?

    struct Constants {
        static let reuseIdentifier = String(describing: AppTabBarController.self)
        static let storyboardName = "Documents"
    }
    
    static func buildViewController() -> AppTabBarController {
        let controller = UIStoryboard(name: Constants.storyboardName,
                                      bundle: .main).instantiateViewController(withIdentifier: Constants.reuseIdentifier)
        return controller as! AppTabBarController
    }
    
    private var observation: NSKeyValueObservation?
    
    required init?(coder: NSCoder) {
        super.init(coder: coder)
    }
    
    deinit {
        observation = nil
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        window = UIWindow(frame: UIScreen.main.bounds)
        self.delegate = self
        
        observation = observe(\.tabBar.semanticContentAttribute) { value, change in
            self.loadLocalization()
        }
        
        setupUI()
    }
    
    private func setupUI() {
        
        loadLocalization()
        
        tabBar.items?[0].image = .folderOpen
        tabBar.items?[2].image = .gallery
        
        tabBar.items?[1].isEnabled = false
        
        tabBar.tintColor = ColorStyle.primary.uiColor
        
        tabBar.unselectedItemTintColor = ColorStyle.gray600.uiColor
    }
    
    private func loadLocalization() {
        tabBar.items?[0].title = .fileManager.localized
        tabBar.items?[2].title = .studio.localized
        
        let attribute = [NSAttributedString.Key.font : UIFont.medium(16)]
        
        tabBar.items?[0].setTitleTextAttributes(attribute, for: .normal)
        tabBar.items?[2].setTitleTextAttributes(attribute, for: .normal)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        if UserDefaults.standard.showWalktroughAtLaunch {
            perform(#selector(showWalkTrough), with: self, afterDelay: 0.88)
        }
    }
    
    let walkthroughs = [
        WalkthroughModel(title: "Quick Overview",
                         subtitle: "Quickly visualize important business metrics. The overview in the home tab shows the most important metrics to monitor how your business is doing in real time.",
                         icon: "analytics-icon"),
        WalkthroughModel(title: "Analytics",
                         subtitle: "Dive deep into charts to extract valuable insights and come up with data driven product initiatives, to boost the success of your business.",
                         icon: "bars-icon"),
        WalkthroughModel(title: "Dashboard Feeds",
                         subtitle: "View your sales feed, orders, customers, products and employees.",
                         icon: "activity-feed-icon"),
        WalkthroughModel(title: "Get Notified",
                         subtitle: "Receive notifications when critical situations occur to stay on top of everything important.",
                         icon: "bell-icon"),
    ]
    
    @objc private func showWalkTrough() {
        let walkthroughVC = self.walkthroughVC()
        walkthroughVC.delegate = self
        addChildViewControllerWithView(walkthroughVC)
    }
    
    fileprivate func walkthroughVC() -> WalkthroughViewController {
        let viewControllers = walkthroughs.map { ClassicWalkthroughViewController(model: $0, nibName: "ClassicWalkthroughViewController", bundle: nil) }
        return WalkthroughViewController(nibName: "WalkthroughViewController",
                                         bundle: nil,
                                         viewControllers: viewControllers)
    }
    
    func walkthroughViewControllerDidFinishFlow(_ vc: WalkthroughViewController) {
        UIView.transition(with: self.view, duration: 1, options: .transitionCrossDissolve, animations: {
            vc.view.removeFromSuperview()
        }, completion: nil)
    }
    
    override func tabBar(_ tabBar: UITabBar, didSelect item: UITabBarItem) {
         let index = tabBar.items?.firstIndex(of: item)
            if index == 2 {
            NotificationCenter.default.post(name: NSNotification.Name.rightButtonTapped, object: item)
        }
    }
}

class AppTabBar: UITabBar, CustomTabBarLayerProtocol {
    
    var shapeLayer: CALayer?
    
    var buttonSize: CGFloat { middleButton.frame.width }
    
    private var middleButton = UIButton()
    
    override func sizeThatFits(_ size: CGSize) -> CGSize {
        var sizeThatFits = super.sizeThatFits(size)
        sizeThatFits.height = 94
        return sizeThatFits
    }
    
    override func awakeFromNib() {
        super.awakeFromNib()
        setupMiddleButton()
    }
    
    override func draw(_ rect: CGRect) {
        self.addShape()
    }
    
    override func hitTest(_ point: CGPoint, with event: UIEvent?) -> UIView? {
        if self.isHidden {
            return super.hitTest(point, with: event)
        }
        
        let from = point
        let to = middleButton.center
        
        return sqrt((from.x - to.x) * (from.x - to.x) + (from.y - to.y) * (from.y - to.y)) <= middleButton.frame.height / 2 ? middleButton : super.hitTest(point, with: event)
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        middleButton.center = CGPoint(x: UIScreen.main.bounds.width / 2, y: 0)
    }
    
    @objc private func setupMiddleButton() {
        let image: UIImage? = .scanButton
        middleButton.setImage(image, for: UIControl.State.normal)
        middleButton.contentVerticalAlignment = .fill
        middleButton.contentHorizontalAlignment = .fill
        middleButton.imageEdgeInsets = .zero
        middleButton.frame.size = CGSize(width: 75, height: 75)
        middleButton.layer.masksToBounds = true
        middleButton.layer.cornerRadius = middleButton.frame.size.height / 2
        
        middleButton.center = CGPoint(x: UIScreen.main.bounds.width / 2, y: 0)
        middleButton.addTarget(self, action: #selector(buttonPressed), for: .touchUpInside)
        addSubview(middleButton)
    }
    
    @objc private func buttonPressed() {
        if let tabBarController = self.window!.rootViewController as? UITabBarController {
            NotificationCenter.default.post(name: NSNotification.Name.MiddleButtonTapped, object: middleButton)
            
            if tabBarController.selectedIndex != 0 {
                UIView.animate(withDuration: 0.33) {
                    tabBarController.selectedIndex = 0
                }
            }
        }
    }
}
