//
//  CoreDataManager.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 7/31/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import Foundation
import CoreData

final class CoreDataManager: CoreDataManagerProtocol {
    
    private let persistentContainer: PersistentContainer
    
    init(persistentContainer: PersistentContainer) {
        self.persistentContainer = persistentContainer
    }
    
    private var backgroundContext: NSManagedObjectContext {
        persistentContainer.backgroundContext
    }
    
    private var viewContext: NSManagedObjectContext {
        persistentContainer.viewContext
    }
}

extension CoreDataManager {
    
    func fetchFolders() -> [FolderEntity] {
        getFolders()
    }
    
    func createFolderEntity() -> FolderEntity {
        FolderEntity(context: viewContext)
    }
    
    func deleteFolder(folder: FolderEntity) {
        delete(folder)
    }
}

extension CoreDataManager {
    
    func fetchScanSessions() -> [ScanSessionEntity] {
        getScanSessions()
    }
    
    func createScanSessionEntity() -> ScanSessionEntity {
        ScanSessionEntity(context: viewContext)
    }
    
    func deleteScanSession(session: ScanSessionEntity) {
        delete(session)
    }
}

extension CoreDataManager {
    
    func saveChanges() {
        saveViewContext()
    }
    
    func saveChangesAndWait() {
        saveViewContextAndWait()
    }
}

extension CoreDataManager {
    
    private func getFolders() -> [FolderEntity] {
        let fetchRequest = FolderEntity.fetchRequest()
        
        let folders = viewContext.performAndWait {
            
            do {
                let result = try viewContext.fetch(fetchRequest)
                
                return result
            }
            catch {
                print("Error in fetching folders \(error)")
            }
            
            return [FolderEntity]()
        }
        
        return folders
    }
    
    private func getScanSessions() -> [ScanSessionEntity] {
        let fetchRequest = ScanSessionEntity.fetchRequest()
        
        let sessions = viewContext.performAndWait {
            
            do {
                let result = try viewContext.fetch(fetchRequest)
                
                return result
            }
            catch {
                print("Error in fetching folders \(error)")
            }
            
            return [ScanSessionEntity]()
        }
        
        return sessions
    }
    
    private func delete(_ object: NSManagedObject) {
        viewContext.delete(object)
    }
    
    private func saveViewContext() {
        persistentContainer.save(context: viewContext)
    }
    
    private func saveViewContextAndWait() {
        persistentContainer.saveAndWait(context: viewContext)
    }
    
    private func saveBackgroundContext() {
        persistentContainer.save(context: backgroundContext)
    }
    
    private func saveBackgroundContextAndWait() {
        persistentContainer.saveAndWait(context: backgroundContext)
    }
}
