//
//  ScanSessionSharable.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 8/6/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit
import LNExtensionExecutor

protocol ScanSessionSharable {
    
    var activityIndicator: UIActivityIndicatorView { get set }
    
    var shareType: ShareType { get set }
    
    var sharingViewController: UIViewController { get }
}

extension ScanSessionSharable {
    
    @MainActor
    func shareScanSession(session: ScanSession, shareOption: ShareOption) {
        
        if shareOption != .more {
            directShare(session: session, shareOption: shareOption)
        }
        else {
            shareScanSession(session: session)
        }
    }
    
    private func directShare(session: ScanSession, shareOption: ShareOption) {
        
        activityIndicator.startAnimating()
        
        Task {
            
            let payload = ScanSessionPayloadManager.getPayload(session: session, payload: shareType.payloadType)
            
            DispatchQueue.main.async {
                self.activityIndicator.stopAnimating()
            }
            
            await directShareWith(extensionBundleIdentifier: shareOption.bundleId,
                                  payload: payload)
            
            ScanSessionPayloadManager.deleteTempPDFFileIfNeeded(session: session, shareType: shareType)
        }
    }
    
    @MainActor
    private func shareScanSession(session: ScanSession) {
        
        activityIndicator.startAnimating()
        
        Task {
            
            let payload = ScanSessionPayloadManager.getPayload(session: session, payload: shareType.payloadType)
            
            let shareController = UIActivityViewController(activityItems: payload,
                                                           applicationActivities: nil)
            
            shareController.completionWithItemsHandler = {
                self.handleShareSheetCompletion(type: $0, result: $1, items: $2, error: $3, session: session)
            }
            
            DispatchQueue.main.async {
                
                self.activityIndicator.stopAnimating()
                
                sharingViewController.present(shareController, animated: true, completion: nil)
            }
        }
    }
}

extension ScanSessionSharable {
    
    @MainActor
    private func directShareWith(extensionBundleIdentifier: String,
                                 payload: [Any]) async {
        do {
            let executor = try LNExtensionExecutor(extensionBundleIdentifier: extensionBundleIdentifier)
            let (completed, returnItems) = try await executor.execute(withActivityItems: payload, on: sharingViewController)
            print("completed: \(completed) return items: \(returnItems)")
            
        } catch(let error) {
            print("error: \(error.localizedDescription)")
        }
    }
    
    private func handleShareSheetCompletion(type: UIActivity.ActivityType?, result: Bool, items: [Any]?, error: Error?, session: ScanSession) {
        
        ScanSessionPayloadManager.deleteTempPDFFileIfNeeded(session: session, shareType: shareType)
    }
}
