//
//  UserDefaultsManager.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 7/17/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import Foundation

protocol UserDefaultsManagerProtocol {
    
    func string(key: UserDefaultsKeys) -> String?
    func bool(key: UserDefaultsKeys) -> Bool
    func get<T: Codable>(key: UserDefaultsKeys) -> T?
    
    func set(value: String, key: UserDefaultsKeys)
    func set(value: Bool, key: UserDefaultsKeys)
    func set<T: Codable>(value: T?, key: UserDefaultsKeys)
    
    func clear(key: UserDefaultsKeys)
}

class UserDefaultsManager: UserDefaultsManagerProtocol {
    
    private let userDefaults: UserDefaults = .standard
    
    func string(key: UserDefaultsKeys) -> String? {
        userDefaults.string(forKey: key.rawValue)
    }
    
    func bool(key: UserDefaultsKeys) -> Bool {
        userDefaults.bool(forKey: key.rawValue)
    }
    
    func set(value: String, key: UserDefaultsKeys) {
        set(value: value as Any?, key: key)
    }
    
    func set(value: Bool, key: UserDefaultsKeys) {
        set(value: value as Any?, key: key)
    }
    
    func get<T: Codable>(key: UserDefaultsKeys) -> T? {
        var value: T? = nil
        
        if let data = get(key: key) as? Data {
            value = try? JSONDecoder().decode(T.self, from: data)
        }
        
        return value
    }
    
    func set<T: Codable>(value: T?, key: UserDefaultsKeys) {
        if let value {
            
            let data = try? JSONEncoder().encode(value)
            set(value: data as Any?, key: key)
        }
        else {
            set(value: value as Any?, key: key)
        }
    }
    
    func clear(key: UserDefaultsKeys) {
        userDefaults.removeObject(forKey: key.rawValue)
    }
    
    private func get(key: UserDefaultsKeys) -> Any? {
        userDefaults.object(forKey: key.rawValue)
    }
    
    private func set(value: Any?, key: UserDefaultsKeys) {
        userDefaults.set(value, forKey: key.rawValue)
        userDefaults.synchronize()
    }
}
