//
//  SettingsDataSource.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 7/17/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import Foundation
import LanguageManager_iOS

protocol SettingsDataSourceProtocol {
    func getLanguages() -> [SupportedLanguage]
    func getLanguage() -> SupportedLanguage
    func changeLanguage(to language: SupportedLanguage)
    func changeManagerLanguage(to language: SupportedLanguage)
    
    func getColorScheme() -> SupportedColorScheme
    func changeColorScheme(to colorScheme: SupportedColorScheme)
}

class SettingsDataSource: SettingsDataSourceProtocol {
    
    private let userDefaultsManager: UserDefaultsManagerProtocol
    
    private let languageManager = LanguageManager.shared
    
    private var language: SupportedLanguage {
        get {
            .from(codeName: userDefaultsManager.string(key: .appLanguage) ?? SupportedLanguage.device.codeName)
        }
        set {
            userDefaultsManager.set(value: newValue.codeName, key: .appLanguage)
        }
    }
    
    private var colorScheme: SupportedColorScheme {
        get {
            .from(codeName: userDefaultsManager.string(key: .appColorScheme) ?? SupportedColorScheme.device.codeName)
        }
        set {
            userDefaultsManager.set(value: newValue.codeName, key: .appColorScheme)
        }
    }
    
    init(userDefaultsManager: UserDefaultsManagerProtocol) {
        self.userDefaultsManager = userDefaultsManager
    }
    
    func getLanguages() -> [SupportedLanguage] {
        var availableLanguages = Bundle.main.localizations
        
        availableLanguages.removeAll { $0 == "Base" }
        
        var languages: [SupportedLanguage] = []
        
        languages.append(.device)
        
        for language in availableLanguages {
            languages.append(.from(codeName: language))
        }
        
        return languages
    }
    
    func getLanguage() -> SupportedLanguage {
        language
    }
    
    func changeLanguage(to language: SupportedLanguage) {
        self.language = language
        
        changeManagerLanguage(to: language)
    }
    
    func changeManagerLanguage(to language: SupportedLanguage) {
        languageManager.setLanguage(language: language.language)
        languageManager.defaultLanguage = language.language
    }
    
    func getColorScheme() -> SupportedColorScheme {
        colorScheme
    }
    
    func changeColorScheme(to colorScheme: SupportedColorScheme) {
        self.colorScheme = colorScheme
    }
}
