//
//  SupportedLanguage.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 7/17/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import Foundation
import LanguageManager_iOS
import SwiftUI

enum SupportedLanguage: CaseIterable, CustomMenuPickerItem {
    
    case device
    case english
    case arabic
    
    var flag: String {
        switch self {
        case .device:
            SupportedLanguage.currentLanguage.flag
        case .english:
            "🇺🇸"
        case .arabic:
            "🇰🇼"
        }
    }
    
    static var currentLanguage: SupportedLanguage {
        .from(language: LanguageManager.shared.currentLanguage)
    }
    
    var isRTL: Bool {
        switch self {
        case .device:
            SupportedLanguage.currentLanguage.isRTL
        case .english:
            false
        case .arabic:
            true
        }
    }
    
    var layoutDirection: LayoutDirection {
        isRTL ? .rightToLeft : .leftToRight
    }
    
    var semanticContentAttribute:  UISemanticContentAttribute {
        isRTL ? .forceRightToLeft : .forceLeftToRight
    }
    
    var locale: Locale {
        switch self {
        case .device:
            SupportedLanguage.currentLanguage.locale
        case .english:
            Locale(identifier: "en")
        case .arabic:
            Locale(identifier: "ar")
        }
    }
    
    var id: Self { self }
    
    var displayedName: String {
        switch self {
        case .device:
                .basedOnDevice.localized
        case .english:
            displayNameForLanguage(.english)
        case .arabic:
            displayNameForLanguage(.arabic)
        }
    }
    
    func displayNameForLanguage(_ language: SupportedLanguage) -> String {
        let locale = NSLocale(localeIdentifier: SupportedLanguage.currentLanguage.codeName)
        return locale.displayName(forKey: .identifier, value: language.codeName) ?? ""
    }
}

extension SupportedLanguage {
    
    static func from(codeName: String) -> SupportedLanguage {
        .from(language: Languages(rawValue: codeName) ?? .deviceLanguage)
    }
    
    var codeName: String {
        switch self {
        case .device:
            Languages.deviceLanguage.rawValue
        case .english:
            Languages.en.rawValue
        case .arabic:
            Languages.ar.rawValue
        }
    }
}

extension SupportedLanguage {
    
    static func from(language: Languages) -> Self {
        switch language {
        case .ar:
                .arabic
        case .en:
                .english
        case .deviceLanguage:
                .device
            
        default:
                .device
        }
    }
    
    var language: Languages {
        switch self {
        case .device:
            LanguageManager.shared.deviceLanguage ?? .en
        case .english:
                .en
        case .arabic:
                .ar
        }
    }
}
