//
//  SettingsRepository.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 7/17/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import Foundation

class SettingsRepository: SettingsRepositoryProtocol {
    
    private let dataSource: SettingsDataSourceProtocol
    
    init(dataSource: SettingsDataSourceProtocol) {
        self.dataSource = dataSource
    }
    
    func getLanguages() -> [SupportedLanguage] {
        dataSource.getLanguages()
    }
    
    func getLanguage() -> SupportedLanguage {
        dataSource.getLanguage()
    }
    
    func changeLanguage(to language: SupportedLanguage) {
        
        if shouldChangeLanguage(to: language) {
            
            dataSource.changeLanguage(to: language)
        }
    }
    
    func getColorScheme() -> SupportedColorScheme {
        dataSource.getColorScheme()
    }
    
    func changeColorScheme(to colorScheme: SupportedColorScheme) {
        
        if shouldChangeColorScheme(to: colorScheme) {
            
            dataSource.changeColorScheme(to: colorScheme)
            
            changeAppColorScheme(to: colorScheme)
        }
    }
    
    func handleAppLanguage() {
        handleLanguage()
    }
    
    func handleAppStart() {
        handleColorScheme()
        handleUISettings()
    }
}

extension SettingsRepository {
    
    private func shouldChangeLanguage(to language: SupportedLanguage) -> Bool {
        dataSource.getLanguage() != language
    }
    
    private func shouldChangeColorScheme(to colorScheme: SupportedColorScheme) -> Bool {
        dataSource.getColorScheme() != colorScheme
    }
    
    private func changeAppColorScheme(to colorScheme: SupportedColorScheme) {
        switch colorScheme {
        case .device:
            changeUserInterfaceStyle(to: .unspecified)
        case .light:
            changeUserInterfaceStyle(to: .light)
        case .dark:
            changeUserInterfaceStyle(to: .dark)
        }
    }
    
    private func changeUserInterfaceStyle(to userInterfaceStyle: UIUserInterfaceStyle) {
        
        let window = UIApplication.shared.connectedScenes
            .flatMap { ($0 as? UIWindowScene)?.windows ?? [] }
            .first { $0.isKeyWindow }
        
        window?.overrideUserInterfaceStyle = userInterfaceStyle
    }
}

extension SettingsRepository {
    
    private func handleLanguage() {
        let language = getLanguage()
        
        dataSource.changeManagerLanguage(to: language)
    }
    
    private func handleColorScheme() {
        let colorScheme = getColorScheme()
        changeAppColorScheme(to: colorScheme)
    }
    
    private func handleUISettings() {
        
        handleScrollViewAppearace()
        
        handleNavigationBarAppearace()
        
        handleBarButtonAppearace()
        
        handleToolBarAppearace()
        
        handleButtonAppearace()
        
        handleTextFieldAppearace()
    }
    
    private func handleScrollViewAppearace() {
        
        UIScrollView.appearance().bounces = false
    }
    
    private func handleNavigationBarAppearace() {
        let navigationBarAppearace = UINavigationBar.appearance()
        
        navigationBarAppearace.customTint(.gray600)
        navigationBarAppearace.barTintColor = ColorStyle.gray0.uiColor
    }
    
    private func handleBarButtonAppearace() {
        let tabbarButton = UIBarButtonItem.appearance()
        
        tabbarButton.tintColor = ColorStyle.gray600.uiColor
    }
    
    private func handleToolBarAppearace() {
        
        let toolBarAppearace = UIToolbar.appearance()
        
        toolBarAppearace.customTint(.gray600)
        toolBarAppearace.barTintColor = ColorStyle.gray0.uiColor
    }
    
    private func handleButtonAppearace() {
        let buttonAppearance = UIButton.appearance()
        buttonAppearance.customTint(.gray600)
    }
    
    private func handleTextFieldAppearace() {
        
        let textFieldAppearence = UITextField.appearance()
        
        textFieldAppearence.backgroundColor = .clear
        
        textFieldAppearence.borderStyle = .none
        
        textFieldAppearence.textColor = ColorStyle.gray600.uiColor
        
        textFieldAppearence.borderWidth = 1
        textFieldAppearence.borderColor = ColorStyle.gray100.uiColor
        textFieldAppearence.cornerRadiuss = 10
    }
}
