//
//  ScanSessionsRepository.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 7/31/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import Foundation

class ScanSessionsRepository: ScanSessionsRepositoryProtocol {
    
    let dataSource: ScanSessionsDataSourceProtocol
    
    init(dataSource: ScanSessionsDataSourceProtocol) {
        self.dataSource = dataSource
    }
    
    private lazy var foldersEntities: [FolderEntity] = {
        dataSource.getFolders()
    }()
    
    private lazy var sessionsEntities: [ScanSessionEntity] = {
        dataSource.getScanSessions()
    }()
}

extension ScanSessionsRepository {
    
    func getFolders() -> [Folder] {
        foldersEntities.toDomain()
    }
    
    func addFolder(params: Folder) {
        
        let folderEntity = params.toEntity(entity: createFolderEntity())
        
        persistChanges()
        
        refreshFoldersEntities()
    }
    
    func updateFolder(params: Folder) {
        
        if let folderEntity = foldersEntities.first(where: { $0.id == params.id }) {
            
            folderEntity.copyValues(from: params)
            
            persistChanges()
        }
    }
    
    func deleteFolder(params: Folder) {
        
        handleRemovingFolderFiles(folder: params)
        
        if let folderEntity = foldersEntities.first(where: { $0.id == params.id }) {
            
            deleteFolderEntity(folderEntity: folderEntity)
        }
    }
}

extension ScanSessionsRepository {
    
    func getScanSessions() -> [ScanSession] {
        sessionsEntities.toDomain()
    }
    
    func addScanSession(params: ScanSession, to folder: Folder) {
        
        handleMovingScanSessionFiles(scanSession: params)
        
        let sessionEntity = params.toEntity(entity: createScanSessionEntity())
        
        if let folderEntity = foldersEntities.first(where: { $0.id == folder.id }) {
            
            addScanSession(sessionEntity: sessionEntity, to: folderEntity)
        }
    }
    
    func updateScanSession(params: ScanSession) {
        
        handleMovingScanSessionFiles(scanSession: params)
        
        if let sessionEntity = sessionsEntities.first(where: { $0.id == params.id }) {
            
            handleRemovingRedundantScanSessionFiles(scanSession: params, scanSessionEntitiy: sessionEntity)
            
            sessionEntity.copyValues(from: params)
            
            persistChanges()
            
            refreshFoldersEntities()
        }
    }
    
    func moveScanSession(params: ScanSession, to folder: Folder) {
        
        let sessionEntity = sessionsEntities.first(where: { $0.id == params.id })
        
        let folderEntity = foldersEntities.first(where: { $0.id == folder.id })
        
        if let folderEntity, let sessionEntity, folderEntity != sessionEntity.folder {
            
            sessionEntity.folder = folderEntity
            
            persistChanges()
        }
    }
    
    func deleteScanSession(params: ScanSession) {
        
        handleRemovingScanSessionFiles(scanSession: params)
        
        if let sessionEntity = sessionsEntities.first(where: { $0.id == params.id }) {
            
            deleteScanSessionEntity(sessionEntity: sessionEntity)
        }
    }
}

extension ScanSessionsRepository {
    
    private func refreshFoldersEntities() {
        foldersEntities = dataSource.getFolders()
    }
    
    private func createFolderEntity() -> FolderEntity {
        dataSource.createFolderEntity()
    }
    
    private func deleteFolderEntity(folderEntity: FolderEntity) {
        
        foldersEntities.remove(element: folderEntity)
        
        sessionsEntities.removeAll { $0.folder == folderEntity }
        
        dataSource.deleteFolder(folder: folderEntity)
        
        persistChanges()
    }
    
    private func refreshSessionsEntities() {
        sessionsEntities = dataSource.getScanSessions()
    }
    
    private func createScanSessionEntity() -> ScanSessionEntity {
        dataSource.createScanSessionEntity()
    }
    
    private func addScanSession(sessionEntity: ScanSessionEntity, to folderEntity: FolderEntity) {
        
        dataSource.addScanSession(session: sessionEntity, to: folderEntity)
        
        persistChanges()
        
        refreshSessionsEntities()
        
        refreshFoldersEntities()
    }
    
    private func deleteScanSessionEntity(sessionEntity: ScanSessionEntity) {
        
        if let index = foldersEntities.firstIndex(of: sessionEntity.folder) {
            foldersEntities[index].removeFromSessions(sessionEntity)
        }
        
        sessionsEntities.remove(element: sessionEntity)
        
        dataSource.deleteScanSession(session: sessionEntity)
        
        persistChanges()
    }
    
    private func handleMovingScanSessionFiles(scanSession: ScanSession) {
        
        scanSession.scannedItems = scanSession.scannedItems.map { item in
            
            if item.type == .temp,
               let originalImage = item.newOriginalImage?.retrieveImage(),
               let renderedImage = item.newRenderedImage?.retrieveImage() {
                
                let compressedOriginalImage = ImageCompressManager.compress(image: originalImage)
                let compressedRenderedImage = ImageCompressManager.compress(image: renderedImage)
                
                let newScannedItem = item.createItem(originalImage: compressedOriginalImage,
                                                     renderedImage: compressedRenderedImage,
                                                     type: .local)
                
                if let imageUrl = item.newOriginalImage {
                    try? FileManager.default.removeItem(at: imageUrl)
                }
                
                if let imageUrl = item.newRenderedImage {
                    try? FileManager.default.removeItem(at: imageUrl)
                }
                
                return newScannedItem
            }
            
            return item
        }
    }
    
    private func handleRemovingRedundantScanSessionFiles(scanSession: ScanSession, scanSessionEntitiy: ScanSessionEntity) {
        var imageUrlsToDelete: [URL] = []
        
        let oldScanSession = scanSessionEntitiy.toDomain()
        
        oldScanSession.scannedItems.forEach { item in
            
            if let renderedImageUrl = item.newRenderedImage,
               let originalImageUrl = item.newOriginalImage {
                
                if !scanSession.scannedItems.contains(where: { $0.newRenderedImage == renderedImageUrl && $0.newOriginalImage == originalImageUrl }) {
                    
                    imageUrlsToDelete.append(renderedImageUrl)
                    imageUrlsToDelete.append(originalImageUrl)
                }
            }
        }
        
        imageUrlsToDelete.forEach { try? FileManager.default.removeItem(at: $0) }
    }
    
    private func handleRemovingFolderFiles(folder: Folder) {
        folder.scanSessions.forEach { handleRemovingScanSessionFiles(scanSession: $0) }
    }
    
    private func handleRemovingScanSessionFiles(scanSession: ScanSession) {
        
        scanSession.scannedItems.forEach { item in
            
            if let imageUrl = item.newOriginalImage {
                try? FileManager.default.removeItem(at: imageUrl)
            }
            
            if let imageUrl = item.newRenderedImage {
                try? FileManager.default.removeItem(at: imageUrl)
            }
        }
    }
    
    private func persistChanges() {
        saveChangesAndWait()
    }
    
    private func saveChanges() {
        dataSource.saveChanges()
    }
    
    private func saveChangesAndWait() {
        dataSource.saveChangesAndWait()
    }
}
