//
//  ImageCompressView.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 8/8/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import SwiftUI
import mozjpeg

struct ImageCompressView: View {
    
    @Binding var isPresented: Bool
    var image: UIImage
    
    var columns = [GridItem(.flexible()), GridItem(.flexible()), GridItem(.flexible())]
    
    @State private var images: [[UIImage]] = Array(repeating: Array(repeating: UIImage(), count: 4), count: 4)
    
    @State var selectedImage: UIImage?
    @State var selectedImageTitle: String = ""
    @State var isZoomableImagePresented: Bool = false
    
    var body: some View {
        VStack(alignment: .leading, spacing: 8) {
            
            close
                .padding(.leading, 8)
            
            ScrollView(showsIndicators: false) {
                
                VStack(spacing: 16) {
                    
                    original
                    
                    imagesGrid("") {
                        
                        uiImage1
                        
                        timeline1
                        
                        mozjpeg1
                        
                        uiImage2
                        
                        timeline2
                        
                        mozjpeg2
                        
                        uiImage3
                        
                        timeline3
                        
                        mozjpeg3
                    }
                    
                    imagesGrid("Dec") {
                        
                        decMozjpeg1
                        
                        decMozjpeg2
                        
                        decMozjpeg3
                    }
                }
                .padding(.all, 8)
            }
        }
        .task {
            await prepareImages()
        }
        .fullScreenCover(isPresented: $isZoomableImagePresented) {
            
            if let selectedImage {
                ZoomableImageView(isPresented: $isZoomableImagePresented,
                                  title: selectedImageTitle,
                                  size: size(image: selectedImage),
                                  image: selectedImage)
            }
        }
    }
    
    private var close: some View {
        Button(action: { isPresented = false }, label: {
            Image.xMark
                .resizable()
                .aspectRatio(contentMode: .fit)
                .frame(width: 16, height: 16)
                .customForeground(.gray600)
        })
    }
    
    private func prepareImages() async {
        
        images[0][1] = compressImages(image: image, compressionQuality: 1)
        images[1][1] = compressImages(image: image, compressionQuality: 0.5)
        images[2][1] = compressImages(image: image, compressionQuality: 0)
        
        images[0][2] = image.wxCompress()
        images[1][2] = image.wxCompress().wxCompress()
        images[2][2] = image.wxCompress().wxCompress().wxCompress()
        
        images[0][3] = compressImagesWithMozJpeg(image: image, quality: 1)
        images[1][3] = compressImagesWithMozJpeg(image: image, quality: 0.5)
        images[2][3] = compressImagesWithMozJpeg(image: image, quality: 0)
        
        images[3][0] = decompressImagesWithMozJpeg(image: image, quality: 1)
        images[3][1] = decompressImagesWithMozJpeg(image: image, quality: 0.5)
        images[3][2] = decompressImagesWithMozJpeg(image: image, quality: 0)
    }
    
    private var original: some View {
        imageView(image: image,
                  title: "Original")
    }
    
    private var uiImage1: some View {
        imageView(image: images[0][1],
                  title: "UI Q 1")
    }
    
    private var uiImage2: some View {
        imageView(image: images[1][1],
                  title: "UI Q 0.5")
    }
    
    private var uiImage3: some View {
        imageView(image: images[2][1],
                  title: "UI Q 0")
    }
    
    private var timeline1: some View {
        imageView(image: images[0][2],
                  title: "Timeline x1")
    }
    
    private var timeline2: some View {
        imageView(image: images[1][2],
                  title: "Timeline x2")
    }
    
    private var timeline3: some View {
        imageView(image: images[2][2],
                  title: "Timeline x3")
    }
    
    private var mozjpeg1: some View {
        imageView(image: images[0][3],
                  title: "Moz Q 1")
    }
    
    private var mozjpeg2: some View {
        imageView(image: images[1][3],
                  title: "Moz Q 0.5")
    }
    
    private var mozjpeg3: some View {
        imageView(image: images[2][3],
                  title: "Moz Q 0")
    }
    
    private var decMozjpeg1: some View {
        imageView(image: images[3][0],
                  title: "DeMoz Q 1")
    }
    
    private var decMozjpeg2: some View {
        imageView(image: images[3][1],
                  title: "DeMoz Q 0.5")
    }
    
    private var decMozjpeg3: some View {
        imageView(image: images[3][2],
                  title: "DeMoz Q 0")
    }
}

extension ImageCompressView {
    
    private func imagesGrid(_ title: String, @ViewBuilder content: () -> some View) -> some View {
        VStack(spacing: 2) {
            
            Text(title)
            
            LazyVGrid(columns: columns) {
                content()
            }
            .frame(maxWidth: .infinity, maxHeight: .infinity)
        }
    }
    
    private func imageView(image: UIImage, title: String) -> some View {
        VStack {
            
            Text(title)
            
            Text(size(image: image))
            
            Image(uiImage: image)
                .resizable()
                .aspectRatio(contentMode: .fit)
        }
        .onTapGesture {
            selectedImageTitle = title
            selectedImage = image
            isZoomableImagePresented = true
        }
    }
    
    private func size(image: UIImage) -> String {
        
        var size = ""
        
        if let sizeInByte = image.jpegData(compressionQuality: 1)?.count {
            
            let sizeInMByte = Double(sizeInByte) / 1024.0 / 1024.0
            
            size = String(format: "%.4f MB", sizeInMByte)
        }
        
        return size
    }
}

extension ImageCompressView {
    
    private func compressImages(image: UIImage, compressionQuality: CGFloat) -> UIImage {
        
        if let data = image.jpegData(compressionQuality: compressionQuality),
           let newImage = UIImage(data: data) {
            return newImage
        }
        
        return image
    }
    
    private func compressImagesWithMozJpeg(image: UIImage, quality: Float) -> UIImage {
        
        if let data = try? image.mozjpegRepresentation(quality: quality),
           let newImage = UIImage(data: data) {
            return newImage
        }
        
        return image
    }
    
    private func decompressImagesWithMozJpeg(image: UIImage, quality: Float) -> UIImage {
        
        if let data = try? image.mozjpegRepresentation(quality: quality),
           let newImage = Mozjpeg().decompress(chunk: data) {
            return newImage
        }
        
        return image
    }
}

#Preview {
    ImageCompressView(isPresented: .constant(true), image: .actions)
}
