//
//  SettingsView.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 7/18/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import SwiftUI

struct SettingsView: View {
    
    @StateObject var viewModel: SettingsViewModel
    
    init(coordinator: SettingsViewCoordinating) {
        self._viewModel = StateObject(wrappedValue: SettingsViewModel(coordinator: coordinator))
    }
    
    var body: some View {
        MainView(viewModel: viewModel) {
            VStack {
                
                NoIndicatorsScrollView {
                    
                    settings
                        .padding(.horizontal, 20)
                }
                .padding(.top, 32)
            }
            .navigationTitle(viewModel.navigationTitle)
            .customNavigationBackButton(defaultBackButtonHidden: viewModel.isDefaultBackButtonHidden,
                                        backButton: backButton)
            .environment(\.layoutDirection, viewModel.layoutDirection)
            .customSheet(isPresented: $viewModel.isLanguageSettingSheetPresented,
                         height: viewModel.sheetSize.height,
                         detents: [.medium],
                         content: {
                languageSheet
                    .readSize(size: $viewModel.sheetSize)
            })
            .customSheet(isPresented: $viewModel.isColorSchemeSettingSheetPresented,
                         height: viewModel.sheetSize.height,
                         detents: [.medium],
                         content : {
                colorSchemeSheet
                    .readSize(size: $viewModel.sheetSize)
            })
            .customSheet(isPresented: $viewModel.isFileTypeSettingSheetPresented,
                         height: viewModel.sheetSize.height,
                         detents: [.medium],
                         content : {
                fileTypeSheet
                    .readSize(size: $viewModel.sheetSize)
            })
            .customSheet(isPresented: $viewModel.isScreenSettingSheetPresented,
                         height: viewModel.sheetSize.height,
                         detents: [.medium],
                         content : {
                screenSheet
                    .readSize(size: $viewModel.sheetSize)
            })
            .fullScreenCover(isPresented: $viewModel.isImageCompresserPresented) {
                ImageCompressView(isPresented: $viewModel.isImageCompresserPresented,
                                  image: viewModel.image)
            }
        }
    }
}

extension SettingsView {
    
    private var backButton: some View {
        Button(action: {
            viewModel.onBackButtonClick()
        }, label: {
            HStack(spacing: 4) {
                
                Image.chevronBackward
                
                Text(viewModel.backButtonTitle)
            }
        })
    }
    
    private var settings: some View {
        VStack(spacing: 24) {
            
            language
            
            colorScheme
            
            fileType
            
            screenType
            
            credits
            
            compress
                .padding(.horizontal, 32)
        }
    }
    
    private var language: some View {
        SettingButton(icon: .languages,
                      title: .appLanguage.localized,
                      value: viewModel.selectedLanguage.displayedName,
                      action: viewModel.onLanguageClick)
    }
    
    private var colorScheme: some View {
        SettingButton(icon: .brush,
                      title: .appAppearence.localized,
                      value: viewModel.selectedColorScheme.displayedName,
                      action: viewModel.onColorSchemeClick)
    }
    
    private var fileType: some View {
        SettingButton(icon: .fileDown,
                      title: .defaultFileTypeForInstalling.localized,
                      value: viewModel.selectedDefaultFileType.displayedName,
                      action: viewModel.onFileTypeClick)
    }
    
    private var screenType: some View {
        SettingButton(icon: .smartphone,
                      title: .defaultScreenWhenAppStarts.localized,
                      value: viewModel.selectedDefaultScreen.displayedName,
                      action: viewModel.onScreenClick)
    }
    
    private var languageSheet: some View {
        SettingSheetView(title: .appLanguage.localized,
                         selectedItem: $viewModel.selectedLanguageForSheet,
                         items: viewModel.languages,
                         saveAction: viewModel.onLanguageSaveClick)
    }
    
    private var colorSchemeSheet: some View {
        SettingSheetView(title: .appAppearence.localized,
                         selectedItem: $viewModel.selectedColorSchemeForSheet,
                         items: viewModel.colorSchemes,
                         saveAction: viewModel.onColorSchemeSaveClick)
    }
    
    private var fileTypeSheet: some View {
        SettingSheetView(title: .defaultFileTypeForInstalling.localized,
                         selectedItem: $viewModel.selectedDefaultFileTypeForSheet,
                         items: viewModel.fileTypes,
                         saveAction: viewModel.onFileTypeSaveClick)
    }
    
    private var screenSheet: some View {
        SettingSheetView(title: .defaultScreenWhenAppStarts.localized,
                         selectedItem: $viewModel.selectedDefaultScreenForSheet,
                         items: viewModel.screens,
                         saveAction: viewModel.onScreenSaveClick)
    }
    
    private var credits: some View {
        Button(action: viewModel.onCreditsClick,
               label: {
            
            HStack(alignment: .top, spacing: 20) {
                
                Image.info
                    .resizable()
                    .aspectRatio(contentMode: .fit)
                    .frame(width: 20, height: 20)
                    .customForeground(.gray600)
                
                VStack(alignment: .leading, spacing: 6) {
                    
                    Text(String.credits.localized)
                        .customForeground(.gray600)
                        .customFont(.bold(16))
                }
                
                Spacer()
            }
        })
    }
    
    private var compress: some View {
        PrimaryButton(title: "Compress Image Tester",
                      action: viewModel.onCompressClick)
    }
}

#Preview {
    SettingsView(coordinator: SettingsViewCoordinator(navigationController: UINavigationController()))
}

fileprivate extension View {
    
    func customNavigationBackButton<T: View>(defaultBackButtonHidden: Bool,
                                             backButton: T) -> some View {
        ModifiedContent(content: self,
                        modifier: CustomBackButtonViewModifier(defaultBackButtonHidden: defaultBackButtonHidden, backButton: backButton))
    }
}

fileprivate struct CustomBackButtonViewModifier<T: View>: ViewModifier {
    
    var defaultBackButtonHidden: Bool
    var backButton: T
    
    func body(content: Content) -> some View {
        
        if defaultBackButtonHidden {
            
            content
                .navigationBarBackButtonHidden(defaultBackButtonHidden)
                .navigationBarItems(leading: backButton)
        }
        else {
            content
        }
    }
}
