//
//  SettingsViewModel.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 7/18/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import SwiftUI
import PhotosUI

final class SettingsViewModel: MainViewModel {
    
    @Published var layoutDirection: LayoutDirection = .leftToRight
    
    @Published var refreshView: Bool = false
    
    @Published var languages: [SupportedLanguage] = []
    @Published var colorSchemes: [SupportedColorScheme] = []
    @Published var fileTypes: [DefaultFileType] = []
    @Published var screens: [DefaultScreen] = []
    
    @Published var selectedLanguage: SupportedLanguage = .device {
        didSet {
            changeLanguage(to: selectedLanguage)
        }
    }
    
    @Published var selectedColorScheme: SupportedColorScheme = .device {
        didSet {
            changeColorScheme(to: selectedColorScheme)
        }
    }
    
    @Published var selectedLanguageForSheet: SupportedLanguage = .device
    @Published var selectedColorSchemeForSheet: SupportedColorScheme = .device
    @Published var selectedDefaultFileTypeForSheet: DefaultFileType = .pdf
    @Published var selectedDefaultScreenForSheet: DefaultScreen = .files
    
    @Published var sheetSize: CGSize = .zero
    
    @Published var isDefaultBackButtonHidden: Bool = false
    
    @Published var isLanguageSettingSheetPresented: Bool = false
    @Published var isColorSchemeSettingSheetPresented: Bool = false
    @Published var isFileTypeSettingSheetPresented: Bool = false
    @Published var isScreenSettingSheetPresented: Bool = false
    
    @Published var image: UIImage = UIImage()
    @Published var isImageCompresserPresented: Bool = false
    
    var selectedDefaultFileType: DefaultFileType {
        get {
            userSettings.defaultFileType
        }
        set {
            userSettings.defaultFileType = newValue
        }
    }
    
    var selectedDefaultScreen: DefaultScreen {
        get {
            userSettings.defaultScreen
        }
        set {
            userSettings.defaultScreen = newValue
        }
    }
    
    var navigationTitle: String { .settings.localized }
    var backButtonTitle: String { .fileManager.localized }
    
    @Inject var getLanguagesUseCase: GetLanguagesUseCase
    @Inject var getLanguageUseCase: GetLanguageUseCase
    @Inject var changeLanguageUseCase: ChangeLanguageUseCase
    
    @Inject var getColorSchemeUseCase: GetColorSchemeUseCase
    @Inject var changeColorSchemeUseCase: ChangeColorSchemeUseCase
    
    @Inject var userSettings: UserSettings
    
    private let coordinator: SettingsViewCoordinating
    
    init(coordinator: SettingsViewCoordinating) {
        self.coordinator = coordinator
    }
}

extension SettingsViewModel {
    
    func onAppear() {
        fetchSettings()
    }
    
    func onLanguageClick() {
        
        selectedLanguageForSheet = selectedLanguage
        
        displayLanguageSheet()
    }
    
    func onLanguageSaveClick() {
        
        selectedLanguage = selectedLanguageForSheet
        
        hideLanguageSheet()
    }
    
    func onColorSchemeClick() {
        
        selectedColorSchemeForSheet = selectedColorScheme
        
        displayColorSchemeSheet()
    }
    
    func onColorSchemeSaveClick() {
        
        selectedColorScheme = selectedColorSchemeForSheet
        
        hideColorSchemeSheet()
    }
    
    func onFileTypeClick() {
        
        selectedDefaultFileTypeForSheet = selectedDefaultFileType
        
        displayFileTypeSheet()
    }
    
    func onFileTypeSaveClick() {
        
        selectedDefaultFileType = selectedDefaultFileTypeForSheet
        
        hideFileTypeSheet()
    }
    
    func onScreenClick() {
        
        selectedDefaultScreenForSheet = selectedDefaultScreen
        
        displayScreenSheet()
    }
    
    func onScreenSaveClick() {
        
        selectedDefaultScreen = selectedDefaultScreenForSheet
        
        hideScreenSheet()
    }
    
    func onCreditsClick() {
        coordinator.coordinateToCredits()
    }
    
    func onCompressClick() {
        coordinator.displayImagePicker(delegate: self)
    }
}

extension SettingsViewModel {
    
    private func displayLanguageSheet() {
        isLanguageSettingSheetPresented = true
    }
    
    private func hideLanguageSheet() {
        isLanguageSettingSheetPresented = false
    }
    
    private func displayColorSchemeSheet() {
        isColorSchemeSettingSheetPresented = true
    }
    
    private func hideColorSchemeSheet() {
        isColorSchemeSettingSheetPresented = false
    }
    
    private func displayFileTypeSheet() {
        isFileTypeSettingSheetPresented = true
    }
    
    private func hideFileTypeSheet() {
        isFileTypeSettingSheetPresented = false
    }
    
    private func displayScreenSheet() {
        isScreenSettingSheetPresented = true
    }
    
    private func hideScreenSheet() {
        isScreenSettingSheetPresented = false
    }
    
    private func displayImageCompresser() {
        isImageCompresserPresented = true
    }
    
    private func hideImageCompresser() {
        isImageCompresserPresented = false
    }
}

extension SettingsViewModel {
    
    private func fetchSettings() {
        languages = getLanguagesUseCase.execute()
        
        colorSchemes = SupportedColorScheme.allCases
        fileTypes = DefaultFileType.allCases
        screens = DefaultScreen.allCases
        
        selectedLanguage = getLanguageUseCase.execute()
        selectedColorScheme = getColorSchemeUseCase.execute()
    }
}

extension SettingsViewModel {
    
    private func changeLanguage(to language: SupportedLanguage) {
        
        let previousLanguage = SupportedLanguage.currentLanguage
        
        changeLanguageUseCase.execute(to: language)
        
        layoutDirection = language.layoutDirection
        
        if language.layoutDirection == .leftToRight {
            refreshView.toggle()
        }
        
        if previousLanguage != SupportedLanguage.currentLanguage {
            
            refreshSemanticContentAttribute()
        }
    }
    
    private func changeColorScheme(to colorScheme: SupportedColorScheme) {
        changeColorSchemeUseCase.execute(to: colorScheme)
    }
}

extension SettingsViewModel {
    
    private func refreshSemanticContentAttribute() {
        let navigationController = coordinator.navigationController

        let semanticContentAttribute = SupportedLanguage.currentLanguage.semanticContentAttribute
        
        navigationController.tabBarController?.tabBar.semanticContentAttribute = semanticContentAttribute
        navigationController.navigationBar.semanticContentAttribute = semanticContentAttribute
        
        isDefaultBackButtonHidden = true
    }
    
    func onBackButtonClick() {
        coordinator.coordinateBack()
    }
}

extension SettingsViewModel: PHPickerViewControllerDelegate {
    
    func picker(_ picker: PHPickerViewController, didFinishPicking results: [PHPickerResult]) {

        picker.dismiss(animated: true) { [weak self] in
            guard let self else { return }
            
            if results.count > 0 {
                handlePHPickerResults(results: results)
            }
        }
    }
    
    private func handlePHPickerResults(results: [PHPickerResult]) {
        
        var images: [UIImage] = []
        
        let dispatchGroup = DispatchGroup()
        
        for result in results {
            
            dispatchGroup.enter()  // Enter the group before each task
            
            result.itemProvider.loadObject(ofClass: UIImage.self) { (object, error) in
                
                defer { dispatchGroup.leave() } // Leave the group after each task, regardless of whether it was successful
                
                if let error = error {
                    print("Error loading image: \(error)")
                } else if let image = object as? UIImage {
                    images.append(image)
                }
            }
        }
        
        dispatchGroup.notify(queue: .main) { [weak self] in
            guard let self else { return }
            
            handlePHPickerImages(images: images)
        }
    }
    
    private func handlePHPickerImages(images: [UIImage]) {
        
        self.image = images[0]
        
        displayImageCompresser()
    }
}
