import UIKit
import PDFKit

class PDFManager {
    
    class func pdfThumbnail(url: URL, width: CGFloat = 240) -> UIImage? {
        guard let data = try? Data(contentsOf: url),
              let page = PDFDocument(data: data)?.page(at: 0) else {
            return nil
        }
        
        let pageSize = page.bounds(for: .mediaBox)
        let pdfScale = width / pageSize.width
        
        let scale = UIScreen.main.scale * pdfScale
        let screenSize = CGSize(width: pageSize.width * scale,
                                height: pageSize.height * scale)
        
        return page.thumbnail(of: screenSize, for: .mediaBox)
    }
    
    class func generatePdfThumbnail(of thumbnailSize: CGSize, for documentUrl: URL) -> UIImage? {
        guard let data = try? Data(contentsOf: documentUrl),
              let page = PDFDocument(data: data)?.page(at: 0) else {
            return nil
        }
        
        return page.thumbnail(of: thumbnailSize, for: .trimBox)
    }
    
    class func getPDFPageToImage(page: PDFPage, pageIndex: Int) -> UIImage? {
        let pageSize = page.bounds(for: .mediaBox)
        let screenSize = CGSize(width: pageSize.width,
                                height: pageSize.height)
        
        return page.thumbnail(of: screenSize, for: .mediaBox)
    }
    
    class func createPDFDocument(from image: UIImage, localFileManager: LocalFileManager?, folder: AppConfigurator.Folder) {
        guard let allDoc = localFileManager?.AllDocUrl else { return }
        guard let scannerURL = localFileManager?.getFolderUrl(folder: folder) else { return }
        
        let urlPDFtoSaveInAllDoc = allDoc.appendingPathComponent(String.getDocumentName())
        let urlPDFtoSave = scannerURL.appendingPathComponent(String.getDocumentName())
        
        let document = PDFDocument()
        
        if let pdfPage = PDFPage(image: image) {
            document.insert(pdfPage, at: 0)
        }
        do {
            try document.dataRepresentation()?.write(to: urlPDFtoSave)
            if folder.savedName != "all_documents" {
                try document.dataRepresentation()?.write(to: urlPDFtoSaveInAllDoc)
            }
        } catch {
            print(error.localizedDescription)
        }
    }
    
    class func createMultiImagesPDFDocument(from images: [UIImage], localFileManager: LocalFileManager?, folder: AppConfigurator.Folder) {
        guard let allDoc = localFileManager?.AllDocUrl else { return }
        guard let scannerURL = localFileManager?.getFolderUrl(folder: folder) else { return }
        
        let urlPDFtoSaveInAllDoc = allDoc.appendingPathComponent(String.getDocumentName())
        let urlPDFtoSave = scannerURL.appendingPathComponent(String.getDocumentName())
        
        let document = PDFDocument()
        for (index, image) in images.enumerated() {
            if let pdfPage = PDFPage(image: image) {
                document.insert(pdfPage, at: index)
            }
        }
        do {
            try document.dataRepresentation()?.write(to: urlPDFtoSave)
            if folder.savedName != "all_documents" {
                try document.dataRepresentation()?.write(to: urlPDFtoSaveInAllDoc)
            }
        } catch {
            print(error.localizedDescription)
        }
    }
    
    class func createPDFDocument(from image: UIImage, urlPDFtoSave: URL) {
        let document = PDFDocument()
        
        if let pdfPage = PDFPage(image: image) {
            document.insert(pdfPage, at: 0)
        }
        do {
            try document.dataRepresentation()?.write(to: urlPDFtoSave)
        } catch {
            print(error.localizedDescription)
        }
    }
    
    class func createMultiPDFPage(from imageURLs: [URL], localFileManager: LocalFileManager?, folder: AppConfigurator.Folder, name: String = "", session: MultiPageScanSession = MultiPageScanSession(), _ completion: @escaping () -> ()) {
        guard let allDoc = localFileManager?.AllDocUrl else { return }
        guard let scannerURL = localFileManager?.getFolderUrl(folder: folder) else { return }
        let displayName = name == "" ? String.getDocumentName() : name
        let urlPDFtoSave = scannerURL.appendingPathComponent(displayName)
        let urlPDFtoSaveInAllDoc = allDoc.appendingPathComponent(displayName)
        
        if imageURLs.isEmpty { return }
        let document = PDFDocument()
        
        let dispatchGroup = DispatchGroup()
        
        var pdfPages = [PDFPage?](repeating: nil, count: imageURLs.count)
        
        for (index, imageURL) in imageURLs.enumerated() {
            dispatchGroup.enter()
            DispatchQueue.global().async {
                if let data = try? Data(contentsOf: imageURL),
                   let image = UIImage(data: data),
                   let pdfPage = PDFPage(image: image) {
                    DispatchQueue.main.async {
                        pdfPages[index] = pdfPage
                        dispatchGroup.leave()
                    }
                } else {
                    dispatchGroup.leave()
                }
            }
        }
        
        dispatchGroup.notify(queue: .main) {
            for pdfPage in pdfPages {
                if let pdfPage = pdfPage {
                    document.insert(pdfPage, at: document.pageCount)
                }
            }
            
            do {
                try document.dataRepresentation()?.write(to: urlPDFtoSave)
                if folder.savedName != "all_documents" {
                    try document.dataRepresentation()?.write(to: urlPDFtoSaveInAllDoc)
                }
                completion()
            } catch {
                print(error.localizedDescription) // handle error
            }
            
            let pdfDocumentObject = PDFDocumentObject(name: displayName.replacingOccurrences(of: ".pdf", with: ""), session: session)
            let defaults = UserDefaults.standard
            let encoder = JSONEncoder()
            let decoder = JSONDecoder()
            
            // Fetch the existing array of PDFDocumentObjects from UserDefaults
            var existingPDFDocumentObjects: [PDFDocumentObject] = []
            if let savedData = defaults.object(forKey: "pdfDocumentObjects") as? Data {
                if let loadedPDFDocumentObjects = try? decoder.decode([PDFDocumentObject].self, from: savedData) {
                    existingPDFDocumentObjects = loadedPDFDocumentObjects
                }
            }
            
            // Add the new PDFDocumentObject to the existing array
            existingPDFDocumentObjects.append(pdfDocumentObject)
            
            // Save the updated array back to UserDefaults
            if let encodedData = try? encoder.encode(existingPDFDocumentObjects) {
                defaults.set(encodedData, forKey: "pdfDocumentObjects")
            }
            
            print("All images have been processed")
        }
    }
    
    class func createMultiPDFPageAndReturn(from imageURLs: [URL], localFileManager: LocalFileManager?, folder: AppConfigurator.Folder, name: String = "", _ completion: @escaping (URL) -> ()) {
        guard let scannerURL = localFileManager?.getFolderUrl(folder: folder) else { return }
        
        let urlPDFtoSave = scannerURL.appendingPathComponent(name == "" ? String.getDocumentName() : "\(name).pdf")
        
        if imageURLs.isEmpty { return }
        let document = PDFDocument()
        
        let dispatchGroup = DispatchGroup()
        
        // Create an array of optional PDFPage with a size equal to the number of images
        var pdfPages = [PDFPage?](repeating: nil, count: imageURLs.count)
        
        // Iterate through image URLs
        for (index, imageURL) in imageURLs.enumerated() {
            dispatchGroup.enter()
            DispatchQueue.global().async {
                if let data = try? Data(contentsOf: imageURL),
                   let image = UIImage(data: data),
                   let pdfPage = PDFPage(image: image) {
                    DispatchQueue.main.async {
                        // Insert the resulting PDFPage into pdfPages at the corresponding index
                        pdfPages[index] = pdfPage
                        dispatchGroup.leave()
                    }
                } else {
                    dispatchGroup.leave()
                }
            }
        }
        dispatchGroup.notify(queue: .main) {
            // Once all images are processed, insert the pages from pdfPages into the PDFDocument in order
            for pdfPage in pdfPages {
                if let pdfPage = pdfPage {
                    document.insert(pdfPage, at: document.pageCount)
                }
            }
            
            do {
                try document.dataRepresentation()?.write(to: urlPDFtoSave)
                completion(urlPDFtoSave)
            } catch {
                print(error.localizedDescription) // handle error
            }
            
            print("All images have been processed")
        }
    }
    
    class func appendPage(image: UIImage, pdfView: PDFView, documentURL: URL?, name: String = "") {
        guard let page = PDFPage(image: image),
              let documentURL = documentURL,
              let lastIndex = pdfView.document?.pageCount else  { return }
        pdfView.document?.insert(page, at: lastIndex)
        pdfView.document?.write(to: documentURL)
        pdfView.go(to: page)
        LocalFileManager().updateThumbnail(forURL: documentURL)
        
        let defaults = UserDefaults.standard
        let encoder = JSONEncoder()
        let decoder = JSONDecoder()
        
        // Fetch the existing array of PDFDocumentObjects from UserDefaults
        var existingPDFDocumentObjects: [PDFDocumentObject] = []
        if let savedData = defaults.object(forKey: "pdfDocumentObjects") as? Data {
            if let loadedPDFDocumentObjects = try? decoder.decode([PDFDocumentObject].self, from: savedData) {
                existingPDFDocumentObjects = loadedPDFDocumentObjects
            }
        }
        
        // Add the new PDFDocumentObject to the existing array
        for item in existingPDFDocumentObjects {
            if item.name.replacingOccurrences(of: ".pdf", with: "") == name {
                let newItem = ScannedItem(originalImage: image, renderImage: image)
                item.session.add(item: newItem)
            }
        }
        
        // Save the updated array back to UserDefaults
        if let encodedData = try? encoder.encode(existingPDFDocumentObjects) {
            defaults.set(encodedData, forKey: "pdfDocumentObjects")
        }
    }
    
    class func appendPage(imageURL: URL, pdfView: PDFView, documentURL: URL?) {
        DispatchQueue.global().async {
            if let data = try? Data(contentsOf: imageURL),
               let image = UIImage(data: data),
               let page = PDFPage(image: image),
               let documentURL = documentURL,
               let lastIndex = pdfView.document?.pageCount {
                DispatchQueue.main.async {
                    pdfView.document?.insert(page, at: lastIndex)
                    pdfView.document?.write(to: documentURL)
                    pdfView.go(to: page)
                    
                    LocalFileManager().updateThumbnail(forURL: documentURL)
                }
            }
        }
    }
    
    class func appendPage(images: [UIImage], pdfView: PDFView, documentURL: URL?) {
        guard let documentURL = documentURL else  { return }
        let currentIndex = pdfView.getCurrentPageIndex()
        
        for image in images {
            if let page = PDFPage(image: image) {
                pdfView.document?.insert(page, at: currentIndex)
            }
        }
        
        if let page = pdfView.visiblePages.first {
            pdfView.document?.write(to: documentURL)
            pdfView.go(to: page)
        }
        
        LocalFileManager().updateThumbnail(forURL: documentURL)
        
        pdfView.document = PDFDocument(url: documentURL)
        pdfView.layoutDocumentView()
    }
    
    class func deletePage(at index: Int, pdfDocument: PDFDocument?, documentURL: URL?, pdfView: PDFView!, name: String = "") -> PDFDocument {
        guard let pdfDocument = pdfDocument, let documentURL = documentURL, let pdfView = pdfView else  { return PDFDocument() }
        pdfDocument.removePage(at: index)
        pdfDocument.write(to: documentURL)
        pdfView.document = pdfDocument
        pdfView.layoutDocumentView()
        
        let defaults = UserDefaults.standard
        let encoder = JSONEncoder()
        let decoder = JSONDecoder()
        
        // Fetch the existing array of PDFDocumentObjects from UserDefaults
        var existingPDFDocumentObjects: [PDFDocumentObject] = []
        var changedExistingPDFDocumentObjects: [PDFDocumentObject] = []
        
        if let savedData = defaults.object(forKey: "pdfDocumentObjects") as? Data {
            if let loadedPDFDocumentObjects = try? decoder.decode([PDFDocumentObject].self, from: savedData) {
                existingPDFDocumentObjects = loadedPDFDocumentObjects
                changedExistingPDFDocumentObjects = existingPDFDocumentObjects
            }
        }
        
        // Add the new PDFDocumentObject to the existing array
        for (i,item) in existingPDFDocumentObjects.enumerated() {
            if item.name.replacingOccurrences(of: ".pdf", with: "") == name {
                changedExistingPDFDocumentObjects[i].session.remove(index: index)
            }
        }
        
        // Save the updated array back to UserDefaults
        if let encodedData = try? encoder.encode(changedExistingPDFDocumentObjects) {
            defaults.set(encodedData, forKey: "pdfDocumentObjects")
        }
        
        LocalFileManager().updateThumbnail(forURL: documentURL)
        if pdfView.canGoToNextPage {
            pdfView.goToNextPage(pdfView)
        } else if pdfView.canGoToPreviousPage {
            pdfView.goToPreviousPage(pdfView)
        } else if pdfView.canGoToLastPage {
            pdfView.goToLastPage(pdfView)
        } else if pdfView.canGoForward {
            pdfView.goForward(pdfView)
        } else if pdfView.canGoBack {
            pdfView.goBack(pdfView)
        } else if pdfView.canGoToFirstPage {
            pdfView.goToFirstPage(pdfView)
        } else {
            pdfView.layoutDocumentView()
        }
        return pdfDocument
    }
    
    class func printPDF(from url: URL) {
        let printInfo = UIPrintInfo(dictionary:nil)
        printInfo.jobName = url.lastPathComponent
        printInfo.outputType = .general
        
        let printController = UIPrintInteractionController.shared
        printController.printInfo = printInfo
        
        let pdfData = try? Data(contentsOf: url)
        if pdfData != nil {
            printController.printingItem = pdfData
        }
        
        printController.present(animated: true, completionHandler: nil)
    }
    
    class func sharePDF(payload: [Any], viewController: UIViewController) {
        let shareController = UIActivityViewController(activityItems: payload, applicationActivities: nil)
        viewController.present(shareController, animated: true, completion: nil)
        
    }
    
    class func savePDF(session: MultiPageScanSession,
                       folder: AppConfigurator.Folder,
                       fileManager: LocalFileManager?,
                       name: String,
                       completion: @escaping () -> ()) {
        let items = session.scannedItems
        var images = [URL]()
        for index in 0..<(items.count) {
            if let url = items[index].renderedImage {
                images.append(url)
            }
        }
        PDFManager.createMultiPDFPage(from: images, localFileManager: fileManager, folder: folder, name: name, session: session, {
            completion()
        })
    }
    class func savePDFAndReturn(session: MultiPageScanSession,
                       folder: AppConfigurator.Folder,
                       fileManager: LocalFileManager?,
                       name: String,
                       completion: @escaping (URL) -> ()) {
        let items = session.scannedItems
        var images = [URL]()
        for index in 0..<(items.count) {
            if let url = items[index].renderedImage {
                images.append(url)
            }
        }
        PDFManager.createMultiPDFPageAndReturn(from: images, localFileManager: fileManager, folder: folder, name: name) { document in
            completion(document)
        }
    }
    
    class func createPDFFile(session: ScanSession, scannedItems: [ScannedItem]) -> URL {
        
        let document = PDFDocument()
        
        let documentURL = URL(fileURLWithPath: NSTemporaryDirectory())
        
        let fileUrl = documentURL.appendingPathComponent(session.name + ".pdf")
        
        for scannedItem in scannedItems {
            
            if let image = scannedItem.newRenderedImage?.retrieveImage(),
               let pdfPage = PDFPage(image: image) {
                
                document.insert(pdfPage, at: document.pageCount)
            }
        }
        
        document.write(to: fileUrl)
        
        return fileUrl
    }
    
    class func deletePDFFile(session: ScanSession) {
        
        let documentURL = URL(fileURLWithPath: NSTemporaryDirectory())
        
        let fileUrl = documentURL.appendingPathComponent(session.name + ".pdf")
        
        try? FileManager.default.removeItem(at: fileUrl)
    }
}
