import UIKit
import SwiftUI
import PDFKit
import PhotosUI
import VisionKit
import ZLImageEditor

protocol DocumentPreviewViewControllerDelegate: AnyObject {
    func sessionDeleted(session: ScanSession)
    func sessionUpdated(session: ScanSession)
    func sessionMoved(session: ScanSession, from: Folder, to: Folder)
}

final class DocumentPreviewViewController: UIViewController, ScanSessionSharable, ScanSessionPrintable {
    
    struct Constants {
        static let reuseIdentifier = String(describing: DocumentPreviewViewController.self)
        
        static let cellIdentifier = String(describing: ScannedItemPagerViewCell.self)
        
        static let storyboardName = "DocumentPreview"
    }
    
    static func buildViewController() -> DocumentPreviewViewController {
        let controller = UIStoryboard(name: Constants.storyboardName,
                                      bundle: .main).instantiateViewController(withIdentifier: Constants.reuseIdentifier)
        
        return controller as! DocumentPreviewViewController
    }
    
    @IBOutlet private weak var pagerView: FSPagerView!
    @IBOutlet weak var collectionView: UICollectionView!
    
    @IBOutlet weak var directShareLabel: UILabel!
    @IBOutlet weak var titleLabel: UILabel!
    @IBOutlet weak var closeButton: UIButton!
    @IBOutlet weak var layoutButton: UIButton!
    
    @IBOutlet weak var currentPageLabel: UILabel!
    @IBOutlet weak var nextPageButton: UIButton!
    @IBOutlet weak var previousPageButton: UIButton!
    
    @IBOutlet weak var shareTypeButton: UIButton!
    @IBOutlet weak var shareTypeLabel: UILabel!
    
    @IBOutlet weak var selectedPagesView: UIView!
    @IBOutlet weak var selectedPagesLabel: UILabel!
    
    @IBOutlet weak var clearSelectionButton: UIButton!
    
    @Inject var updateScanSessionUseCase: UpdateScanSessionUseCase
    @Inject var moveScanSessionUseCase: MoveScanSessionUseCase
    @Inject var deleteScanSessionUseCase: DeleteScanSessionUseCase
    
    @Inject var userSettings: UserSettings
    
    var session: ScanSession!
    var selectedFolder: Folder!
    var _navigationController: UINavigationController?
    weak var delegate: DocumentPreviewViewControllerDelegate?
    
    @Inject var getFoldersUseCase: GetFoldersUseCase
    
    private var currentPageIndex: Int = 0
    private var selectedPagesCount: Int = 0
    
    var sharingViewController: UIViewController { self }
    
    var shareType: ShareType = .pdf {
        didSet {
            handleShareTypeChanged()
        }
    }
    
    private var renameAlertController: UIAlertController?
    private var renameFileName: String?
    
    private var layoutType: PreviewCellLayout = .pager {
        didSet {
            switch layoutType {
            case .pager:
                displayPager()
            case .grid:
                displayGrid()
            }
        }
    }
    
    lazy var activityIndicator: UIActivityIndicatorView = {
        let activityIndicator = UIActivityIndicatorView(style: .large)
        activityIndicator.color = .gray
        activityIndicator.hidesWhenStopped = true
        activityIndicator.translatesAutoresizingMaskIntoConstraints = false
        return activityIndicator
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupUI()
    }
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        titleLabel.text = session.name
        
        directShareLabel.set(localized: .directShare)
        
        view.addSubview(activityIndicator)
        
        activityIndicator.centerXAnchor.constraint(equalTo: view.centerXAnchor).isActive = true
        activityIndicator.centerYAnchor.constraint(equalTo: view.centerYAnchor).isActive = true
        
        currentPageLabel.layer.cornerRadius = 5
        
        currentPageLabel.clipsToBounds = true
        
        clearSelectionButton.setTitle(.clear.localized, for: .normal)
        
        updateSelectedImagesLabel()
        
        updateCurrentPageLabel()
        
        shareType = userSettings.defaultFileType.shareType
        
        layoutType = .pager
        
        setupMenu()
        
        setupPagerView()
        
        setupCollectionView()
    }
    
    private func setupColors() {
        
        view.customBackground(.base)
        
        titleLabel.set(color: .gray600)
        closeButton.customTint(.gray600)
        
        nextPageButton.customTint(.gray600)
        previousPageButton.customTint(.gray600)
        
        currentPageLabel.set(color: .gray600)
        
        currentPageLabel.customBackground(.gray100)
        
        selectedPagesLabel.set(color: .primary)
        clearSelectionButton.customTint(.primary)
        
        shareTypeLabel.set(color: .gray600)
        layoutButton.customTint(.gray600)
    }
    
    private func setupFonts() {
        
        titleLabel.set(font: .regular(16))
        
        clearSelectionButton.titleLabel?.set(font: .medium(16))
        
        selectedPagesLabel.set(font: .medium(16))
        
        currentPageLabel.set(font: .medium(15))
        
        shareTypeLabel.set(font: .semibold(16))
    }
    
    private func setupPagerView() {
        
        pagerView.register(ScannedItemPagerViewCell.self, forCellWithReuseIdentifier: Constants.cellIdentifier)
        
        pagerView.transformer = FSPagerViewTransformer(type: .linear)
        
        let transform = CGAffineTransform(scaleX: 0.8, y: 0.8)
        self.pagerView.itemSize = self.pagerView.frame.size.applying(transform)
        
        pagerView.decelerationDistance = FSPagerView.automaticDistance
        pagerView.interitemSpacing = 16
        
        pagerView.dataSource = self
        pagerView.delegate = self
        
        pagerView.customBackground(.clear)
    }
    
    private func setupCollectionView() {
        
        let layout = UICollectionViewFlowLayout()
        
        layout.scrollDirection = .vertical
        layout.minimumLineSpacing = 10
        layout.minimumInteritemSpacing = 10
        
        collectionView.collectionViewLayout = layout
        
        collectionView.register(ScannedItemPagerViewCell.self, forCellWithReuseIdentifier: Constants.cellIdentifier)
        
        collectionView.delegate = self
        collectionView.dataSource = self
        
        collectionView.customBackground(.clear)
    }
    
    private func dismiss() {
        session.scannedItems.forEach { $0.isSelected = false }
        self.dismiss(animated: true)
    }
    
    private func clearSelection() {
        session.scannedItems.forEach { $0.isSelected = false }
        
        selectedPagesCount = 0
        
        updateSelectedImagesLabel()
        
        refreshContainer()
    }
    
    private func updateCurrentPageIndex(index: Int) {
        
        currentPageIndex = index
        
        updateCurrentPageLabel()
    }
    
    private func updateCurrentPageLabel() {
        
        currentPageLabel.set(text: "\(currentPageIndex + 1)/\(session.scannedItems.count)")
        
        currentPageLabel.textAlignment = .center
        
        previousPageButton.isEnabled = currentPageIndex > 0
        nextPageButton.isEnabled = currentPageIndex < session.itemsCount - 1
    }
    
    private func updateSelectedImagesLabel() {
        
        selectedPagesView.isHidden = selectedPagesCount == 0
        closeButton.isHidden = selectedPagesCount != 0
        
        selectedPagesLabel.set(text: String(selectedPagesCount))
    }
    
    private func handleShareTypeChanged() {
        shareTypeLabel.set(localized: shareType.displayName)
    }
    
    private func changeLayout(to layout: PreviewCellLayout) {
        self.layoutType = layout
    }
    
    private func displayPager() {
        pagerView.isHidden = false
        collectionView.isHidden = true
    }
    
    private func displayGrid() {
        pagerView.isHidden = true
        collectionView.isHidden = false
    }
    
    private func refreshContainer() {
        
        switch layoutType {
        case .pager:
            pagerView.reloadData()
        case .grid:
            collectionView.reloadData()
        }
    }
    
    private func refreshContainer(at index: Int) {
        
        switch layoutType {
        case .pager:
            pagerView.reloadData(at: index)
        case .grid:
            collectionView.reloadItems(at: [IndexPath(row: index, section: 0)])
        }
    }
}

extension DocumentPreviewViewController: FSPagerViewDelegate, FSPagerViewDataSource {
    
    func numberOfItems(in pagerView: FSPagerView) -> Int {
        session.itemsCount
    }
    
    func pagerView(_ pagerView: FSPagerView, cellForItemAt index: Int) -> FSPagerViewCell {
        
        let cell = pagerView.dequeueReusableCell(withReuseIdentifier: Constants.cellIdentifier, at: index) as! ScannedItemPagerViewCell
        
        let scannedItem = session.scannedItems[index]
        
        cell.configure(with: scannedItem, at: index, delegate: self)
        
        return cell
    }
    
    func pagerViewWillEndDragging(_ pagerView: FSPagerView, targetIndex: Int) {
        
        updateCurrentPageIndex(index: targetIndex)
    }
}

// MARK: - Collection view data source and delegate

extension DocumentPreviewViewController: UICollectionViewDelegate, UICollectionViewDataSource, UICollectionViewDelegateFlowLayout {
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        
        let itemHeight = 200.0
        
        let itemWidth = collectionView.frame.width / 2 - (5 + 32)
        
        return CGSize(width: itemWidth, height: itemHeight)
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, insetForSectionAt section: Int) -> UIEdgeInsets {
        UIEdgeInsets(top: 0, left: 32, bottom: 32, right: 32)
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        session.itemsCount
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        guard let cell = collectionView.dequeueReusableCell(withReuseIdentifier: Constants.cellIdentifier, for: indexPath) as? ScannedItemPagerViewCell else { return UICollectionViewCell() }
        
        let index = indexPath.row
        
        let scannedItem = session.scannedItems[index]
        
        cell.configure(with: scannedItem, at: index, delegate: self)
        
        return cell
    }
}

extension DocumentPreviewViewController: ScannedItemPagerViewCellDelegate {
    
    func select(at index: Int) {
        
        session.scannedItems[index].isSelected.toggle()
        
        selectedPagesCount += session.scannedItems[index].isSelected ? 1 : -1
        
        refreshContainer(at: index)
        
        updateSelectedImagesLabel()
    }
    
    func preview(at index: Int) {
        displayImagePreviewSheet(at: index)
    }
    
    func delete(at index: Int) {
        displayDeletePageDialog(at: index)
    }
}

// MARK: - UIPrintInteractionControllerDelegate

extension DocumentPreviewViewController {
    
    func printInteractionControllerDidDismissPrinterOptions(_ printInteractionController: UIPrintInteractionController) {
        
        printControllerDidDismiss(session: session)
    }
}

extension DocumentPreviewViewController {
    
    private func openCamera() {
        
        let scannerOptions = ImageScannerOptions(scanMultipleItems: true,
                                                 allowAutoScan: false,
                                                 allowTapToFocus: false,
                                                 defaultColorRenderOption: .color)
        
        let scannerViewController = ScannerViewController(scanSession: nil, options: scannerOptions, isAddedToPDF: true)
        scannerViewController.previewDelegate = self
        scannerViewController.hidesBottomBarWhenPushed = true
        
        dismiss()
        
        self._navigationController?.pushViewController(scannerViewController, animated: false)
    }
    
    private func openGallery() {
        var configuration = PHPickerConfiguration()
        configuration.selectionLimit = 0  // 0 means no limit
        configuration.filter = .images  // Only show images
        let picker = PHPickerViewController(configuration: configuration)
        picker.delegate = self
        picker.modalPresentationStyle = .fullScreen
        present(picker, animated: true, completion: nil)
    }
    
    private func displayImagePreviewSheet(at index: Int) {
        
        let previewController = ScanSessionPreviewViewController(scanSession: session, selectedIndex: index)
        
        previewController.modalPresentationStyle = .fullScreen
        
        present(previewController, animated: true)
    }
    
    private func displayEditViewController() {
        let editViewController = EditViewController(scanSession: session)
        
        editViewController.selectedFolder = selectedFolder
        editViewController.hidesBottomBarWhenPushed = true
        
        dismiss()
        
        self._navigationController?.pushViewController(editViewController, animated: false)
    }
}

// MARK: - Scanner preview view delegate

extension DocumentPreviewViewController: ScannerViewControllerPreviewDelegate {
    
    func addToPDF(_ scannerViewController: ScannerViewController, session: ScanSession) {
        
        scannerViewController.dismiss(animated: true)
        
        if let oldSession = self.session {
            
            oldSession.add(items: session.scannedItems)
            
            handleUpdatedSession()
        }
    }
}

// MARK: - PHPicker view delegate

extension DocumentPreviewViewController: PHPickerViewControllerDelegate {
    
    func picker(_ picker: PHPickerViewController, didFinishPicking results: [PHPickerResult]) {

        picker.dismiss(animated: true) { [weak self] in
            guard let self else { return }
            
            if !results.isEmpty {
                
                activityIndicator.startAnimating()
                
                handlePHPickerResults(results: results)
            }
        }
    }
    
    private func handlePHPickerResults(results: [PHPickerResult]) {
        
        var images: [UIImage] = []
        
        let dispatchGroup = DispatchGroup()
        
        for result in results {
            
            dispatchGroup.enter()  // Enter the group before each task
            
            result.itemProvider.loadObject(ofClass: UIImage.self) { (object, error) in
                
                defer { dispatchGroup.leave() } // Leave the group after each task, regardless of whether it was successful
                
                if let error = error {
                    print("Error loading image: \(error)")
                } else if let image = object as? UIImage {
                    images.append(image)
                }
            }
        }
        
        dispatchGroup.notify(queue: .main) { [weak self] in
            guard let self else { return }
            
            handlePHPickerImages(images: images)
        }
    }
    
    private func handlePHPickerImages(images: [UIImage]) {
        
        var scannedItems: [ScannedItem] = []
        
        images.forEach { image in
            
            let compressedImage = ImageCompressManager.compress(image: image)
            
            scannedItems.append(ScannedItem(originalImage: compressedImage, renderImage: compressedImage, type: .local))
        }
        
        if !scannedItems.isEmpty {
            
            session.add(items: scannedItems)
            
            handleUpdatedSession()
        }
        
        activityIndicator.stopAnimating()
    }
}

extension DocumentPreviewViewController {
    
    @IBAction func closeButtonTapped(_ sender: Any) {
        dismiss()
    }
    
    @IBAction func clearSelectionTapped(_ sender: Any) {
        clearSelection()
    }
    
    @IBAction func layoutTapped(_ sender: Any) {
        layoutType = layoutType.toggle()
        layoutButton.setImage(layoutType.image, for: .normal)
        
        changeLayout(to: layoutType)
    }
    
    @IBAction func nextPageTapped(_ sender: UIButton) {
        
        if currentPageIndex < (session.itemsCount - 1) {
            
            pagerView.scrollToItem(at: currentPageIndex + 1, animated: true)
            
            updateCurrentPageIndex(index: currentPageIndex + 1)
        }
    }
    
    @IBAction func previousPageTapped(_ sender: UIButton) {
        if currentPageIndex > 0 {
            
            pagerView.scrollToItem(at: currentPageIndex - 1, animated: true)
            
            updateCurrentPageIndex(index: currentPageIndex - 1)
        }
    }
    
    @IBAction func whatsappTapped(_ sender: UIButton) {
        shareScanSession(session: session, shareOption: .whatsapp)
    }
    
    @IBAction func telegramTapped(_ sender: UIButton) {
        shareScanSession(session: session, shareOption: .telegram)
    }
    
    @IBAction func gmailTapped(_ sender: UIButton) {
        shareScanSession(session: session, shareOption: .gmail)
    }
    
    @IBAction func airdropTapped(_ sender: Any) {
        shareScanSession(session: session, shareOption: .airdop)
    }
    
    @IBAction func printTapped(_ sender: UIButton) {
        printScanSession(session: session)
    }
    
    @IBAction func moreTapped(_ sender: UIButton) {
        shareScanSession(session: session, shareOption: .more)
    }
}

extension DocumentPreviewViewController {
    
    @IBAction func addPageTapped(_ sender: UIBarButtonItem) {
        let alertController = UIAlertController(title: nil,
                                                message: nil, preferredStyle: .actionSheet)
        
        let cameraAction = UIAlertAction(title: .camera.localized, style: .default, handler: { action in
            self.openCamera()
        })
        
        let galleryAction = UIAlertAction(title: .gallery.localized, style: .default, handler: { action in
            self.openGallery()
        })
        
        alertController.addAction(cameraAction)
        cameraAction.setValue(UIImage.camera, forKey: "image")
        cameraAction.setValue(CATextLayerAlignmentMode.left, forKey: "titleTextAlignment")
        
        alertController.addAction(galleryAction)
        galleryAction.setValue(UIImage.photo, forKey: "image")
        galleryAction.setValue(CATextLayerAlignmentMode.left, forKey: "titleTextAlignment")
        
        alertController.addAction(UIAlertAction(title: .cancel.localized, style: .cancel, handler: nil))
        alertController.popoverPresentationController?.barButtonItem = sender
        present(alertController, animated: true)
    }
    
    @IBAction func drawTapped(_ sender: UIBarButtonItem) {
        displayEditViewController()
    }
    
    @IBAction func deleteTapped(_ sender: UIBarButtonItem) {
        
        if selectedPagesCount == 0 {
            displayDeleteDocumentDialog()
        }
        else {
            displayDeleteDocumentOrPagesDialog()
        }
    }
    
    private func displayDeleteDocumentDialog() {
        
        let alertController = UIAlertController(title: .deleteDocument.localized,
                                                message: .areYouSure.localized,
                                                preferredStyle: .alert)
        
        let okAction = UIAlertAction(title: .delete.localized, style: .destructive, handler: { [weak self] _ in
            guard let self else { return }
            
            handleDeleteDocument()
        })
        
        alertController.addAction(okAction)
        
        alertController.addAction(UIAlertAction(title: .cancel.localized, style: .cancel, handler: nil))
        
        present(alertController, animated: true)
    }
    
    private func displayDeleteDocumentOrPagesDialog() {
        
        let alertController = UIAlertController(title: .delete.localized,
                                                message: .areYouSure.localized,
                                                preferredStyle: .alert)
        
        let selectedPagesAction = UIAlertAction(title: .deleteSelectedPages.localized, style: .destructive, handler: { [weak self] _ in
            guard let self else { return }
            
            handleDeleteSelectedPages()
        })
        
        let documentAction = UIAlertAction(title: .entireDocument.localized, style: .destructive, handler: { [weak self] _ in
            guard let self else { return }
            
            handleDeleteDocument()
        })
        
        alertController.addAction(selectedPagesAction)
        alertController.addAction(documentAction)
        
        alertController.addAction(UIAlertAction(title: .cancel.localized, style: .cancel, handler: nil))
        
        present(alertController, animated: true)
    }
    
    private func handleDeleteSelectedPages() {
        
        let selectedItems = session.scannedItems.filter { $0.isSelected }
        
        for item in selectedItems {
            session.removeItem(item: item)
        }
        
        selectedPagesCount = 0
        
        updateSelectedImagesLabel()
        
        handleUpdatedSession()
    }
    
    private func handleDeleteDocument() {
        
        deleteScanSessionUseCase.execute(with: session)
        
        delegate?.sessionDeleted(session: session)
        
        dismiss()
    }
    
    private func displayDeletePageDialog(at index: Int) {
        
        let alertController = UIAlertController(title: .delete.localized,
                                                message: .deletePageMsg.localized,
                                                preferredStyle: .alert)
        
        let okAction = UIAlertAction(title: .delete.localized, style: .destructive, handler: { [weak self] _ in
            guard let self else { return }
            
            handleDeletePage(at: index)
        })
        
        alertController.addAction(okAction)
        
        alertController.addAction(UIAlertAction(title: .cancel.localized, style: .cancel, handler: nil))
        
        present(alertController, animated: true)
    }
    
    private func handleDeletePage(at index: Int) {
        
        session.removeItem(at: index)
        
        handleUpdatedSession()
    }
    
    private func handleUpdatedSession() {
        
        if session.itemsCount == 0 {
            handleDeleteDocument()
        }
        else {
            
            updateScanSessionUseCase.execute(with: session)
            
            refreshContainer()
            
            delegate?.sessionUpdated(session: session)
            
            updateCurrentPageLabel()
        }
    }
}

extension DocumentPreviewViewController {
    
    private func setupMenu() {
        
        shareTypeButton.showsMenuAsPrimaryAction = true
        
        let pdf = UIAction(title: .pdf.localized) { _ in
            self.shareType = .pdf
        }
        
        let jpg = UIAction(title: .jpeg.localized) { _ in
            self.shareType = .jpeg
        }
        
        let menu = UIMenu(title: "", children: [pdf, jpg])
        
        if #available(iOS 16.0, *) {
            menu.preferredElementSize = .small
        }
        
        shareTypeButton.menu = menu
    }
}

extension DocumentPreviewViewController {
    
    private func displayRenameScanSessionDialog() {
        
        renameAlertController = UIAlertController(title: .renameDocument.localized, message: nil, preferredStyle: .alert)
        
        renameAlertController!.addTextField { [weak self] textField in
            guard let self else { return }
            
            textField.placeholder = session.name
            textField.text = session.name
            textField.clearButtonMode = .always
            
            renameFileName = session.name
            
            textField.addTarget(self, action: #selector(self.textFieldDidChange(_:)), for: .editingChanged)
        }
        
        let continueAction = UIAlertAction(title: .rename.localized,
                                           style: .default) { [weak self] _ in
            guard let self else { return }
            guard let textFields = renameAlertController?.textFields, let newName = textFields.first?.text else {
                return }
            
            handleRenameScanSession(to: newName)
        }
        
        renameAlertController!.addAction(continueAction)
        
        renameAlertController!.addAction(UIAlertAction(title: .cancel.localized, style: .cancel, handler: nil))
        
        renameAlertController?.actions.first?.isEnabled = false
        
        present(renameAlertController!, animated: true)
    }
    
    @objc func textFieldDidChange(_ textField: UITextField) {
        guard let text = textField.text, let renameFileName = renameFileName,
              renameFileName != text, !text.trimmingCharacters(in: .whitespaces).isEmpty else {
            renameAlertController?.actions.first?.isEnabled = false
            return
        }
        
        renameAlertController?.actions.first?.isEnabled = true
    }
    
    private func handleRenameScanSession(to name: String) {
        
        if name != session.name {
            
            session.name = name
            
            titleLabel.set(text: name)
            
            updateScanSessionUseCase.execute(with: session)
            
            delegate?.sessionUpdated(session: session)
        }
    }
    
    private func displayMoveScanSessionSheet() {
        
        let folders = getFoldersUseCase.execute()
        
        if let sourceFolder = folders.first (where: { $0.id == session.folderId }) {
            
            displayFoldersViewSheet(excludedFolders: [sourceFolder])
        }
    }
    
    private func displayFoldersViewSheet(excludedFolders: [Folder]) {
        
        let foldersViewController = FoldersViewController()
        
        foldersViewController.excludedFolders = excludedFolders
        foldersViewController.delegate = self
        
        present(foldersViewController, animated: true)
    }
    
    private func handleMoveScanSession(to destinationFolder: Folder) {
        
        moveScanSessionUseCase.execute(with: session, to: destinationFolder)
        
        delegate?.sessionMoved(session: session, from: selectedFolder, to: destinationFolder)
        
        session.folderId = destinationFolder.id
        selectedFolder = destinationFolder
    }
}

extension DocumentPreviewViewController: FoldersViewControllerDelegate {
    
    func dismissed(_ foldersViewController: FoldersViewController, folder: Folder?) {
        
        if let folder {
            handleMoveScanSession(to: folder)
        }
    }
}
