//
//  ScanSessionPreviewViewController.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 8/13/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit

class ScanSessionPreviewViewController: UIViewController {
    
    struct Constants {
        static let cellIdentifier = String(describing: ScannedItemPreviewPagerViewCell.self)
    }
    
    @IBOutlet weak var closeButton: UIButton!
    
    @IBOutlet private weak var pagerView: FSPagerView!
    
    @IBOutlet weak var currentPageLabel: UILabel!
    @IBOutlet private weak var previousPageButton: UIButton!
    @IBOutlet private weak var nextPageButton: UIButton!
    
    private var scanSession: ScanSession
    private var selectedIndex: Int
    private var currentPageIndex: Int = 0
    
    init(scanSession: ScanSession, selectedIndex: Int) {
        self.scanSession = scanSession
        self.selectedIndex = selectedIndex
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder aDecoder: NSCoder) {
        scanSession = .newSession()
        fatalError("init(coder:) should not be called for this class")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        pagerView.scrollToItem(at: selectedIndex, animated: false)
        
        updateCurrentPageIndex(index: selectedIndex)
    }
}

extension ScanSessionPreviewViewController {
    
    private func dismiss() {
        dismiss(animated: true)
    }
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        currentPageLabel.layer.cornerRadius = 5
        
        currentPageLabel.clipsToBounds = true
        
        updateCurrentPageLabel()
        
        setupPagerView()
    }
    
    private func setupColors() {
        
        currentPageLabel.set(color: .gray600)
        
        currentPageLabel.customBackground(.gray100)
        
        view.customBackground(.base)
        
        closeButton.customTint(.gray600)
        
        nextPageButton.customTint(.gray600)
        previousPageButton.customTint(.gray600)
        
        pagerView.customBackground(.clear)
    }
    
    private func setupFonts() {
        currentPageLabel.set(font: .medium(16))
    }
    
    private func updateCurrentPageIndex(index: Int) {
        
        currentPageIndex = index
        
        updateCurrentPageLabel()
    }
    
    private func updateCurrentPageLabel() {
        
        currentPageLabel.set(text: "\(currentPageIndex + 1)/\(scanSession.scannedItems.count)")
        
        currentPageLabel.textAlignment = .center
        
        previousPageButton.isEnabled = currentPageIndex > 0
        nextPageButton.isEnabled = currentPageIndex < scanSession.itemsCount - 1
    }
    
    private func setupPagerView() {
        
        pagerView.register(ScannedItemPreviewPagerViewCell.self, forCellWithReuseIdentifier: Constants.cellIdentifier)
        
        pagerView.transformer = FSPagerViewTransformer(type: .linear)
        
        pagerView.decelerationDistance = FSPagerView.automaticDistance
        
        pagerView.interitemSpacing = 16
        
        pagerView.dataSource = self
        pagerView.delegate = self
    }
}

extension ScanSessionPreviewViewController {
    
    @IBAction func closeTapped(_ sender: Any) {
        dismiss()
    }
    
    @IBAction func previousPageTapped(_ sender: UIButton) {
        if currentPageIndex > 0 {
            
            pagerView.scrollToItem(at: currentPageIndex - 1, animated: true)
            
            updateCurrentPageIndex(index: currentPageIndex - 1)
        }
    }
    
    @IBAction func nextPageTapped(_ sender: UIButton) {
        
        if currentPageIndex < (scanSession.itemsCount - 1) {
            
            pagerView.scrollToItem(at: currentPageIndex + 1, animated: true)
            
            updateCurrentPageIndex(index: currentPageIndex + 1)
        }
    }
}

extension ScanSessionPreviewViewController: FSPagerViewDataSource {
    
    func numberOfItems(in pagerView: FSPagerView) -> Int {
        scanSession.itemsCount
    }
    
    func pagerView(_ pagerView: FSPagerView, cellForItemAt index: Int) -> FSPagerViewCell {
        
        let cell = pagerView.dequeueReusableCell(withReuseIdentifier: Constants.cellIdentifier, at: index) as! ScannedItemPreviewPagerViewCell
        
        let scannedItem = scanSession.scannedItems[index]
        
        cell.configure(with: scannedItem)
        
        return cell
    }
    
    func pagerView(_ pagerView: FSPagerView, willDisplay cell: FSPagerViewCell, forItemAt index: Int) {
        
        if let cell = cell as? ScannedItemPreviewPagerViewCell {
            cell.resetZoomScale()
        }
    }
}

extension ScanSessionPreviewViewController: FSPagerViewDelegate {
    
    func pagerViewWillEndDragging(_ pagerView: FSPagerView, targetIndex: Int) {
        
        updateCurrentPageIndex(index: targetIndex)
    }
}
