//
//  ScannedItemPagerViewCell.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 8/4/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import Foundation

protocol ScannedItemPagerViewCellDelegate: AnyObject {
    func delete(at: Int)
    func preview(at: Int)
    func select(at: Int)
}

class ScannedItemPagerViewCell: FSPagerViewCell {
    
    var scannedItem: ScannedItem?
    
    weak var cellDelegate: ScannedItemPagerViewCellDelegate?
    
    lazy var buttonsView: UIView = {
        var view = UIView()
        
        view.addSubview(buttonsStackView)
        
        view.translatesAutoresizingMaskIntoConstraints = false
        
        buttonsStackView.leftAnchor.constraint(equalTo: view.leftAnchor, constant: 8).isActive = true
        view.rightAnchor.constraint(equalTo: buttonsStackView.rightAnchor, constant: 8).isActive = true
        
        buttonsStackView.topAnchor.constraint(equalTo: view.topAnchor, constant: 6).isActive = true
        view.bottomAnchor.constraint(equalTo: buttonsStackView.bottomAnchor, constant: 6).isActive = true
        
        buttonsStackView.heightAnchor.constraint(equalToConstant: 24).isActive = true
        
        view.customBackground(.transparentOverlay)
        view.cornerRadiuss = 5
        
        view.layoutIfNeeded()
        
        return view
    }()
    
    lazy var buttonsStackView: UIStackView = {
        let view = UIStackView()
        
        view.axis = .horizontal
        view.spacing = 16
        
        view.addArrangedSubview(select)
        view.addArrangedSubview(preview)
        view.addArrangedSubview(trash)
        
        view.translatesAutoresizingMaskIntoConstraints = false
        
        return view
    }()
    
    lazy var preview: UIButton = {
        var button = getButton(image: .preview)
        button.addTarget(self, action: #selector(previewTapped), for: .touchUpInside)
        return button
    }()
    
    lazy var trash: UIButton = {
        var button = getButton(image: .trash)
        button.addTarget(self, action: #selector(deleteTapped), for: .touchUpInside)
        return button
    }()
    
    lazy var select: UIButton = {
        var button = getButton(image: .add)
        button.addTarget(self, action: #selector(selectTapped), for: .touchUpInside)
        return button
    }()
    
    func configure(with scannedItem: ScannedItem,
                   at index: Int,
                   delegate: ScannedItemPagerViewCellDelegate) {
        
        self.scannedItem = scannedItem
        
        imageView?.image = scannedItem.newRenderedImage?.retrieveImage()
        
        addBorder()
        
        trashButton.isHidden = true
        previewButton.isHidden = true
        
        cellDelegate = delegate
        self.index = index
        
        contentView.addSubview(buttonsView)
        
        contentView.clipsToBounds = true
        
        buttonsView.centerXAnchor.constraint(equalTo: contentView.centerXAnchor).isActive = true
        contentView.bottomAnchor.constraint(equalTo: buttonsView.bottomAnchor, constant: 12).isActive = true
        
        if scannedItem.isSelected {
            select.setImage(.checkmarkCircleFill, for: .normal)
        }
        else {
            select.setImage(.add, for: .normal)
        }
    }
}

extension ScannedItemPagerViewCell {
    
    @objc private func previewTapped() {
        if let index {
            cellDelegate?.preview(at: index)
        }
    }
    
    @objc private func deleteTapped() {
        if let index {
            cellDelegate?.delete(at: index)
        }
    }
    
    @objc private func selectTapped() {
        if let index {
            cellDelegate?.select(at: index)
        }
    }
    
    private func getButton(image: UIImage) -> UIButton {
        let button = UIButton(frame: CGRect(x: 0, y: 0, width: 24, height: 24))
        
        button.setImage(image, for: .normal)
        
        button.customTint(.gray0)

        button.heightAnchor.constraint(equalToConstant: 24).isActive = true
        button.widthAnchor.constraint(equalToConstant: 24).isActive = true
        
        return button
    }
}
