//
//  AllFolderTableViewCell.swift
//  MiniScanner
//
//  Created by George Makhoul on 06/06/2024.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit

protocol AllFolderTableViewCellDelegate {
    func selectFolder(folder: Folder)
    func deleteFolder(folder: Folder)
    func addFolderTapped()
}

final class AllFolderTableViewCell: UIView {
    // MARK: - Outlets
    @IBOutlet private weak var view: UIView!
    @IBOutlet weak var collectionView: UICollectionView!
    @IBOutlet weak var addFolderButton: UIButton!
    @IBOutlet weak var foldersLabel: UILabel!
    
    // MARK: - Properties
    var folders: [Folder] = []
    var delegate: AllFolderTableViewCellDelegate?
    
    // MARK: - LifeCycle
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        setupNib()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        
        setupNib()
        setupUI()
    }
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        setupCollectionView()
        
        addFolderButton.set(localized: .add)
        foldersLabel.set(localized: .folders)
    }
    
    func reloadLocalization() {
        addFolderButton.set(localized: .add)
        foldersLabel.set(localized: .folders)
    }
    
    private func setupNib() {
        Bundle.main.loadNibNamed("AllFolderTableViewCell", owner: self, options: nil)
        
        view.frame = bounds
        view.autoresizingMask = [.flexibleHeight, .flexibleWidth]
        
        addSubview(view)
    }
    
    private func setupColors() {
        
        view.customBackground(.clear)
        collectionView.customBackground(.clear)
        
        foldersLabel.set(color: .gray600)
        
        addFolderButton.customTitleColor(.primary)
    }
    
    private func setupFonts() {
        
        foldersLabel.set(font: .bold(20))
    }
    
    @IBAction func addFolderButtonTapped(_ sender: UIButton) {
        delegate?.addFolderTapped()
    }
    
    private func setupCollectionView() {
        self.collectionView.collectionViewLayout = .fixedSpacedFlowLayout()
        
        self.collectionView.delegate = self
        self.collectionView.dataSource = self
        self.collectionView.register(UINib(nibName: "FolderCollectionViewCell", bundle: nil), forCellWithReuseIdentifier: "FolderCollectionViewCell")
    }
    
    func set(folders: [Folder]) {
        
        self.folders = folders
        collectionView.reloadData()
    }
    
    func set(delegate: AllFolderTableViewCellDelegate?) {
        self.delegate = delegate
    }
    
    func set(selectedFolder: Folder) {
        
        selectFolder(folder: selectedFolder)
    }
}

extension AllFolderTableViewCell: UICollectionViewDelegate, UICollectionViewDataSource {
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        folders.count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        if let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "FolderCollectionViewCell", for: indexPath) as? FolderCollectionViewCell {
            cell.set(folder: folders[indexPath.row])
            cell.delegate = self
            return cell
        }
        return UICollectionViewCell()
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        
        let folder = folders[indexPath.row]
        
        selectFolder(folder: folder)
        
        delegate?.selectFolder(folder: folder)
    }
}

extension AllFolderTableViewCell {
    
    private func selectFolder(folder: Folder) {
        
        if let index = folders.firstIndex(where: { $0.isSelected }) {
            folders[index].isSelected = false
        }
        
        if let index = folders.firstIndex(where: { $0.id == folder.id }) {
            folders[index].isSelected = true
        }
        
        collectionView.reloadData()
    }
}

extension AllFolderTableViewCell: FolderCollectionViewCellDelegate {
    func deleteFolder(folder: Folder) {
        delegate?.deleteFolder(folder: folder)
    }
}

extension UICollectionViewLayout {
    
    static func fixedSpacedFlowLayout() -> UICollectionViewLayout {
        
        let itemSize = NSCollectionLayoutSize(
            widthDimension: .estimated(60),
            heightDimension: .absolute(50)
        )
        
        let item = NSCollectionLayoutItem(layoutSize: itemSize)
        
        item.edgeSpacing = NSCollectionLayoutEdgeSpacing(
            leading: .fixed(0),
            top: .fixed(0),
            trailing: .fixed(0),
            bottom: .fixed(0)
        )
        
        let groupSize = NSCollectionLayoutSize(
            widthDimension: .fractionalWidth(1.0),
            heightDimension: .estimated(112)
        )
        
        let group = NSCollectionLayoutGroup.horizontal(layoutSize: groupSize, subitems: [item])
        
        let section = NSCollectionLayoutSection(group: group)
        
        let layout = UICollectionViewCompositionalLayout(section: section)
        
        return layout
    }
}
