//
//  CustomTabBarView.swift
//  MiniScanner
//
//  Created by George Makhoul on 10/06/2024.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit
import ImageViewer_swift

protocol CustomTabBarViewDelegate {
    func scanTapped()
    func fileManagerTapped()
    func editButtonTapped()
    func shareTapped()
}

class CustomTabBarView: UIView {
    // MARK: - Outlet
    @IBOutlet var view: UIView!
    @IBOutlet weak var editImageView: UIView!
    @IBOutlet weak var editImageLabel: UILabel!
    @IBOutlet weak var editImageIcon: UIImageView!
    @IBOutlet weak var scannedImage: UIImageView!
    @IBOutlet weak var badgeView: UIView!
    @IBOutlet weak var badgeLabel: UILabel!
    @IBOutlet weak var scannedImageView: UIView!
    @IBOutlet weak var fileManagerButton: UIButton!
    @IBOutlet weak var galleryButton: UIButton!
    @IBOutlet weak var shareButton: UIButton!
    @IBOutlet weak var shareImage: UIImageView!
    @IBOutlet weak var scanButton: UIButton!
    @IBOutlet weak var editImage_trailing: NSLayoutConstraint!
    @IBOutlet weak var editImage_width: NSLayoutConstraint!
    @IBOutlet weak var customtabBarLayer: CustomTabBarLayer!
    
    // MARK: - Properties
    var delegate: CustomTabBarViewDelegate?
    var isAddedToPDF: Bool = false
    
    @IBOutlet weak var galleryView: UIView!
    @IBOutlet weak var fileManagerView: UIView!
    
    @IBOutlet weak var fileManagerLabel: UILabel!
    @IBOutlet weak var galleryLabel: UILabel!
    
    // MARK: - LifeCycle
    init(frame: CGRect, isAddedToPDF: Bool = false) {
        super.init(frame: frame)
        self.isAddedToPDF = isAddedToPDF
        setupNib()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        self.isAddedToPDF = false
        setupNib()
    }
    
    private func setupNib() {
        Bundle.main.loadNibNamed("CustomTabBarView", owner: self, options: nil)
        view.frame = bounds
        view.autoresizingMask = [.flexibleHeight, .flexibleWidth]
        addSubview(view)
        
        setupUI()
    }
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        editImageIcon.image = .preview
        
        editImageLabel.set(text: .preview.localized)
        
        editImageView.layer.cornerRadius = 10
        
        badgeView.layer.cornerRadius = 10
        
        customtabBarLayer.isHidden = isAddedToPDF
        fileManagerView.isHidden = isAddedToPDF
        galleryView.isHidden = isAddedToPDF
        
        scanButton.setImage(.scanButton, for: .normal)
        fileManagerButton.setImage(.folderOpen, for: .normal)
        galleryButton.setImage(.gallery, for: .normal)
        
        fileManagerLabel.set(localized: .fileManager)
        
        galleryLabel.set(localized: .gallery)
    }
    
    private func setupColors() {
        editImageLabel.set(color: .gray0)
        editImageView.customBackground(.primary)
        badgeView.customBackground(.statusFalse)
        
        badgeLabel.set(color: .gray0)
        
        fileManagerLabel.set(color: .gray600)
        galleryLabel.set(color: .gray600)
    }
    
    private func setupFonts() {
        
        fileManagerLabel.set(font: .medium(16))
        
        galleryLabel.set(font: .medium(16))
        
        editImageLabel.set(font: .medium(18))
        
        badgeLabel.set(font: .regular(14))
    }
    
    func updateScannedView(session: ScanSession) {
        
        if !session.scannedItems.isEmpty {
            
            editImageView.isHidden = false
            scannedImageView.isHidden = false
            shareImage.isHidden = isAddedToPDF
            shareButton.isHidden = isAddedToPDF
            
            badgeView.isHidden = false
            
            let items = session.scannedItems
            
            scannedImage.image = items.last?.newRenderedImage?.retrieveImage()
            
            badgeLabel.set(text: "\(items.count)")
            
            badgeLabel.textAlignment = .center
            
            if isAddedToPDF {
                
                editImageIcon.image = nil
                editImageLabel.set(text: .done.localized)
                editImageLabel.textAlignment = .center
                editImage_width.constant = 0
                editImage_trailing.constant = 0
                
                var images: [UIImage] = []
                
                for item in session.scannedItems {
                    if let image = item.renderedImage?.retrieveImage() {
                        images.append(image)
                    }
                    else if let image = item.originalImage?.retrieveImage() {
                        images.append(image)
                    }
                }
                
                scannedImage.setupImageViewer(images: images)
            }
            
        } else {
            editImageView.isHidden = true
            scannedImageView.isHidden = true
            badgeView.isHidden = true
        }
    }
    
    @IBAction func scanButtonTapped(_ sender: UIButton) {
        delegate?.scanTapped()
    }
    
    @IBAction func filesManagerButtonTapped(_ sender: UIButton) {
        delegate?.fileManagerTapped()
    }
    
    @IBAction func editImageTapped(_ sender: UIButton) {
        delegate?.editButtonTapped()
    }
    
    @IBAction func shareButtonTapped(_ sender: UIButton) {
        delegate?.shareTapped()
    }
}
