//
//  KNAlertViewController.swift
//  opa
//
//  Created by George Makhoul on 2/23/22.
//

import UIKit

final class KNAlertViewController: BMViewController {
    
    enum KNAlertViewType {
        case none
        
        case success
        case error
        case decision
    }

    // MARK: - Outlets
    @IBOutlet private weak var dimView: UIView!
    
    @IBOutlet private weak var alertView: UIView!
    @IBOutlet private weak var alertView_height: NSLayoutConstraint!
    
    @IBOutlet private weak var alertImage: UIImageView!
    @IBOutlet private weak var titleLabel: UILabel!
    @IBOutlet private weak var messageTextView: UITextView!
    
    @IBOutlet weak var messageTextViewBottomConstraint: NSLayoutConstraint!
    
    @IBOutlet weak var textFieldView: UIView!
    @IBOutlet weak var folderName: UITextField!
    
    @IBOutlet weak var textFieldViewHeight: NSLayoutConstraint!
    @IBOutlet weak var textFieldViewBottomConstrait: NSLayoutConstraint!
    
    @IBOutlet private weak var buttonStackView: UIStackView!
    @IBOutlet private weak var confirmationButton: UISecondaryButton!
    @IBOutlet private weak var destructiveButton: UITertiaryButton!
    
    @IBOutlet private weak var buttonView_leading: NSLayoutConstraint!
    @IBOutlet private weak var buttonsView_trailing: NSLayoutConstraint!
    
    @IBOutlet private weak var buttonsView_height: NSLayoutConstraint!
    
    // MARK: - Properties

    private let dimViewColor: UIColor = UIColor.gray.withAlphaComponent(0.6)
    private var type: KNAlertViewType = .none
    private var content_height: CGFloat = 150
    
    private var alertTitle: String = ""
    private var image: UIImage?
    private var message: String = ""
    private var needsFolderTextField: Bool = true
    
    private var confirmButtonTitle: String = .yes
    private var destructiveButtonTitle: String = .no
    
    var shouldDismissAndLogin: Bool = false
    
    var confirmClosure: AlertViewTextClosure?
    var destructiveClosure: AlertViewClosure?
    
    private let BUTTON_HEIGHT: CGFloat = 52
    private let BUTTON_WIDTH: CGFloat = 128
    
    private var buttonsView_Spacing: CGFloat = 26
    private var folderNameText: String = ""
    
    // MARK: - Init
    init(type: KNAlertViewType,
         image: UIImage,
         title: String,
         message: String,
         needsFolderTextField: Bool) {
        super.init()
        
        self.type = type
        
        self.image = image
        self.alertTitle = title
        self.message = message
        self.needsFolderTextField = needsFolderTextField
    }
    
    init(type: KNAlertViewType,
         image: UIImage,
         title: String,
         message: String,
         confirm: String,
         destructive: String,
         needsFolderTextField: Bool) {
        super.init()
        
        self.type = type
        self.image = image
        self.alertTitle = title
        self.message = message
        self.needsFolderTextField = needsFolderTextField
        
        confirmButtonTitle = confirm
        destructiveButtonTitle = destructive
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    // MARK: - Lifecycle
    override func viewDidLoad() {
        super.viewDidLoad()
        
        view.endEditing(true)
        
        setupUI()
        updateUI()
        updateHeight()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        showPopup()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
    }
}

// MARK: - UI
extension KNAlertViewController {
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        // alert
        alertView.layer.cornerRadius = 16
        
        titleLabel.textAlignment = .center
        
        // text
        messageTextView.textAlignment = .center
        
        messageTextView.text = ""
        messageTextView.isEditable = false
        messageTextView.isSelectable = false
        messageTextView.textContainerInset = .zero
        messageTextView.textContainer.lineFragmentPadding = 0
        
        // buttons stack
        buttonStackView.spacing = 27
        
        // buttons
        
        confirmationButton.set(localized: confirmButtonTitle)
        
        confirmationButton.addTarget(self, action: #selector(confirmationButtonTapped), for: .touchUpInside)
        
        destructiveButton.set(localized: destructiveButtonTitle)
        
        destructiveButton.addTarget(self, action: #selector(destructiveButtonTapped), for: .touchUpInside)
        
        buttonView_leading.constant = buttonsView_Spacing
        buttonsView_trailing.constant = buttonsView_Spacing

        destructiveButton.isHidden = true
        
        confirmationButton.isHidden = true
        
        buttonsView_height.constant = 0
        
        folderName.placeholder = .folderFiles.localized
        
        folderName.addTarget(self, action: #selector(textChanged), for: .editingChanged)
        confirmationButton.isEnabled = folderName.text != "" ? true : false
    }
    
    private func setupColors() {
        
        view.customBackground(.clear)
        dimView.backgroundColor = dimViewColor
        
        alertView.customBackground(.base)
        
        titleLabel.set(color: .gray600)
        
        messageTextView.textColor = ColorStyle.gray600.uiColor
        
        buttonStackView.customBackground(.clear)
    }
    
    private func setupFonts() {
        
        folderName.font = .regular(14)
        
        titleLabel.set(font: .bold(20))
        
        messageTextView.font = .regular(18)
    }
    
    @objc func textChanged() {
        folderNameText = folderName.text ?? ""
        confirmationButton.isEnabled = folderName.text != "" ? true : false
    }
    
    private func updateHeight() {
        var height: CGFloat = 0
        
        height += 28
        height += 20
        height += 28
        height += buttonsView_height.constant
        height += textFieldViewHeight.constant
        height += 108
        height += 38
        height += 27
        height += 20
        
        //@TODO @LOW dynamic height? using a tableView?
        let message_height = messageTextView.height()
        
        if message_height > 500 {
            height += 500
            messageTextView.isScrollEnabled = true
        } else {
            height += message_height
            messageTextView.isScrollEnabled = false
        }
        
        if height < 130 {
            content_height = 130
        } else {
            content_height = height + 20
        }
    }
    
    private func updateUI() {
        titleLabel.text = alertTitle
        messageTextView.text = message
        alertImage.image = image
        
        destructiveButton.isHidden = true
        
        confirmationButton.isHidden = true
        
        buttonsView_height.constant = 0
        
        if message.isEmpty {
            messageTextView.isHidden = true
            messageTextViewBottomConstraint.constant = 0
        }
        
        if !needsFolderTextField {
            textFieldView.isHidden = true
            textFieldViewHeight.constant = 0
            textFieldViewBottomConstrait.constant = 0
        }
        
        switch type {
        case .success, .error:
            let alert_width = alertView.frame.width
            
            buttonsView_Spacing = (alert_width - BUTTON_WIDTH) / 2
            
            buttonView_leading.constant = buttonsView_Spacing
            buttonsView_trailing.constant = buttonsView_Spacing

            confirmationButton.isHidden = false
            confirmationButton.set(localized: .ok)
            buttonsView_height.constant = BUTTON_HEIGHT
            
        case .decision:
            buttonsView_Spacing = 26
            
            buttonView_leading.constant = buttonsView_Spacing
            buttonsView_trailing.constant = buttonsView_Spacing

            destructiveButton.isHidden = false
            
            confirmationButton.isHidden = false
            confirmationButton.set(localized: .yes)

            buttonsView_height.constant = BUTTON_HEIGHT
        default:
            break
        }
    }
    
    private func showPopup() {
        var transform = CGAffineTransform.identity
        transform = transform.translatedBy(x: 0, y: UIScreen.main.bounds.height)
        transform = transform.scaledBy(x: 0, y: 0)
        alertView.transform = transform
        
        dimView.alpha = 0
        UIView.animate(withDuration: 0.3) { [weak self] in
            self?.alertView_height.constant = (self?.content_height ?? 0)
            self?.alertView.transform = .identity
            self?.dimView.alpha = 1.0
            
            self?.view.layoutIfNeeded()
        }
    }
    
    @objc private func dismissPopup() {
        UIView.animate(withDuration: 0.3) { [weak self] in
            var transform = CGAffineTransform.identity
            transform = transform.translatedBy(x: 0, y: UIScreen.main.bounds.height)
            transform = transform.scaledBy(x: 0, y: 0)
            self?.alertView.transform = transform
            
            self?.dimView.alpha = 0
            self?.view.layoutIfNeeded()
        } completion: { [weak self] finished in
            if finished {
                self?.dismiss(animated: false) {
                    //
                }
            }
        }
    }
}

// MARK: - Delegate
extension KNAlertViewController {
    @objc func destructiveButtonTapped() {
        if let closure = destructiveClosure {
            closure()
        } else {
            dismissPopup()
        }
    }
    
    @objc func confirmationButtonTapped() {
        if let closure = confirmClosure {
           closure(folderNameText)
        } else {
            dismissPopup()
        }
    }
    
    func hideEverything() {
        view.isHidden = true
    }
}
