//
//  SearchFilesView.swift
//  MiniScanner
//
//  Created by George Makhoul on 08/06/2024.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit
protocol SearchFilesViewDelegate {
    func searchFor(text: String)
    func changeLayout(to layout: DocumentLayout)
}

final class SearchFilesView: UIView {
    // MARK: - Outlets
    @IBOutlet private weak var view: UIView!
    @IBOutlet private weak var titleLabel: UILabel!
    @IBOutlet private weak var searchButton: UIButton!
    @IBOutlet private weak var layoutButton: UIButton!
    @IBOutlet weak var searchTextField: UITextField!
    @IBOutlet weak var exitButton: UIButton!
    @IBOutlet weak var boxView: UIView!
    
    // MARK: - Properties
    var delegate: SearchFilesViewDelegate?
    var layout: DocumentLayout = .list {
        didSet {
            layoutButton.setImage(layout.image, for: .normal)
        }
    }
    
    // MARK: - LifeCycle
    override init(frame: CGRect) {
        super.init(frame: frame)
        setupNib()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        
        setupNib()
    }
    
    private func setupNib() {
        Bundle.main.loadNibNamed("SearchFilesView", owner: self, options: nil)
        view.frame = bounds
        view.autoresizingMask = [.flexibleHeight, .flexibleWidth]
        addSubview(view)
        
        setupUI()
    }
    
    @IBAction func searchTapped(_ sender: UIButton) {
        searchTextField.isHidden = !searchTextField.isHidden
        titleLabel.isHidden = !searchTextField.isHidden
        searchButton.isHidden = !searchTextField.isHidden
        layoutButton.isHidden = !searchTextField.isHidden
        exitButton.isHidden = searchTextField.isHidden
        if !searchTextField.isHidden {
            searchTextField.clearButtonMode = .always
            searchTextField.becomeFirstResponder()
        }
    }
    
    @IBAction func layoutTapped(_ sender: Any) {
        layout = layout.toggle()
        layoutButton.setImage(layout.image, for: .normal)
        delegate?.changeLayout(to: layout)
    }
    
    @IBAction func exitButtonTapped(_ sender: UIButton) {
        searchTextField.isHidden = true
        exitButton.isHidden = true
        searchButton.isHidden = false
        layoutButton.isHidden = false
        titleLabel.isHidden = false
        searchTextField.text = ""
        searchTextField.resignFirstResponder()
        delegate?.searchFor(text: "")
    }
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        titleLabel.set(text: .folderFiles.localized)
        
        searchTextField.isHidden = true
        searchTextField.addTarget(self, action: #selector(textFieldDidChange), for: .editingChanged)
        
        searchTextField.delegate = self
        
        titleLabel.isHidden = !searchTextField.isHidden
        searchButton.isHidden = !searchTextField.isHidden
        layoutButton.isHidden = !searchTextField.isHidden
        exitButton.isHidden = searchTextField.isHidden
        
        searchButton.setImage(.search, for: .normal)
        
        layoutButton.setImage(layout.image, for: .normal)
    }
    
    private func setupColors() {
        
        titleLabel.set(color: .gray600)
    }
    
    private func setupFonts() {
        
        searchTextField.font = .regular(14)
        
        titleLabel.set(font: .semibold(18))
    }
    
    func reloadLocalization() {
        titleLabel.set(text: .folderFiles.localized)
    }
    
    @objc private func textFieldDidChange() {
        
        if let text = searchTextField.text {
            delegate?.searchFor(text: text)
        }
    }
}

// MARK: - TextFieldDelegate
extension SearchFilesView: UITextFieldDelegate {
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        return true
    }
}
