//
//  ShareSheetViewController.swift
//  MiniScanner
//
//  Created by George Makhoul on 10/06/2024.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit
protocol ShareSheetViewControllerDelegate {
    
    func shareOptionTapped(name: String, shareOption: ShareOption, shareType: ShareType)
    
    func printTapped(shareType: ShareType)
    
    func saveTapped(name: String)
}

protocol PreviewDelegate {
    func previewTapped()
}

class ShareSheetViewController: UIViewController {
    // MARK: - Outlets
    @IBOutlet weak var thumbnailImage: UIImageView!
    @IBOutlet weak var fileNameTextField: UITextField!
    @IBOutlet weak var pagesCountLabel: UILabel!
    
    @IBOutlet weak var backgroundView: UIView!
    @IBOutlet weak var previewButton: UIButton!
    
    @IBOutlet weak var saveView: UIView!
    
    @IBOutlet weak var divider: UIView!
    
    @IBOutlet weak var dividerBottomConstraint: NSLayoutConstraint!
    @IBOutlet weak var shareStackView: UIStackView!
    @IBOutlet weak var shareStack_height: NSLayoutConstraint!
    
    @IBOutlet weak var shareStackBottomConstraint: NSLayoutConstraint!
    
    @IBOutlet weak var shareTypeButton: UIButton!
    @IBOutlet weak var shareTypeView: UIView!
    @IBOutlet weak var shareTypeLabel: UILabel!
    @IBOutlet weak var saveLabel: UILabel!
    @IBOutlet weak var whatsappLabel: UILabel!
    @IBOutlet weak var telegramLabel: UILabel!
    @IBOutlet weak var gmailLabel: UILabel!
    @IBOutlet weak var airdropLabel: UILabel!
    @IBOutlet weak var printLabel: UILabel!
    @IBOutlet weak var moreLabel: UILabel!
    
    @Inject var userSettings: UserSettings
    
    // MARK: - Properties
    var session: ScanSession!
    
    var delegate: ShareSheetViewControllerDelegate?
    var previewDelegate: PreviewDelegate?
    
    var isShareShown: Bool = true
    
    var shareType: ShareType = .pdf {
        didSet {
            handleShareTypeChanged()
        }
    }
    
    // MARK: - LifeCycle
    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupUI()
        
        setup()
    }
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        pagesCountLabel.set(text: "")
        
        saveLabel.set(localized: .save)
        
        shareTypeLabel.set(localized: shareType.displayName)
        
        whatsappLabel.set(localized: .whatsapp)
        
        telegramLabel.set(localized: .telegram)
        
        gmailLabel.set(localized: .gmail)
        
        airdropLabel.set(localized: .airdrop)
        
        printLabel.set(localized: .print)
        
        moreLabel.set(localized: .more)
        
        whatsappLabel.textAlignment = .center
        telegramLabel.textAlignment = .center
        gmailLabel.textAlignment = .center
        airdropLabel.textAlignment = .center
        printLabel.textAlignment = .center
        moreLabel.textAlignment = .center
        
        fileNameTextField.placeholder = .fileName.localized
        
        fileNameTextField.borderWidth = 0
        
        if session.folderId != nil {
            fileNameTextField.text = session.name
            fileNameTextField.isEnabled = false
        }
        
        backgroundView.layer.cornerRadius = 30
        backgroundView.layer.maskedCorners = [.layerMinXMinYCorner, .layerMaxXMinYCorner]
        
        thumbnailImage.layer.cornerRadius = 10
        thumbnailImage.clipsToBounds = true
        thumbnailImage.contentMode = .scaleAspectFill
        
        previewButton.setImage(.preview.tint(with: .gray600), for: .normal)
        
        divider.isHidden = !isShareShown
        shareStackView.isHidden = !isShareShown
        shareTypeView.isHidden = !isShareShown
        shareStack_height.constant = isShareShown ? 96 : 0
        
        if !isShareShown {
            dividerBottomConstraint.constant = 0
            shareStackBottomConstraint.constant = 0
        }
        
        shareType = userSettings.defaultFileType.shareType
        
        setupMenu()
    }
    
    private func setupColors() {
        
        pagesCountLabel.set(color: .gray350)
        
        saveLabel.set(color: .gray600)
        
        shareTypeLabel.set(color: .gray600)
        
        whatsappLabel.set(color: .gray600)
        telegramLabel.set(color: .gray600)
        gmailLabel.set(color: .gray600)
        airdropLabel.set(color: .gray600)
        printLabel.set(color: .gray600)
        moreLabel.set(color: .gray600)
        
        backgroundView.customBackground(.base)
    }
    
    private func setupFonts() {
        
        fileNameTextField.font = .semibold(24)
        
        pagesCountLabel.set(font: .regular(14))
        
        shareTypeLabel.set(font: .semibold(16))
        
        saveLabel.set(font: .semibold(16))
        
        whatsappLabel.set(font: .regular(16))
        telegramLabel.set(font: .regular(16))
        gmailLabel.set(font: .regular(16))
        airdropLabel.set(font: .regular(16))
        printLabel.set(font: .regular(16))
        moreLabel.set(font: .regular(16))
    }
    
    private func setup() {
        
        thumbnailImage.image = session.scannedItems[0].newRenderedImage?.retrieveImage()
        
        let pagesCount = session.scannedItems.count
        let text = String.dPages.localizedPlural(argument: pagesCount)
        
        pagesCountLabel.set(text: text)
    }
    
    private func handleShareTypeChanged() {
        shareTypeLabel.set(localized: shareType.displayName)
    }
    
    private func setupMenu() {
        
        shareTypeButton.showsMenuAsPrimaryAction = true
        
        let pdf = UIAction(title: .pdf.localized) { _ in
            self.shareType = .pdf
        }
        
        let jpg = UIAction(title: .jpeg.localized) { _ in
            self.shareType = .jpeg
        }
        
        let menu = UIMenu(title: "", children: [pdf, jpg])
        
        if #available(iOS 16.0, *) {
            menu.preferredElementSize = .small
        }
        
        shareTypeButton.menu = menu
    }
    
    @IBAction func previewTapped(_ sender: UIButton) {
        preview()
    }
    
    @IBAction func saveTapped(_ sender: UIButton) {
        save()
    }
    
    @IBAction func whatsappTapped(_ sender: UIButton) {
        share(shareOption: .whatsapp)
    }
    
    @IBAction func telegramTapped(_ sender: UIButton) {
        share(shareOption: .telegram)
    }
    
    @IBAction func gmailTapped(_ sender: Any) {
        share(shareOption: .gmail)
    }
    
    @IBAction func airdropTapped(_ sender: UIButton) {
        share(shareOption: .airdop)
    }
    
    @IBAction func printTapped(_ sender: UIButton) {
        print()
    }
    
    @IBAction func moreTapped(_ sender: UIButton) {
        share(shareOption: .more)
    }
}

extension ShareSheetViewController {
    
    private func share(shareOption: ShareOption) {
        dismiss(animated: true) { [weak self] in
            guard let self else { return }
            
            delegate?.shareOptionTapped(name: fileNameTextField.text ?? "", shareOption: shareOption, shareType: shareType)
            
        }
    }
    
    private func save() {
        dismiss(animated: true) { [weak self] in
            guard let self else { return }
            
            delegate?.saveTapped(name: fileNameTextField.text ?? "")
        }
    }
    
    private func preview() {
        dismiss(animated: true) { [weak self] in
            guard let self else { return }
            
            previewDelegate?.previewTapped()
        }
    }
    
    private func print() {
        delegate?.printTapped(shareType: shareType)
    }
}
