import UIKit
import NBBottomSheet
import PDFKit
import VisionKit
import PhotosUI

final class DocumentsTableViewController: UIViewController {
    
    struct Constants {
        static let reuseIdentifier = String(describing: DocumentsTableViewController.self)
        static let storyboardName = "Documents"
        
        static let tableCellReuseIdentifier = String(describing: DocumentsTableViewCell.self)
        static let collectionCellReuseIdentifier = String(describing: DocumentsCollectionViewCell.self)
    }
    
    static func buildViewController() -> DocumentsTableViewController {
        let controller = UIStoryboard(name: Constants.storyboardName,
                                      bundle: .main).instantiateViewController(withIdentifier: Constants.reuseIdentifier)
        return controller as! DocumentsTableViewController
    }
    
    @IBOutlet weak var allFoldersView: AllFolderTableViewCell!
    @IBOutlet weak var searchForFilesView: SearchFilesView!
    
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var collectionView: UICollectionView!
    @IBOutlet weak var noFilesLabel: UILabel!
    
    @Inject var getFoldersUseCase: GetFoldersUseCase
    @Inject var addFolderUseCase: AddFolderUseCase
    @Inject var deleteFoldersUseCase: DeleteFolderUseCase
    
    @Inject var addScanSessionUseCase: AddScanSessionUseCase
    @Inject var updateScanSessionUseCase: UpdateScanSessionUseCase
    @Inject var moveScanSessionUseCase: MoveScanSessionUseCase
    @Inject var deleteScanSessionUseCase: DeleteScanSessionUseCase
    
    @Inject var userSettings: UserSettings
    
    private var folders: [Folder] = []
    
    private var sessions: [ScanSession] = []
    private var displayedSessions: [ScanSession] = []
    
    private var sortType: SortyFileType = .date
    
    private var layoutType: DocumentLayout = .list {
        didSet {
            switch layoutType {
            case .list:
                displayList()
            case .grid:
                displayGrid()
            }
        }
    }
    
    private var renameAlertController: UIAlertController?
    private var renameFileName: String?
    private var selectedFolder: Folder?
    
    private var selectedSessionToMove: ScanSession?
    
    private var scannedItems: [ScannedItem]?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        handleViewDidLoad()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        handleViewWillAppear()
    }
    
    deinit {
        NotificationCenter.default.removeObserver(self)
    }
    
    private func getScanSession(at indexPath: IndexPath) -> ScanSession {
        displayedSessions[indexPath.row]
    }
    
    @objc private func middleButtonTapped() {
        openCamera()
    }
    
    @objc private func rightButtonTapped() {
        openGallery()
    }
    
    @objc private func settingsTapped() {
        openSettings()
    }
    
    @objc private func premiumTapped() {
        openPremium()
    }
}

extension DocumentsTableViewController {
    
    private func handleViewDidLoad() {
        
        setupUI()
    }
    
    private func setupUI() {
        
        setupToolbar()
        
        setupTableView()
        
        setupCollectionView()
        
        setupFoldersView()
        
        setupTabbarNotifications()
        
        setupColors()
        
        layoutType = .list
        
        searchForFilesView.layout = layoutType
        
        searchForFilesView.delegate = self
        
        navigationController?.navigationBar.prefersLargeTitles = false
        
        noFilesLabel.set(text: .noFilesToShow.localized)
        
        if userSettings.defaultScreen == .camera {
            openCamera()
        }
    }
    
    private func setupColors() {
        
        view.customBackground(.base)
        
        noFilesLabel.set(color: .gray600)
        
        tableView.customBackground(.clear)
        collectionView.customBackground(.clear)
    }
    
    private func setupFonts() {
        
        noFilesLabel.set(font: .medium(22))
    }
    
    private func setupToolbar() {
        
        let settingsButton = UIBarButtonItem(image: .settings, style: .done, target: self, action: #selector(settingsTapped))
        
        let premiumButton = UIBarButtonItem(image: .sparkles, style: .done, target: self, action: #selector(premiumTapped))
        
        navigationItem.rightBarButtonItem = settingsButton
        
        navigationItem.leftBarButtonItem = premiumButton
    }
    
    private func setupTableView() {
        
        tableView.register(UINib(nibName: Constants.tableCellReuseIdentifier, bundle: nil), forCellReuseIdentifier: Constants.tableCellReuseIdentifier)
        
        tableView.tableFooterView = UIView()
        tableView.delegate = self
        tableView.dataSource = self
    }
    
    private func setupCollectionView() {
        let layout = UICollectionViewFlowLayout()
        
        layout.scrollDirection = .vertical
        layout.minimumLineSpacing = 10
        layout.minimumInteritemSpacing = 10
        
        collectionView.collectionViewLayout = layout
        
        collectionView.register(UINib(nibName: Constants.collectionCellReuseIdentifier, bundle: nil), forCellWithReuseIdentifier: Constants.collectionCellReuseIdentifier)
        
        collectionView.delegate = self
        collectionView.dataSource = self
    }
    
    private func setupFoldersView() {
        
        allFoldersView.set(delegate: self)
    }
    
    private func setupTabbarNotifications() {
        
        tabBarController?.delegate = UIApplication.shared.delegate as? UITabBarControllerDelegate
        
        NotificationCenter.default.addObserver(self, selector: #selector(middleButtonTapped),
                                               name: NSNotification.Name.MiddleButtonTapped, object: nil)
        
        NotificationCenter.default.addObserver(self, selector: #selector(rightButtonTapped),
                                               name: NSNotification.Name.rightButtonTapped, object: nil)
    }
}

extension DocumentsTableViewController {
    
    private func handleViewWillAppear() {
        
        self.tabBarController?.tabBar.isHidden = false
        
        fetchFolders()
        
        fetchScanSessions()
        
        handleBackIcon()
        
        navigationItem.title = .fileManager.localized
        
        noFilesLabel.set(text: .noFilesToShow.localized)
        
        setupFonts()
        
        allFoldersView.reloadLocalization()
        searchForFilesView.reloadLocalization()
    }
    
    private func handleBackIcon() {
        
        let navigationController = navigationController!
        
        var backImage: UIImage? = nil
        
        let deviceLanguage = SupportedLanguage.device.language
        let currentLanguageLayoutDirection = SupportedLanguage.currentLanguage.layoutDirection
        
        if deviceLanguage == .ar {
            backImage = currentLanguageLayoutDirection == .leftToRight ? .chevronRight : .chevronLeft
        }
        else {
            backImage = currentLanguageLayoutDirection == .leftToRight ? .chevronLeft : .chevronRight
        }
        
        navigationController.navigationBar.backIndicatorImage = backImage
        navigationController.navigationBar.backIndicatorTransitionMaskImage = backImage
    }
}

extension DocumentsTableViewController {
    
    private func fetchFolders() {
        
        let index = folders.firstIndex { $0.id == selectedFolder?.id } ?? 0
        
        var folders = getFoldersUseCase.execute()
        folders.insert(.allDocuments, at: 0)
        
        folders[index].isSelected = true
        
        self.folders = folders
        
        refreshDisplayedFolders()
    }
    
    private func refreshDisplayedFolders() {
        
        selectedFolder = folders.first { $0.isSelected }
        
        allFoldersView.set(folders: folders)
        
        if let selectedFolder {
            allFoldersView.set(selectedFolder: selectedFolder)
        }
    }
    
    private func fetchScanSessions() {
        
        sessions.removeAll()
        
        folders.forEach { sessions.append(contentsOf: $0.scanSessions) }
        
        refreshDisplayedScanSessions()
    }
    
    private func refreshDisplayedScanSessions() {
        
        if let selectedFolder {
            
            if selectedFolder != .allDocuments {
                displayedSessions = selectedFolder.scanSessions
            }
            else {
                displayedSessions = sessions
            }
            
            displayedSessions.sort(by: { $0.creationDate.compare($1.creationDate) == .orderedDescending })
            
            refreshScanSessionsContainers()
        }
    }
    
    private func refreshScanSessionsContainers() {
        
        noFilesLabel.isHidden = !displayedSessions.isEmpty
        
        tableView.reloadData()
        collectionView.reloadData()
    }
}

extension DocumentsTableViewController {
    
    private func displayList() {
        tableView.isHidden = false
        collectionView.isHidden = true
    }
    
    private func displayGrid() {
        tableView.isHidden = true
        collectionView.isHidden = false
    }
    
    private func openCamera() {
        
        let scannerOptions = ImageScannerOptions(scanMultipleItems: true,
                                                 allowAutoScan: false,
                                                 allowTapToFocus: false,
                                                 defaultColorRenderOption:.color)
        
        let scannerViewController = ScannerViewController(scanSession: nil, options: scannerOptions)
        
        scannerViewController.delegate = self
        scannerViewController.selectedFolder = selectedFolder
        scannerViewController.hidesBottomBarWhenPushed = true
        
        self.navigationController?.pushViewController(scannerViewController, animated: false)
    }
    
    private func openGallery() {
        var configuration = PHPickerConfiguration()
        configuration.selectionLimit = 0  // 0 means no limit
        configuration.filter = .images  // Only show images
        let picker = PHPickerViewController(configuration: configuration)
        picker.delegate = self
        picker.modalPresentationStyle = .fullScreen
        present(picker, animated: true, completion: nil)
    }
    
    private func openSettings() {
        
        if let navigationController {
            let settingsCoorindator = SettingsViewCoordinator(navigationController: navigationController)
            settingsCoorindator.start()
        }
    }
    
    private func openPremium() {
        self.view.makeToast("Will open something soon")
    }
    
    private func openDocumentPreview(session: ScanSession) {
        
        let controller = DocumentPreviewViewController.buildViewController()
        controller.session = session
        controller.selectedFolder = folders.first { $0.id == session.folderId }
        controller._navigationController = self.navigationController
        controller.delegate = self
        
        controller.hidesBottomBarWhenPushed = true
        
        controller.view.frame = CGRect(x: 0, y: 0, width: view.frame.width, height: view.frame.height * 0.85)
        
        controller.view.layer.cornerRadius = 30
        controller.view.layer.maskedCorners = [.layerMinXMinYCorner, .layerMaxXMinYCorner]
        
        let configuration = NBBottomSheetConfiguration(animationDuration: 0.4, sheetSize: .fixed(controller.view.frame.height))
        
        let bottomSheetController = NBBottomSheetController(configuration: configuration)
        
        bottomSheetController.present(controller, on: self)
    }
}

// MARK: - Table view data source and delegate

extension DocumentsTableViewController: UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        // #warning Incomplete implementation, return the number of sections
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // #warning Incomplete implementation, return the number of rows
        displayedSessions.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: Constants.tableCellReuseIdentifier, for: indexPath) as? DocumentsTableViewCell else { return UITableViewCell() }
        
        // Configure the cell...
        let session = getScanSession(at: indexPath)
        
        cell.configure(with: session, indexPath: indexPath)
        
        cell.delegate = self
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 100
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        return UIView()
    }
    
    func tableView(_ tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat {
        20
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let session = getScanSession(at: indexPath)
        
        openDocumentPreview(session: session)
    }
    
    func tableView(_ tableView: UITableView,
                   editingStyleForRowAt indexPath: IndexPath) -> UITableViewCell.EditingStyle {
        return .none
    }
}

// MARK: - Collection view data source and delegate

extension DocumentsTableViewController: UICollectionViewDelegate, UICollectionViewDataSource, UICollectionViewDelegateFlowLayout {
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        
        let itemHeight = 181.0
        
        let itemWidth = collectionView.frame.width / 2 - (5 + 16)
        
        return CGSize(width: itemWidth, height: itemHeight)
    }
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, insetForSectionAt section: Int) -> UIEdgeInsets {
        UIEdgeInsets(top: 0, left: 16, bottom: 32, right: 16)
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        displayedSessions.count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        guard let cell = collectionView.dequeueReusableCell(withReuseIdentifier: Constants.collectionCellReuseIdentifier, for: indexPath) as? DocumentsCollectionViewCell else { return UICollectionViewCell() }
        
        let session = getScanSession(at: indexPath)
        
        cell.configure(with: session, indexPath: indexPath)
        
        return cell
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        
        let session = getScanSession(at: indexPath)
        
        openDocumentPreview(session: session)
    }
}

// MARK: - Scanner view delegate

extension DocumentsTableViewController: ScannerViewControllerDelegate {
    
    func scannerViewController(_ scannerViewController: ScannerViewController, reviewItems inSession: MultiPageScanSession) {
    }
    
    func scannerViewController(_ scannerViewController: ScannerViewController, didFail withError: Error) {
        scannerViewController.dismiss(animated: true)
    }
    
    func scannerViewControllerDidCancel(_ scannerViewController: ScannerViewController) {
        
    }
}

// MARK: - PHPicker view delegate

extension DocumentsTableViewController: PHPickerViewControllerDelegate {
    
    func picker(_ picker: PHPickerViewController, didFinishPicking results: [PHPickerResult]) {
        
        picker.dismiss(animated: true) { [weak self] in
            guard let self else { return }
            
            if results.isEmpty {
                tabBarController?.selectedIndex = 0
            }
            else {
                handlePHPickerResults(results: results)
            }
        }
    }
    
    private func handlePHPickerResults(results: [PHPickerResult]) {
        
        var images: [UIImage] = []
        
        let dispatchGroup = DispatchGroup()
        
        for result in results {
            
            dispatchGroup.enter()  // Enter the group before each task
            
            result.itemProvider.loadObject(ofClass: UIImage.self) { (object, error) in
                
                defer { dispatchGroup.leave() } // Leave the group after each task, regardless of whether it was successful
                
                if let error = error {
                    print("Error loading image: \(error)")
                } else if let image = object as? UIImage {
                    images.append(image)
                }
            }
        }
        
        dispatchGroup.notify(queue: .main) { [weak self] in
            guard let self else { return }
            
            handlePHPickerImages(images: images)
        }
    }
    
    private func handlePHPickerImages(images: [UIImage]) {
        
        var scannedItems: [ScannedItem] = []
        
        images.forEach { image in
            
            let compressedImage = ImageCompressManager.compress(image: image)
            
            if let selectedFolder, selectedFolder != .allDocuments {
                
                scannedItems.append(ScannedItem(originalImage: compressedImage, renderImage: compressedImage, type: .local))
            }
            else {
                scannedItems.append(ScannedItem(originalImage: compressedImage, renderImage: compressedImage))
            }
        }
        
        if !scannedItems.isEmpty {
            
            if let selectedFolder {
                
                if selectedFolder != .allDocuments {
                    addScanSession(scannedItems: scannedItems, folder: selectedFolder)
                }
                else {
                    tabBarController?.selectedIndex = 0
                    
                    self.scannedItems = scannedItems
                    displayFoldersViewSheet(excludedFolders: [])
                }
            }
        }
    }
    
    private func addScanSession(scannedItems: [ScannedItem], folder: Folder) {
        
        let scanSession = ScanSession(id: UUID(),
                                      name: .getScanSessionName(),
                                      creationDate: .now,
                                      folderId: folder.id,
                                      scannedItems: scannedItems)
        
        addScanSessionUseCase.execute(with: scanSession, to: folder)
        
        handleAddedScanSession(session: scanSession, to: folder)
    }
    
    private func handleAddedScanSession(session: ScanSession, to folder: Folder) {
        
        if folder.id == selectedFolder?.id {
            selectedFolder?.scanSessions.append(session)
        }
        
        if let index = folders.firstIndex(of: folder) {
            folders[index].scanSessions.append(session)
        }
        
        tabBarController?.selectedIndex = 0
        
        fetchScanSessions()
    }
}

// MARK: - All folders view delegate

extension DocumentsTableViewController: AllFolderTableViewCellDelegate {
    
    func addFolderTapped() {
        displayAddFolderDialog()
    }
    
    func selectFolder(folder: Folder) {
        
        if let index = folders.firstIndex(of: folder) {
            selectedFolder = folders[index]
        }
        
        refreshDisplayedScanSessions()
    }
    
    func deleteFolder(folder: Folder) {
        
        if let index = folders.firstIndex(of: folder) {
            displayDeleteFolderDialog(folder: folders[index])
        }
    }
    
    private func displayAddFolderDialog() {
        alert(confirm: .addFolder,
              destructive: .cancel,
              image: .folder,
              title: .addFolderMessage,
              message: "",
              handleAddFolderDialog)
    }
    
    private func handleAddFolderDialog(folderName: String) {
        addFolder(folderName: folderName)
    }
    
    private func addFolder(folderName: String) {
        
        let folder = Folder(id: UUID(),
                            name: folderName)
        
        addFolderUseCase.execute(with: folder)
        
        folders.append(folder)
        
        allFoldersView.set(folders: folders)
        
        self.dismiss(animated: true)
    }
    
    private func displayDeleteFolderDialog(folder: Folder) {
        
        let deleteMessage = String(format: .deleteFolderMsgS.localized, folder.localizedName)
        
        let alertController = UIAlertController(title: Bundle.appName,
                                                message: deleteMessage,
                                                preferredStyle: .alert)
        
        let okAction = UIAlertAction(title: .delete.localized, style: .destructive, handler: { [weak self] _ in
            guard let self else { return }
            
            handleDeleteFolder(folder: folder)
        })
        
        alertController.addAction(okAction)
        
        alertController.addAction(UIAlertAction(title: .cancel.localized, style: .cancel, handler: nil))
        
        present(alertController, animated: true)
    }
    
    private func handleDeleteFolder(folder: Folder) {
        // TODO: - Delete the folder's associated files.
        
        deleteFoldersUseCase.execute(with: folder)
        
        folders.remove(element: folder)
        
        folders[0].isSelected = true
        
        folder.scanSessions.forEach { session in
            
            sessions.remove(element: session)
        }
        
        refreshDisplayedFolders()
        
        refreshDisplayedScanSessions()
    }
}

// MARK: - Search view delegate

extension DocumentsTableViewController: SearchFilesViewDelegate {
    
    func searchFor(text: String) {
        
        if text.isEmpty {
            refreshDisplayedScanSessions()
        }
        else {
            
            guard let selectedFolder else { return }
            
            if selectedFolder != .allDocuments {
                
                displayedSessions = selectedFolder.scanSessions.filter { $0.name.lowercased().contains(text.lowercased()) }
            }
            else {
                displayedSessions = sessions.filter { $0.name.lowercased().contains(text.lowercased()) }
            }
            
            refreshScanSessionsContainers()
        }
    }
    
    func changeLayout(to layout: DocumentLayout) {
        self.layoutType = layout
    }
}

// MARK: - Documents cells delegate

extension DocumentsTableViewController: DocumentsCellDelegate {
    
    func renameScanSession(at indexPath: IndexPath) {
        let session = getScanSession(at: indexPath)
        
        displayRenameScanSessionDialog(session: session)
    }
    
    func moveScanSession(at indexPath: IndexPath) {
        let session = getScanSession(at: indexPath)
        
        displayMoveScanSessionSheet(session: session)
    }
    
    func deleteScanSession(at indexPath: IndexPath) {
        let session = getScanSession(at: indexPath)
        
        displayDeleteScanSessionDialog(session: session, at: indexPath)
    }
    
    private func displayRenameScanSessionDialog(session: ScanSession) {
        
        renameAlertController = UIAlertController(title: .renameDocument.localized, message: nil, preferredStyle: .alert)
        
        renameAlertController!.addTextField { [weak self] textField in
            guard let self else { return }
            
            textField.placeholder = session.name
            textField.text = session.name
            textField.clearButtonMode = .always
            
            renameFileName = session.name
            
            textField.addTarget(self, action: #selector(self.textFieldDidChange(_:)), for: .editingChanged)
        }
        
        let continueAction = UIAlertAction(title: .rename.localized,
                                           style: .default) { [weak self] _ in
            guard let self else { return }
            guard let textFields = renameAlertController?.textFields, let newName = textFields.first?.text else {
                return }
            
            handleRenameScanSession(session: session, to: newName)
        }
        
        renameAlertController!.addAction(continueAction)
        
        renameAlertController!.addAction(UIAlertAction(title: .cancel.localized, style: .cancel, handler: nil))
        
        renameAlertController?.actions.first?.isEnabled = false
        
        present(renameAlertController!, animated: true)
    }
    
    @objc func textFieldDidChange(_ textField: UITextField) {
        guard let text = textField.text, let renameFileName = renameFileName,
              renameFileName != text, !text.trimmingCharacters(in: .whitespaces).isEmpty else {
            renameAlertController?.actions.first?.isEnabled = false
            return
        }
        
        renameAlertController?.actions.first?.isEnabled = true
    }
    
    private func handleRenameScanSession(session: ScanSession, to name: String) {
        
        if name != session.name {
            
            session.name = name
            
            updateScanSessionUseCase.execute(with: session)
            
            refreshScanSessionsContainers()
        }
    }
    
    private func displayMoveScanSessionSheet(session: ScanSession) {
        
        if let sourceFolder = folders.first (where: { $0.id == session.folderId }) {
            
            selectedSessionToMove = session
            
            displayFoldersViewSheet(excludedFolders: [sourceFolder])
        }
    }
    
    private func displayFoldersViewSheet(excludedFolders: [Folder]) {
        
        let foldersViewController = FoldersViewController()
        
        foldersViewController.excludedFolders = excludedFolders
        foldersViewController.delegate = self
        
        present(foldersViewController, animated: true)
    }
    
    private func handleMoveScanSession(session: ScanSession,
                                       from sourceFolder: Folder,
                                       to destinationFolder: Folder) {
        
        moveScanSessionUseCase.execute(with: session, to: destinationFolder)
        
        handleMovedScanSession(session: session, from: sourceFolder, to: destinationFolder)
    }
    
    private func handleMovedScanSession(session: ScanSession,
                                        from sourceFolder: Folder,
                                        to destinationFolder: Folder) {
        
        if let index = folders.firstIndex(of: sourceFolder) {
            folders[index].scanSessions.remove(element: session)
        }
        
        selectedFolder?.scanSessions.remove(element: session)
        
        if let index = folders.firstIndex(of: destinationFolder) {
            folders[index].scanSessions.append(session)
        }
        
        fetchScanSessions()
    }
    
    private func displayDeleteScanSessionDialog(session: ScanSession, at indexPath: IndexPath) {
        
        let alertController = UIAlertController(title: Bundle.appName,
                                                message: .deleteDocumentMsg.localized, preferredStyle: .alert)
        
        let okAction = UIAlertAction(title: .yesDelete.localized, style: .destructive, handler: { [weak self] action in
            guard let self = self else { return }
            
            handleDeleteScanSession(session: session)
        })
        
        alertController.addAction(okAction)
        
        alertController.addAction(UIAlertAction(title: .cancel.localized, style: .cancel, handler: nil))
        
        present(alertController, animated: true)
    }
    
    private func handleDeleteScanSession(session: ScanSession) {
        // TODO: - Delete the session's associated files.
        
        deleteScanSessionUseCase.execute(with: session)
        
        handleDeletedSession(session: session)
    }
    
    private func handleDeletedSession(session: ScanSession) {
        
        sessions.remove(element: session)
        displayedSessions.remove(element: session)
        
        selectedFolder?.scanSessions.remove(element: session)
        
        if let index = folders.firstIndex(where: { $0.id == session.folderId }) {
            folders[index].scanSessions.remove(element: session)
        }
        
        refreshScanSessionsContainers()
    }
}

// MARK: - Folders view delegate

extension DocumentsTableViewController: FoldersViewControllerDelegate {
    
    func dismissed(_ foldersViewController: FoldersViewController, folder: Folder?) {
        
        if let folder, let session = selectedSessionToMove, let sourceFolder = folders.first(where: { $0.id == session.folderId }) {
            
            selectedSessionToMove = nil
            
            handleMoveScanSession(session: session, from: sourceFolder, to: folder)
        }
        else if let scannedItems {
            
            if let folder {
                
                addScanSession(scannedItems: scannedItems, folder: folder)
                
                self.scannedItems = nil
            }
            else {
                
                let scanSession = ScanSession(id: UUID(), name: "", creationDate: .now, scannedItems: scannedItems)
                
                deleteScanSessionUseCase.execute(with: scanSession)
            }
        }
    }
}

extension DocumentsTableViewController: DocumentPreviewViewControllerDelegate {
    
    func sessionDeleted(session: ScanSession) {
        handleDeletedSession(session: session)
    }
    
    func sessionUpdated(session: ScanSession) {
        refreshScanSessionsContainers()
    }
    
    func sessionMoved(session: ScanSession,
                      from sourceFolder: Folder,
                      to destinationFolder: Folder) {
        
        handleMovedScanSession(session: session, from: sourceFolder, to: destinationFolder)
    }
}
