//
//  FoldersViewController.swift
//  MiniScanner
//
//  Created by George Makhoul on 24/06/2024.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit
protocol FoldersViewControllerDelegate {
    func dismissed(_ foldersViewController: FoldersViewController, folder: Folder?)
}

class FoldersViewController: UIViewController {
    
    // MARK: - Outlets
    @IBOutlet weak var searchBar: UISearchBar!
    @IBOutlet weak var collectionView: UICollectionView!
    
    // MARK: - Properties
    
    var excludedFolders: [Folder] = []
    var delegate: FoldersViewControllerDelegate?
    
    @Inject var getFoldersUseCase: GetFoldersUseCase
    
    private var folders: [Folder] = []
    private var displayedFolders: [Folder] = []
    private var selectedFolder: Folder? = nil
    
    @IBOutlet weak var titleLabel: UILabel!
    
    // MARK: - LifeCycle
    override func viewDidLoad() {
        super.viewDidLoad()
        
        fetchFolders()
        
        setupUI()
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        delegate?.dismissed(self, folder: selectedFolder)
    }
    
    private func fetchFolders() {
        
        folders = getFoldersUseCase.execute()
        
        excludedFolders.forEach { folder in
            folders.remove(element: folder)
        }
        
        displayedFolders = folders
    }
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        searchBar.delegate = self
        
        titleLabel.text = .selectFolder.localized
        
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical
        layout.minimumLineSpacing = 10
        layout.minimumInteritemSpacing = 10
        
        self.collectionView.collectionViewLayout = layout
        
        self.collectionView.register(UINib(nibName: "FolderSelectCollectionViewCell", bundle: nil), forCellWithReuseIdentifier: "FolderSelectCollectionViewCell")
        
        collectionView.delegate = self
        collectionView.dataSource = self
        
        collectionView.reloadData()
    }
    
    private func setupColors() {
        view.customBackground(.base)
        collectionView.customBackground(.clear)
        
        titleLabel.set(color: .gray600)
    }
    
    private func setupFonts() {
        titleLabel.set(font: .semibold(18))
    }
}

// MARK: - UICollectionViewDelegate
extension FoldersViewController: UICollectionViewDataSource, UICollectionViewDelegate {
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        displayedFolders.count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        if let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "FolderSelectCollectionViewCell", for: indexPath) as? FolderSelectCollectionViewCell {
            
            cell.set(title: displayedFolders[indexPath.row].localizedName)
            
            return cell
        }
        
        return UICollectionViewCell()
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        
        selectedFolder = displayedFolders[indexPath.row]
        
        dismiss(animated: true)
    }
}

// MARK: - UICollectionViewDelegateFlowLayout
extension FoldersViewController: UICollectionViewDelegateFlowLayout {
    
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        
        let itemWidth = 125
        let itemHeight = 150
        
        return CGSize(width: itemWidth, height: itemHeight)
    }
}

extension FoldersViewController: UISearchBarDelegate {
    
    func searchBar(_ searchBar: UISearchBar, textDidChange searchText: String) {
        
        if !searchText.isEmpty {
            displayedFolders = folders.filter { $0.name.lowercased().contains(searchText.lowercased()) }
        }
        else {
            displayedFolders = folders
        }
        
        collectionView.reloadData()
    }
}
