//
//  BannerView.swift
//  MiniScanner
//
//  Created by George Makhoul on 11/06/2024.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit

protocol BannerViewDelegate {
    func updateSelectedIndex(index: Int)
    func delete(index: Int)
    func preview(index: Int)
    func update(session: ScanSession)
    func willStartDragging()
    func deleteSignAt(index: Int)
    func posChangedTo(pos: CGPoint, at: Int)
    func sizeChanged(size: CGSize,angle: CGFloat, at: Int)
}

class BannerView: UIView {
    
    // MARK: - Outlets
    @IBOutlet var view: UIView!
    
    @IBOutlet weak var pagerView: FSPagerView!{
        didSet {
            self.pagerView.register(FSPagerViewCell.self, forCellWithReuseIdentifier: "cell")
            self.typeIndex = 3
            pagerView.dataSource = self
            pagerView.delegate = self
            
        }
    }
    
    var signatureImage: UIImage!
    var signAllDoc: Bool = false
    var donePressed: Bool = false
    var isEditing: Bool = false
    
    fileprivate let transformerTypes: [FSPagerViewTransformerType] = [.crossFading,
                                                                      .zoomOut,
                                                                      .depth,
                                                                      .linear,
                                                                      .overlap,
                                                                      .ferrisWheel,
                                                                      .invertedFerrisWheel,
                                                                      .coverFlow,
                                                                      .cubic]
    
    private var options:ImageScannerOptions!
    private var deviceOrientationHelper = DeviceOrientationHelper()

    fileprivate var typeIndex = 0 {
        didSet {
            let type = self.transformerTypes[typeIndex]
            self.pagerView.transformer = FSPagerViewTransformer(type:type)
            switch type {
            case .crossFading, .zoomOut, .depth:
                self.pagerView.itemSize = FSPagerView.automaticSize
                self.pagerView.decelerationDistance = 1
            case .linear, .overlap:
                let transform = CGAffineTransform(scaleX: 0.8, y: 0.75)
                self.pagerView.itemSize = self.pagerView.frame.size.applying(transform)
                self.pagerView.decelerationDistance = FSPagerView.automaticDistance
            case .ferrisWheel, .invertedFerrisWheel:
                self.pagerView.itemSize = CGSize(width: 180, height: 140)
                self.pagerView.decelerationDistance = FSPagerView.automaticDistance
            case .coverFlow:
                self.pagerView.itemSize = CGSize(width: 220, height: 170)
                self.pagerView.decelerationDistance = FSPagerView.automaticDistance
            case .cubic:
                let transform = CGAffineTransform(scaleX: 0.9, y: 0.9)
                self.pagerView.itemSize = self.pagerView.frame.size.applying(transform)
                self.pagerView.decelerationDistance = 1
            }
        }
    }
    
    // MARK: - Properties
    var delegate: BannerViewDelegate?
    var scanSession: ScanSession!
    
    // MARK: - Init
    override init(frame: CGRect) {
        super.init(frame: frame)
        setupNib()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        setupNib()
    }
    
    private func setupNib() {
        Bundle.main.loadNibNamed("BannerView", owner: self, options: nil)
        
        view.frame = bounds
        view.autoresizingMask = [.flexibleHeight, .flexibleWidth]
        
        addSubview(view)
        
        options = ImageScannerOptions()
        
        setupUI()
        
        clearUI()
    }
    
    private func setupUI() {
        setupColors()
    }
    
    private func setupColors() {
        
        view.customBackground(.clear)
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        clearUI()
    }
    
    // MARK: - Methods
    
    func set(session: ScanSession) {
        self.scanSession = session
        clearUI()
    }
    
    private func clearUI() {
        
        deviceOrientationHelper.startDeviceOrientationNotifier { (deviceOrientation) in
            print("orientation: \(deviceOrientation)")
            self.orientationChanged(deviceOrientation: deviceOrientation)
        }
    }
    
    private func orientationChanged(deviceOrientation: UIDeviceOrientation) {
        print("Orientation changed: \(self.deviceOrientationHelper.currentDeviceOrientation.rawValue)")
    }
}

extension BannerView: FSPagerViewDataSource,FSPagerViewDelegate {
    
    func numberOfItems(in pagerView: FSPagerView) -> Int {
        scanSession.scannedItems.count
    }
    
    func pagerView(_ pagerView: FSPagerView, cellForItemAt index: Int) -> FSPagerViewCell {
        
        let cell = pagerView.dequeueReusableCell(withReuseIdentifier: "cell", at: index)
        
        let item = scanSession.scannedItems[index]
        
        configure(cell: cell, at: index, with: item)
        
        return cell
    }
    
    func pagerView(_ pagerView: FSPagerView, willDisplay cell: FSPagerViewCell, forItemAt index: Int) {
        
        let item = scanSession.scannedItems[index]
        
        configure(cell: cell, at: index, with: item)
    }

    func pagerViewWillEndDragging(_ pagerView: FSPagerView, targetIndex: Int) {
        delegate?.updateSelectedIndex(index: targetIndex)
    }
    
    func pagerViewWillBeginDragging(_ pagerView: FSPagerView) {
        delegate?.willStartDragging()
    }
}

extension BannerView: FSPagerViewCellDelegate {
    
    func sizeChangedto(size: CGSize, angle: CGFloat, at: Int) {
        delegate?.sizeChanged(size: size, angle: angle, at: at)
    }
    
    func positionChangedTo(pos: CGPoint, at: Int) {
        delegate?.posChangedTo(pos: pos, at: at)
    }
    
    func preview(at: Int) {
        delegate?.preview(index: at)
    }
    
    func delete(at: Int) {
        delegate?.delete(index: at)
    }
    
    func deleteSign(at: Int) {
        delegate?.deleteSignAt(index: at)
    }
}

extension BannerView {
    
    private func configure(cell: FSPagerViewCell, at index: Int, with scannedItem: ScannedItem) {
        
        cell.sign = UIImage()
        cell.selectedStickerView1?.removeFromSuperview()
        
        cell.index = index
        cell.delegate = self
        cell.session = scannedItem
        cell.imageView?.image = scannedItem.newRenderedImage?.retrieveImage()
        
        cell.addBorder()
        
        guard let signature = scannedItem.signtureImage,
              signature != UIImage(),
              let position = scannedItem.signaturePosition
        else { return }
        
        let subviews = cell.contentView.subviews
        
        configure(cell: cell, with: scannedItem, position: position, signature: signature)
    }
    
    private func configure(cell: FSPagerViewCell, with scannedItem: ScannedItem, position: CGPoint, signature: UIImage) {
        
        cell.size = scannedItem.size
        cell.position = position
        cell.rotation = scannedItem.rotate ?? 0
        cell.sign = signature
    }
}
