//
//  EditViewController.swift
//  MiniScanner
//
//  Created by George Makhoul on 09/06/2024.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit
import LNExtensionExecutor
import NBBottomSheet
import QuickLook
import AVKit
import EPSignature
import AVFoundation
import AVKit
import ZLImageEditor

public protocol EditViewControllerDelegate: NSObjectProtocol {
    func editViewController(_ editViewController: EditViewController, finished session: MultiPageScanSession)
}

public class EditViewController: UIViewController, ScanSessionSharable, ScanSessionPrintable {
    
    // MARK: - Outlets
    @IBOutlet weak var banner: BannerView!
    @IBOutlet weak var pageIndex: UILabel!
    @IBOutlet weak var continueButton: UIPrimaryButton!
    @IBOutlet weak var directShareLabel: UILabel!
    
    @IBOutlet weak var signatureLabel: UILabel!
    @IBOutlet weak var cropLabel: UILabel!
    @IBOutlet weak var editLabel: UILabel!
    @IBOutlet weak var exportTextLabel: UILabel!
    
    @IBOutlet weak var nextPageButton: UIButton!
    @IBOutlet weak var previousPageButton: UIButton!
    
    @IBOutlet weak var editButtonsView: UIView!
    
    @IBOutlet weak var cropButtonsStackView: UIStackView!
    
    @IBOutlet weak var saveCropButton: UIPrimaryButton!
    
    @IBOutlet weak var rotateRightLabel: UILabel!
    @IBOutlet weak var autoDetectionLabel: UILabel!
    @IBOutlet weak var rotateLeftLabel: UILabel!
    
    @IBOutlet weak var shareTypeLabel: UILabel!
    @IBOutlet weak var shareTypeButton: UIButton!
    
    @IBOutlet weak var cropImageView: UIImageView!
    @IBOutlet weak var editImageView: UIImageView!
    @IBOutlet weak var exportTextImageView: UIImageView!
    @IBOutlet weak var signatureImageView: UIImageView!
    
    @IBOutlet weak var rotateLeftImageView: UIImageView!
    @IBOutlet weak var rotateRightImageView: UIImageView!
    @IBOutlet weak var autoDetectionImageView: UIImageView!
    
    @Inject var addScanSessionUseCase: AddScanSessionUseCase
    @Inject var updateScanSessionUseCase: UpdateScanSessionUseCase
    
    @Inject var userSettings: UserSettings
    
    weak public var delegate:EditViewControllerDelegate?
    var selectedFolder: Folder?
    
    var shareType: ShareType = .pdf {
        didSet {
            handleShareTypeChanged()
        }
    }
    
    var sharingViewController: UIViewController { self }
    
    // MARK: - Properties
    
    private var screenMode: EditScreenMode = .normal {
        didSet {
            handleScreenMode()
        }
    }
    
    private var scanSession: ScanSession!
    
    private var selectedIndex: Int = 0
    
    private var options:ImageScannerOptions!
    private let config = ZLImageEditorConfiguration.default()
    private var deviceOrientationHelper = DeviceOrientationHelper()
    
    private var signatureImage = UIImage()
    
    private var originalImage = UIImage()
    private var croppedImage = CIImage()
    
    private var overlayView = CDOverlayView()
    private var detectedRectangleFeature = CIRectangleFeature()
    private var sharedDetector = CDImageRectangleDetector()
    
    private var name: String = ""
    
    lazy var activityIndicator: UIActivityIndicatorView = {
        let activityIndicator = UIActivityIndicatorView(style: .large)
        activityIndicator.color = .gray
        activityIndicator.hidesWhenStopped = true
        activityIndicator.translatesAutoresizingMaskIntoConstraints = false
        return activityIndicator
    }()
    
    // MARK: - LifeCycle
    public override func viewDidLoad() {
        super.viewDidLoad()
        
        setupUI()
    }
    
    public override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        deviceOrientationHelper.startDeviceOrientationNotifier { [weak self] (deviceOrientation) in
            guard let self else { return }
            
            print("orientation: \(deviceOrientation)")
            orientationChanged(deviceOrientation: deviceOrientation)
        }
    }
    
    init(scanSession: ScanSession) {
        self.scanSession = scanSession
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder aDecoder: NSCoder) {
        scanSession = .newSession()
        fatalError("init(coder:) should not be called for this class")
    }
}

extension EditViewController {
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        banner.set(session: scanSession)
        banner.delegate = self
        
        updatePageLabel()
        
        view.addSubview(activityIndicator)
        
        activityIndicator.centerXAnchor.constraint(equalTo: view.centerXAnchor).isActive = true
        activityIndicator.centerYAnchor.constraint(equalTo: view.centerYAnchor).isActive = true
        
        continueButton.set(localized: .Continue)
        
        options = ImageScannerOptions()
        
        configImageEditor()
        
        let backButton = UIBarButtonItem(title: .back.localized, style: .plain, target: self, action: #selector(backTapped))
        navigationItem.leftBarButtonItem = backButton
        
        directShareLabel.set(localized: .directShare)
        
        shareType = userSettings.defaultFileType.shareType
        
        setupMenu()
        
        signatureLabel.text = .signature.localized
        exportTextLabel.text = .exportText.localized
        editLabel.text = .edit.localized
        cropLabel.text = .crop.localized
        
        saveCropButton.set(localized: .save)
        
        rotateLeftLabel.text = .rotateLeft.localized
        autoDetectionLabel.text = .autoDetection.localized
        rotateRightLabel.text = .rotateRight.localized
        
        pageIndex.layer.cornerRadius = 5
        
        pageIndex.clipsToBounds = true
        
        hideCropButtons()
    }
    
    private func setupColors() {
        
        view.customBackground(.base)
        
        nextPageButton.customTitleColor(.gray600)
        previousPageButton.customTitleColor(.gray600)
        
        pageIndex.customBackground(.gray100)
        pageIndex.set(color: .gray600)
        
        rotateLeftLabel.set(color: .gray600)
        autoDetectionLabel.set(color: .gray600)
        rotateRightLabel.set(color: .gray600)
        
        shareTypeLabel.set(color: .gray600)
        
        directShareLabel.set(color: .gray600)
        signatureLabel.set(color: .gray600)
        exportTextLabel.set(color: .gray600)
        editLabel.set(color: .gray600)
        cropLabel.set(color: .gray600)
        
        cropImageView.customTint(.gray600)
        editImageView.customTint(.gray600)
        exportTextImageView.customTint(.gray600)
        signatureImageView.customTint(.gray600)
        
        rotateLeftImageView.customTint(.gray600)
        rotateRightImageView.customTint(.gray600)
        autoDetectionImageView.customTint(.gray600)
    }
    
    private func setupFonts() {
        
        pageIndex.set(font: .medium(16))
        directShareLabel.set(font: .medium(16))
        
        shareTypeLabel.set(font: .semibold(16))
        
        signatureLabel.set(font: .medium(14))
        exportTextLabel.set(font: .medium(14))
        editLabel.set(font: .medium(14))
        cropLabel.set(font: .medium(14))
    }
    
    private func configImageEditor() {
        ZLImageEditorConfiguration.default()
            .imageStickerContainerView(ImageStickerContainerView())
            .fontChooserContainerView(FontChooserContainerView())
        
        config.tools.append(.draw)
        config.tools.append(.clip)
        config.tools.append(.textSticker)
        config.tools.append(.mosaic)
        config.tools.append(.filter)
        config.tools.append(.adjust)
    }
    
    private func getCurrentRotationAngle() -> Double{
        switch self.deviceOrientationHelper.currentDeviceOrientation {
        case .landscapeRight:
            return -90.0
        case .landscapeLeft:
            return 90.0
        case .portrait:
            return 0.0
        case .portraitUpsideDown:
            return 180.0
        default:
            return 0.0
        }
    }
    
    private func orientationChanged(deviceOrientation: UIDeviceOrientation) {
        print("Orientation changed: \(deviceOrientationHelper.currentDeviceOrientation.rawValue)")
    }
    
    private func updatePageLabel() {
        
        pageIndex.set(text: "\(selectedIndex + 1)/\(scanSession.scannedItems.count)")
        
        pageIndex.textAlignment = .center
        
        previousPageButton.isEnabled = selectedIndex > 0
        nextPageButton.isEnabled = selectedIndex < scanSession.itemsCount - 1
    }
}

extension EditViewController {
    
    @objc func backTapped() {
        
        switch screenMode {
        case .normal:
            handleBackInNormalMode()
        case .cropping:
            handleBackInCroppingMode()
        case .signing:
            handleBackInSigningMode()
        }
    }
    
    @objc func onTouchDoneButton() {
        
        switch screenMode {
        case .normal:
            handleDoneInNormalMode()
        case .cropping:
            handleDoneInCroppingMode()
        case .signing:
            handleDoneInSigningMode()
        }
    }
    
    // MARK: - Actions
    @IBAction func backIndex(_ sender: UIButton) {
        if banner.pagerView.currentIndex != 0 {
            
            banner.pagerView.scrollToItem(at: banner.pagerView.currentIndex - 1, animated: true)
            
            selectedIndex = banner.pagerView.currentIndex - 1
            
            updatePageLabel()
        }
    }
    
    @IBAction func nextIndex(_ sender: UIButton) {
        if banner.pagerView.currentIndex != scanSession.scannedItems.count - 1 {
            
            banner.pagerView.scrollToItem(at: banner.pagerView.currentIndex + 1, animated: true)
            
            selectedIndex = banner.pagerView.currentIndex + 1
            
            updatePageLabel()
        }
    }
    
    @IBAction func continueTapped(_ sender: UIButton) {
        
        switch screenMode {
        case .normal:
            handleContinueInNormalMode()
        case .cropping:
            handleContinueInCroppingMode()
        case .signing:
            handleContinueInSigningMode()
        }
    }
    
    @IBAction func whatsappShare(_ sender: UIButton) {
        
        if scanSession.name.isEmpty {
            scanSession.name = .getScanSessionName()
        }
        
        shareScanSession(session: scanSession, shareOption: .whatsapp)
    }
    
    @IBAction func telegramTapped(_ sender: UIButton) {
        
        if scanSession.name.isEmpty {
            scanSession.name = .getScanSessionName()
        }
        
        shareScanSession(session: scanSession, shareOption: .telegram)
    }
    
    @IBAction func gmailTapped(_ sender: UIButton) {
        
        if scanSession.name.isEmpty {
            scanSession.name = .getScanSessionName()
        }
        
        shareScanSession(session: scanSession, shareOption: .gmail)
    }
    
    @IBAction func airdropTapped(_ sender: Any) {
        
        if scanSession.name.isEmpty {
            scanSession.name = .getScanSessionName()
        }
        
        shareScanSession(session: scanSession, shareOption: .airdop)
    }
    
    @IBAction func printTapped(_ sender: UIButton) {
        
        if scanSession.name.isEmpty {
            scanSession.name = .getScanSessionName()
        }
        
        printScanSession(session: scanSession)
    }
    
    @IBAction func moreTapped(_ sender: UIButton) {
        
        if scanSession.name.isEmpty {
            scanSession.name = .getScanSessionName()
        }
        
        shareScanSession(session: scanSession, shareOption: .more)
    }
    
    // MARK: - TABBAR ACTION
    @IBAction func editButtonTapped(_ sender: UIButton) {
        displayEditImageViewController()
    }
    
    @IBAction func ocrTapped(_ sender: UIButton) {
        displayOCRViewController()
    }
    
    @IBAction func cropTapped(_ sender: UIButton) {
        displayCropOptions()
    }
    
    @IBAction func signatureTapped(_ sender: UIButton) {
        displaySignatureViewController()
    }
    
    @IBAction func saveCropButtonTapped(_ sender: Any) {
        onTouchDoneButton()
    }
    
    @IBAction func rotateRightTapped(_ sender: Any) {
        
    }
    
    @IBAction func autoDetectionTapped(_ sender: Any) {
        
    }
    
    @IBAction func rotateLeftTapped(_ sender: Any) {
        
    }
}


extension EditViewController {
    
    private func handleScreenMode() {
        switch screenMode {
        case .normal:
            
            navigationItem.rightBarButtonItem = nil
            
            banner.pagerView.isScrollEnabled = true
            hideCropButtons()
            
        case .cropping:
            
            banner.pagerView.isScrollEnabled = false
            displayCropButtons()
            
        case .signing:
            
            banner.pagerView.isScrollEnabled = true
        }
    }
}

extension EditViewController {
    
    private func handleDoneInNormalMode() {
        
    }
    
    private func handleDoneInCroppingMode() {
        
        screenMode = .normal
        
        if let editView = banner.pagerView.cellForItem(at: banner.pagerView.currentIndex) {
            editView.trashButton.isHidden = false
            editView.previewButton.isHidden = false
        }
        
        if let image = overlayView.cropImage(self.originalImage) {
            
            let index = banner.pagerView.currentIndex
            
            replacePageImage(image: image, at: index)
            
            banner.pagerView.reloadData(at: index)
            
            overlayView.removeFromSuperview()
        }
    }
    
    private func handleDoneInSigningMode() {
        
        screenMode = .normal
        
        let items = scanSession.scannedItems
        
        activityIndicator.startAnimating()
        
        Task {
            
            for (index, item) in items.enumerated() {
                
                if item.hasSignature {
                    await applySignatureToScannedItem(at: index)
                }
            }
            
            await MainActor.run {
                banner.pagerView.reloadData()
                
                activityIndicator.stopAnimating()
            }
        }
    }
    
    private func applySignatureToScannedItem(at index: Int) async {
        
        let cell = await MainActor.run {
            banner.pagerView.cellForItemat(index: banner.pagerView.currentIndex)
        }
        
        let scannedItem = await MainActor.run {
            scanSession.scannedItems[index]
        }
        
        var signedImage: UIImage?
        
        if let image = scannedItem.newRenderedImage?.retrieveImage(),
           let signature = scannedItem.signtureImage,
           let signaturePoint = scannedItem.signaturePosition,
           let signatureSize = scannedItem.size,
           let rotate = scannedItem.rotate {
            
            var rotatedSignature: UIImage = signature
            
            if rotate != 0,
               let rotatedImage = rotatedSignature.rotated(by: Measurement(value: rotate, unit: .radians)) {
                rotatedSignature = rotatedImage
            }
            
            let originPoint = CGPoint(x: signaturePoint.x - signatureSize.width / 2, y: signaturePoint.y - signatureSize.height / 2)
            
            let signatureViewRect = CGRect(origin: originPoint, size: signatureSize)
            
            if let imageViewRect = cell?.imageView?.frame {
                
                signedImage = await getSignedImage(image: image,
                                                   imageViewRect: imageViewRect,
                                                   signature: rotatedSignature,
                                                   signatureViewRect: signatureViewRect)
                
                scannedItem.removeSign()
                
                if let signedImage {
                    
                    replacePageImage(image: signedImage, at: index)
                }
            }
        }
    }
}

extension EditViewController {
    
    private func handleContinueInNormalMode() {
        displayShareSheet()
    }
    
    private func handleContinueInCroppingMode() {
        displayDiscardChangesDialog()
    }
    
    private func handleContinueInSigningMode() {
        displayDiscardChangesDialog()
    }
}
extension EditViewController {
    
    private func handleBackInNormalMode() {
        navigationController?.popViewController(animated: true)
    }
    
    private func handleBackInCroppingMode() {
        displayDiscardChangesDialog()
    }
    
    private func handleBackInSigningMode() {
        displayDiscardChangesDialog()
    }
    
    private func displayDiscardChangesDialog() {
        
        let alertController = UIAlertController(title: .alert.localized, message: .discardChangesMsg.localized, preferredStyle: .alert)
        
        let okAction = UIAlertAction(title: .discard.localized, style: .destructive) {  [weak self] (action) in
            guard let self else { return }
            
            dismiss(animated: true) {
                
                switch self.screenMode {
                case .normal:
                    break
                case .cropping:
                    self.discardCropChanges()
                case .signing:
                    self.discardSignatureChanges()
                }
                
                self.banner.pagerView.reloadData()
                
                self.navigationItem.rightBarButtonItem = nil
                
                self.screenMode = .normal
            }
        }
        
        let cancelAction = UIAlertAction(title: .cancel.localized, style: .cancel) {  [weak self] (action) in
            guard let self else { return }
            
            dismiss(animated: true)
        }
        
        alertController.addAction(okAction)
        alertController.addAction(cancelAction)
        
        present(alertController, animated: true, completion: nil)
    }
    
    private func discardSignatureChanges() {
        
        let items = scanSession.scannedItems
        
        for item in items {
            item.removeSign()
        }
    }
    
    private func discardCropChanges() {
        
        overlayView.removeFromSuperview()
    }
}

extension EditViewController {
    
    private func displayEditImageViewController() {
        
        if let image = scanSession.scannedItems[banner.pagerView.currentIndex].newRenderedImage?.retrieveImage() {
            
            ZLEditImageViewController.showEditImageVC(parentVC: self, image: image, editModel: nil) { [weak self] resImage, editModel in
                guard let self else { return }
                
                let index = banner.pagerView.currentIndex
                
                replacePageImage(image: resImage, at: index)
                
                banner.pagerView.reloadData(at: index)
            }
        }
    }
    
    private func displayShareSheet() {
        
        let popup = ShareSheetViewController()
        
        popup.session = scanSession
        popup.isShareShown = false
        popup.delegate = self
        
        let configuration = NBBottomSheetConfiguration(animationDuration: 0.4, sheetSize: .fixed(180))
        
        let bottomSheetController = NBBottomSheetController(configuration: configuration)
        
        bottomSheetController.present(popup, on: self)
    }
    
    private func displaySignatureViewController() {
        
        let signatureVC = SignatureViewController(signatureDelegate: self)
        
        navigationController?.pushViewController(signatureVC, animated: true)
    }
    
    private func displayOCRViewController() {
        if let image = scanSession.scannedItems[banner.pagerView.currentIndex].newRenderedImage?.retrieveImage() {
            
            let controller = OCRViewController(ocrImage: image)
            
            navigationController?.pushViewController(controller, animated: true)
        }
    }
    
    private func displayCropOptions() {
        
        if let origImage = scanSession.scannedItems[banner.pagerView.currentIndex].newRenderedImage?.retrieveImage() {
            
            originalImage = origImage
        }
        
        DispatchQueue.main.async {
            let rect = AVMakeRect(aspectRatio: self.originalImage.size, insideRect: self.banner.bounds)
            if let editView = self.banner.pagerView.cellForItem(at: self.banner.pagerView.currentIndex) {
                editView.trashButton.isHidden = true
                editView.previewButton.isHidden = true
                editView.contentView.frame = rect
            }
        }
        
        screenMode = .cropping
        
        let cropButton = UIBarButtonItem(title: .crop.localized, style: .plain, target: self, action: #selector(self.onTouchDoneButton))
        
        navigationItem.rightBarButtonItem = cropButton
        
        perform(#selector(setUpCropView), with: self, afterDelay: 0.2)
    }
    
    private func displayCropButtons() {
        editButtonsView.isHidden = true
        cropButtonsStackView.isHidden = false
    }
    
    private func hideCropButtons() {
        editButtonsView.isHidden = false
        cropButtonsStackView.isHidden = true
    }
    
    private func displayAddSignatureDialog(signatureImage: UIImage) {
        
        let alertController = UIAlertController(title: "", message: .signMsg.localized, preferredStyle: .alert)
        
        let allAction = UIAlertAction(title: .allDocument.localized, style: .default) { [weak self] (action) in
            guard let self else { return }
            
            addSignatureToAllPages(signatureImage: signatureImage)
            
            banner.signAllDoc = true
            banner.pagerView.reloadData()
            
            addAddButton()
            
            dismiss(animated: true)
        }
        
        let currentAction = UIAlertAction(title: .currentPage.localized, style: .default) { [weak self] (action) in
            guard let self else { return }
            
            banner.pagerView.isScrollEnabled = false
            
            addSignatureToCurrentPage(signatureImage: signatureImage)
            
            addAddButton()
            
            dismiss(animated: true)
        }
        
        let cancelAction = UIAlertAction(title: .cancel.localized, style: .cancel) { [weak self] (action) in
            guard let self else { return }
            
            screenMode = .normal
            
            dismiss(animated: true)
        }
        
        alertController.addAction(allAction)
        alertController.addAction(currentAction)
        alertController.addAction(cancelAction)
        
        present(alertController, animated: true, completion: nil)
    }
    
    private func addAddButton() {
        
        let addButton = UIBarButtonItem(title: .add.localized, style: .plain, target: self, action: #selector(self.onTouchDoneButton))
        
        navigationItem.rightBarButtonItem = addButton
    }
    
    private func addSignatureToCurrentPage(signatureImage: UIImage) {
        let index = banner.pagerView.currentIndex
        
        if let initPosition = banner.pagerView.cellForItem(at: index)?.contentView.center {
            
            addSignatureToPage(signatureImage: signatureImage, at: index, position: initPosition)
        }
        
        banner.signAllDoc = false
        banner.pagerView.reloadData()
    }
    
    private func addSignatureToAllPages(signatureImage: UIImage) {
        
        let index = banner.pagerView.currentIndex
        
        let items = scanSession.scannedItems
        
        if let initPosition = banner.pagerView.cellForItem(at: index)?.contentView.center {
            
            for (index, _) in items.enumerated() {
                
                addSignatureToPage(signatureImage: signatureImage, at: index, position: initPosition)
            }
        }
    }
    
    private func addSignatureToPage(signatureImage: UIImage, at index: Int, position: CGPoint) {
        
        let scale = 125 / signatureImage.width
        let newHeight = signatureImage.height * scale
        let size = CGSize(width: 125, height: newHeight)
        
        scanSession.scannedItems[index].addSign(signature: signatureImage, position: position, size: size, rotate: 0)
    }
    
    private func replacePageImage(image: UIImage, at index: Int) {
        
        let scannedItem = ScannedItem(originalImage: image, renderImage: image, quad: nil)
        
        scannedItem.colorOption = self.options.defaultColorRenderOption
        scannedItem.rotation = getCurrentRotationAngle()
        
        scanSession = scanSession.replaceItemAndGet(item: scannedItem, at: index)
    }
    
    private func setupMenu() {
        
        shareTypeButton.showsMenuAsPrimaryAction = true
        
        let pdf = UIAction(title: .pdf.localized) { _ in
            self.shareType = .pdf
        }
        
        let jpg = UIAction(title: .jpeg.localized) { _ in
            self.shareType = .jpeg
        }
        
        let menu = UIMenu(title: "", children: [pdf, jpg])
        
        if #available(iOS 16.0, *) {
            menu.preferredElementSize = .small
        }
        
        shareTypeButton.menu = menu
    }
    
    private func handleShareTypeChanged() {
        shareTypeLabel.set(localized: shareType.displayName)
    }
    
    private func displayImagePreviewSheet(at index: Int) {
        
        let previewController = ScanSessionPreviewViewController(scanSession: scanSession, selectedIndex: index)
        
        previewController.modalPresentationStyle = .fullScreen
        
        present(previewController, animated: true)
    }
}

// MARK: - BannerViewDelegate
extension EditViewController: BannerViewDelegate {
    
    func sizeChanged(size: CGSize, angle: CGFloat, at: Int) {
        scanSession.scannedItems[at].size = size
        scanSession.scannedItems[at].rotate = angle
        
        banner.pagerView.reloadData()
    }
    
    func posChangedTo(pos: CGPoint, at: Int) {
        scanSession.scannedItems[at].signaturePosition = pos
        
        banner.pagerView.reloadData()
    }
    
    func deleteSignAt(index: Int) {
        scanSession.scannedItems[index].removeSign()
        
        let signaturesCount = scanSession.scannedItems.filter { $0.hasSignature }.count
        
        if signaturesCount == 0 {
            screenMode = .normal
        }
        
        banner.pagerView.reloadData()
    }
    
    func willStartDragging() {
        
    }
    
    func update(session: ScanSession) {
        scanSession = session
    }
    
    func preview(index: Int) {
        displayImagePreviewSheet(at: index)
    }
    
    func delete(index: Int) {
        let alertController = UIAlertController(title: .alert.localized, message: .deleteImageMsg.localized, preferredStyle: .alert)
        
        let okAction = UIAlertAction(title: .yes.localized, style: .default) { [weak self] (action) in
            guard let self else { return }
            
            scanSession.removeItem(at: index)
            
            if index != 0 {
                selectedIndex = index - 1
            }
            
            banner.pagerView.reloadData()
            
            if scanSession.scannedItems.count != 0 {
                
                updatePageLabel()
            } else {
                
                dismiss(animated: true) {
                    self.navigationController?.popViewController(animated: true)
                }
            }
            
            dismiss(animated: true)
        }
        
        let cancelAction = UIAlertAction(title: .cancel.localized, style: .cancel) {  [weak self] (action) in
            guard let self else { return }
            
            dismiss(animated: true)
        }
        
        alertController.addAction(okAction)
        alertController.addAction(cancelAction)
        
        present(alertController, animated: true, completion: nil)
    }
    
    func updateSelectedIndex(index: Int) {
        selectedIndex = index
        updatePageLabel()
    }
}

// MARK: - ShareSheetViewControllerDelegate

extension EditViewController: ShareSheetViewControllerDelegate {
    
    func shareOptionTapped(name: String, shareOption: ShareOption, shareType: ShareType) {
        
    }
    
    func printTapped(shareType: ShareType) {
        
    }
    
    func saveTapped(name: String) {
        self.name = name
        saveScanSession(name: name)
    }
    
    private func saveScanSession(name: String) {
        
        if let _ = scanSession.folderId {
            updateScanSession()
        }
        else {
            saveNewScanSession(name: name)
        }
    }
    
    private func saveNewScanSession(name: String) {
        
        if let selectedFolder {
            
            if selectedFolder != .allDocuments {
                addScanSession(name: name, to: selectedFolder)
            }
            else {
                displayFoldersViewSheet()
            }
        }
    }
    
    private func updateScanSession() {
        updateScanSessionUseCase.execute(with: scanSession)
        
        navigationController?.popToRootViewController(animated: true)
    }
    
    private func addScanSession(name: String, to folder: Folder) {
        
        if !name.isEmpty {
            scanSession.name = name
        }
        else {
            scanSession.name = .getScanSessionName()
        }
        
        scanSession.folderId = folder.id
        
        scanSession.creationDate = .now
        
        addScanSessionUseCase.execute(with: scanSession, to: folder)
        
        navigationController?.popToRootViewController(animated: true)
    }
    
    private func displayFoldersViewSheet() {
        
        let foldersViewController = FoldersViewController()
        
        foldersViewController.delegate = self
        
        present(foldersViewController, animated: true)
    }
}

extension EditViewController: FoldersViewControllerDelegate {
    
    func dismissed(_ foldersViewController: FoldersViewController, folder: Folder?) {
        
        if let folder {
            addScanSession(name: name, to: folder)
        }
    }
}

// MARK: - EPSignatureDelegate
extension EditViewController: SignatureDelegate {
    
    func epSignature(didSign signatureImage: UIImage, boundingRect: CGRect) {
        
        screenMode = .signing
        
        self.signatureImage = signatureImage
        
        if scanSession.scannedItems.count > 1 {
            
            displayAddSignatureDialog(signatureImage: signatureImage)
        } else {
            
            addSignatureToCurrentPage(signatureImage: signatureImage)
            
            addAddButton()
        }
    }
}

// MARK: - setupCropView
extension EditViewController {
    
    @objc func setUpCropView()
    {
        sharedDetector = CDImageRectangleDetector.shared()
        
        let array = NSArray.init(array: (sharedDetector.highAccuracyRectangleDetector()?.features(in: croppedImage))!)
        
        if array.count > 0 {
            
            detectedRectangleFeature = sharedDetector.biggestRectangle(inRectangles: (array as! [Any]))
            
            croppedImage = sharedDetector.drawHighlightOverlay(forPoints: croppedImage, topLeft: detectedRectangleFeature.topLeft, topRight: detectedRectangleFeature.topRight, bottomLeft: detectedRectangleFeature.bottomLeft, bottomRight: detectedRectangleFeature.bottomRight)
            
            banner.pagerView.isUserInteractionEnabled = true
            
            magnetActivated()
        } else {
            
            banner.pagerView.isUserInteractionEnabled = true
            
            magnetDeActivated()
        }
    }
    
    func magnetActivated() {
        
        if let imageView = banner.pagerView.cellForItem(at: banner.pagerView.currentIndex)?.contentView {
            
            overlayView = CDOverlayView.init(frame: imageView.bounds)
            overlayView.translatesAutoresizingMaskIntoConstraints = false
            imageView.addSubview(overlayView)
            
            let absoluteHeight: CGFloat = originalImage.size.height / imageView.frame.size.height
            let absoluteWidth: CGFloat = originalImage.size.width / imageView.frame.size.width
            
            overlayView.absoluteHeight = absoluteHeight
            overlayView.absoluteWidth = absoluteWidth
            
            let detectedTopLeftPoint = CGPoint(x: detectedRectangleFeature.topLeft.x / CGFloat(absoluteWidth), y: imageView.frame.size.height - detectedRectangleFeature.topLeft.y / CGFloat(absoluteHeight))
            let detectedTopRightPoint = CGPoint(x: detectedRectangleFeature.topRight.x / CGFloat(absoluteWidth), y: imageView.frame.size.height - detectedRectangleFeature.topRight.y / CGFloat(absoluteHeight))
            let detectedBottomLeftPoint = CGPoint(x: detectedRectangleFeature.bottomLeft.x / CGFloat(absoluteWidth), y: imageView.frame.size.height - detectedRectangleFeature.bottomLeft.y / CGFloat(absoluteHeight))
            let detectedBottomRightPoint = CGPoint(x: detectedRectangleFeature.bottomRight.x / CGFloat(absoluteWidth), y: imageView.frame.size.height - detectedRectangleFeature.bottomRight.y / CGFloat(absoluteHeight))
            
            overlayView.topLeftPath = overlayView.correct(detectedTopLeftPoint)
            overlayView.topRightPath = overlayView.correct(detectedTopRightPoint)
            overlayView.bottomLeftPath = overlayView.correct(detectedBottomLeftPoint)
            overlayView.bottomRightPath = overlayView.correct(detectedBottomRightPoint)
            
            overlayView.initializeSubView()
        }
    }
    
    func magnetDeActivated() {
        
        if let imageView = banner.pagerView.cellForItem(at: banner.pagerView.currentIndex)?.contentView {
            
            overlayView = CDOverlayView.init(frame: imageView.bounds)
            overlayView.translatesAutoresizingMaskIntoConstraints = false
            imageView.addSubview(overlayView)
            
            overlayView.initializeSubView()
            
            let margin: CGFloat = 60.0
            let absoluteHeight: CGFloat = originalImage.size.height / imageView.frame.size.height
            let absoluteWidth: CGFloat = originalImage.size.width / imageView.frame.size.width
            
            overlayView.absoluteHeight = absoluteHeight
            overlayView.absoluteWidth = absoluteWidth
            
            overlayView.topLeftPath = CGPoint(x: CGFloat(margin), y: CGFloat(margin))
            overlayView.topRightPath = CGPoint(x: CGFloat(imageView.frame.width - margin), y: CGFloat(margin))
            overlayView.bottomLeftPath = CGPoint(x: CGFloat(margin), y: CGFloat(imageView.frame.height - margin))
            overlayView.bottomRightPath = CGPoint(x: CGFloat(imageView.frame.width - margin), y: CGFloat(imageView.frame.height - margin))
            
            overlayView.initializeSubView()
        }
    }
}

extension EditViewController {
    
    func getSignedImage(image: UIImage,
                        imageViewRect: CGRect,
                        signature: UIImage,
                        signatureViewRect: CGRect) async -> UIImage? {
        
        var signedImage: UIImage?
        
        let imageRect = CGRect(x: 0, y: 0, width: image.width, height: image.height)
        
        let scaledImageRect = AVMakeRect(aspectRatio: image.size,
                                         insideRect: imageViewRect)
        
        let widthScale: CGFloat = imageRect.width / scaledImageRect.width
        let heightScale: CGFloat = imageRect.height / scaledImageRect.height
        
        let newWidth = widthScale * signatureViewRect.width
        let newHeight = heightScale * signatureViewRect.height
        
        let xScale: CGFloat = image.width / imageViewRect.width
        let yScale: CGFloat = image.height / imageViewRect.height
        
        let newX = xScale * signatureViewRect.minX
        let newY = yScale * signatureViewRect.minY
        
        let newSignatureRect = CGRect(x: newX, y: newY, width: newWidth, height: newHeight)
        
        let format = UIGraphicsImageRendererFormat()
        format.scale = 1
        
        let renderer = UIGraphicsImageRenderer(size: imageRect.size)
        
        signedImage = renderer.image { context in
            
            image.draw(in: imageRect)
            
            signature.draw(in: newSignatureRect)
        }
        
        return signedImage
    }
}
