//
//  OCRViewController.swift
//  MiniScanner
//
//  Created by George Makhoul on 22/06/2024.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit
import VisionKit
import Vision
import Toast

class OCRViewController: UIViewController {
    // MARK: - Outlets
    @IBOutlet weak var ocrTextView: UITextView!
    @IBOutlet weak var directShareLabel: UILabel!
    
    // MARK: - Properties
    var ocrImage = UIImage()
    var recognizedText = ""
    
    var textRecognitionRequest = VNRecognizeTextRequest()
    var capturedimage = VNImageBasedRequest()
    
    lazy private var activityIndicator: UIActivityIndicatorView = {
        let activityIndicator = UIActivityIndicatorView(style: .large)
        activityIndicator.color = .gray
        activityIndicator.hidesWhenStopped = true
        activityIndicator.translatesAutoresizingMaskIntoConstraints = false
        return activityIndicator
    }()
    
    // MARK: - LifeCycle
    override func viewDidLoad() {
        super.viewDidLoad()
        
        recognizeText()
        
        let handler = VNImageRequestHandler(cgImage: ocrImage.cgImage!, options: [:])
        do {
            try handler.perform([textRecognitionRequest])
        } catch {
            print(error)
        }
        
        setupUI()
    }
    
    public init(ocrImage: UIImage){
        self.ocrImage = ocrImage
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) should not be called for this class")
    }
    
    private func recognizeText() {
        textRecognitionRequest = VNRecognizeTextRequest(completionHandler: { (request, error) in
            if let results = request.results, !results.isEmpty {
                if let requestResults = request.results as? [VNRecognizedTextObservation] {
                    self.recognizedText = ""
                    for observation in requestResults {
                        guard let candidiate = observation.topCandidates(1).first else { return }
                        self.recognizedText += candidiate.string
                        self.recognizedText += "\n"
                    }
                    
                    self.ocrTextView.text = self.recognizedText
                }
            }
        })
        textRecognitionRequest.recognitionLevel = .accurate
        textRecognitionRequest.usesLanguageCorrection = false
        textRecognitionRequest.customWords = ["@gmail.com", "@outlook.com", "@yahoo.com", "@icloud.com"]
    }
    
    private func setupUI() {
        
        setupColors()
        
        setupFonts()
        
        view.addSubview(activityIndicator)
        
        activityIndicator.centerXAnchor.constraint(equalTo: view.centerXAnchor).isActive = true
        activityIndicator.centerYAnchor.constraint(equalTo: view.centerYAnchor).isActive = true
        
        directShareLabel.set(localized: .directShare)
        
        ocrTextView.borderWidth = 1
        ocrTextView.cornerRadiuss = 5
    }
    
    private func setupColors() {
        
        view.customBackground(.base)
        
        directShareLabel.set(color: .gray600)
        
        ocrTextView.customBorderColor(.gray450)
        
        ocrTextView.textColor = ColorStyle.gray600.uiColor
    }
    
    private func setupFonts() {
        ocrTextView.font = .regular(16)
    }
}

extension OCRViewController {
    
    @IBAction func whatsappShare(_ sender: UIButton) {
        shareText(with: ShareOptions.whatsapp)
    }
    
    @IBAction func telegramTapped(_ sender: UIButton) {
        shareText(with: ShareOptions.telegram)
    }
    
    @IBAction func gmailTapped(_ sender: UIButton) {
        shareText(with: ShareOptions.gmail)
    }
    
    @IBAction func airdropTapped(_ sender: UIButton) {
        shareText(with: ShareOptions.airdop)
    }
    
    @IBAction func copyTapped(_ sender: UIButton) {
        copyText()
    }
    
    @IBAction func moreTapped(_ sender: UIButton) {
        shareText()
    }
}

extension OCRViewController {
    
    func copyText() {
        UIPasteboard.general.string = self.ocrTextView.text
        self.view.makeToast(.copiedToClipboard.localized)
    }
    
    private func shareText(with extensionBundleIdentifier: String) {
        if let ocrText = ocrTextView.text {
            activityIndicator.startAnimating()
            
            Task {
                await self.directShareWith(extensionBundleIdentifier: extensionBundleIdentifier, payload: [ocrText],
                                           view: self)
            }
            
            activityIndicator.stopAnimating()
        }
    }
    
    private func shareText() {
        if let ocrText = ocrTextView.text {
            let shareController = UIActivityViewController(activityItems: [ocrText],
                                                           applicationActivities: nil)
            
            present(shareController, animated: true, completion: nil)
        }
    }
}
