//
//  SignatureViewController.swift
//  MiniScanner
//
//  Created by Mustafa Merza on 8/15/24.
//  Copyright © 2024 AppsNectar. All rights reserved.
//

import UIKit
import NBBottomSheet
import EPSignature

protocol SignatureDelegate: AnyObject {
    func epSignature(didSign signatureImage: UIImage, boundingRect: CGRect)
}

class SignatureViewController: UIViewController {
    
    @IBOutlet weak var signatureBackgroundView: UIView!
    @IBOutlet weak var signatureView: EPSignatureView!
    
    @IBOutlet weak var deleteSignatureLabel: UILabel!
    @IBOutlet weak var rotateRightLabel: UILabel!
    @IBOutlet weak var rotateLeftLabel: UILabel!
    
    @IBOutlet weak var defaultSignatureSwitch: CustomSwitch!
    @IBOutlet weak var saveDefaultSignatureLabel: UILabel!
    
    @IBOutlet weak var saveToDefaultsView: UIStackView!
    @IBOutlet weak var saveButton: UIPrimaryButton!
    @IBOutlet weak var defaultSignatureButton: UISecondaryButton!
    
    @IBOutlet weak var rotateRightImageView: UIImageView!
    
    @IBOutlet weak var rotateLeftImageView: UIImageView!
    
    weak var signatureDelegate: SignatureDelegate?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
    }
    
    init(signatureDelegate: SignatureDelegate) {
        self.signatureDelegate = signatureDelegate
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    private lazy var userDefaultSignatureExists: Bool = {
        defaultSignatureExists()
    }()
    
    private func setupUI() {
        
        setupColors()
        setupFonts()
        
        signatureBackgroundView.borderWidth = 1
        signatureBackgroundView.cornerRadiuss = 5
        
        saveButton.set(localized: .save)
        
        defaultSignatureButton.set(localized: .defaultSignature)
        
        saveDefaultSignatureLabel.set(localized: .saveSignature)
        
        rotateLeftImageView.image = .rotateSigLeft
        rotateRightImageView.image = .rotateSigRight
        
        deleteSignatureLabel.text = .deleteSignature.localized
        rotateLeftLabel.text = .rotateLeft.localized
        rotateRightLabel.text = .rotateRight.localized
        
        defaultSignatureSwitch.isOn = false
        
        defaultSignatureSwitch.padding = 4
        
        defaultSignatureSwitch.cornerRadius = 0.5
        defaultSignatureSwitch.thumbCornerRadius = 0.5
        defaultSignatureSwitch.thumbTintColor = .white
        defaultSignatureSwitch.animationDuration = 0.5
        defaultSignatureSwitch.thumbSize = CGSize(width: 22, height: 22)
        
        setupDefaultSignatureViews()
    }
    
    private func setupColors() {
        
        view.customBackground(.base)
        
        signatureBackgroundView.borderColor = ColorStyle.gray450.uiColor
        
        rotateLeftImageView.customTint(.gray600)
        rotateRightImageView.customTint(.gray600)
        
        deleteSignatureLabel.set(color: .gray600)
        rotateLeftLabel.set(color: .gray600)
        rotateRightLabel.set(color: .gray600)
        
        saveDefaultSignatureLabel.set(color: .gray600)
        
        defaultSignatureSwitch.onTintColor = ColorStyle.primary.uiColor
        
        defaultSignatureSwitch.offTintColor = ColorStyle.gray350.uiColor
    }
    
    private func setupFonts() {
        
        deleteSignatureLabel.set(font: .medium(16))
        rotateLeftLabel.set(font: .medium(16))
        rotateRightLabel.set(font: .medium(16))
        
        saveDefaultSignatureLabel.set(font: .medium(20))
    }
    
    private func setupDefaultSignatureViews() {
        saveToDefaultsView.isHidden = userDefaultSignatureExists
        defaultSignatureButton.isHidden = !userDefaultSignatureExists
    }
    
    // MARK: - Button Actions
    
    @IBAction func onTouchDefaultSignatureButton(_ sender: Any) {
        
        let sheet = DefaultSignatureSheetViewController(delegate: self)
        
        let viewHeight = sheet.view.frame.height
        
        let configuration = NBBottomSheetConfiguration(animationDuration: 0.4, sheetSize: .fixed(viewHeight))
        
        let bottomSheetController = NBBottomSheetController(configuration: configuration)
        
        bottomSheetController.present(sheet, on: self)
    }
    
    @IBAction func onTouchSaveButton(_ sender: Any) {
        if let signature = signatureView.getSignatureAsImage() {
            
            if defaultSignatureSwitch.isOn {
                
                let docPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first
                
                let filePath = (docPath! as NSString).appendingPathComponent("sig.data")
                
                signatureView.saveSignature(filePath)
            }
            
            navigationController?.popViewController(animated: true)
            
            signatureDelegate?.epSignature(didSign: signature, boundingRect: signatureView.getSignatureBoundsInCanvas())
        } else {
            
            let alertController = UIAlertController(title: .didntSignAlertTitle.localized,
                                                    message: .didntSignAlertMessage.localized,
                                                    preferredStyle: .alert)
            
            let okAction = UIAlertAction(title: .ok.localized,
                                         style: .default,
                                         handler: nil)
            
            alertController.addAction(okAction)
            
            present(alertController, animated: true)
        }
    }
    
    @IBAction func rotateRightTapped(_ sender: Any) {
        signatureView.rotate(degrees: 90)
    }
    
    @IBAction func rotateLeftTapped(_ sender: Any) {
        signatureView.rotate(degrees: -90)
    }
    
    @IBAction func deleteSignatureTapped(_ sender: Any) {
        signatureView.clear()
    }
    
    private func defaultSignatureExists() -> Bool {
        let docPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first
        let filePath = (docPath! as NSString).appendingPathComponent("sig.data")
        
        return FileManager.default.fileExists(atPath: filePath)
    }
    
    override open func didRotate(from fromInterfaceOrientation: UIInterfaceOrientation) {
        signatureView.reposition()
    }
}

extension SignatureViewController: DefaultSignatureSheetViewControllerDelegate {
    
    func useDefaultSignature() {
        let docPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first
        let filePath = (docPath! as NSString).appendingPathComponent("sig.data")
        
        userDefaultSignatureExists = true
        
        setupDefaultSignatureViews()
        
        signatureView.loadSignature(filePath)
    }
    
    func deleteDefaultSignature() {
        userDefaultSignatureExists = false
        
        setupDefaultSignatureViews()
        
        signatureView.removeSignature()
    }
}
