//
//  ImageScannerController.swift
//  WeScan
//
//  Created by Boris Emorine on 2/12/18.
//  Copyright © 2018 WeTransfer. All rights reserved.
//

import UIKit
import AVFoundation

public struct ImageScannerOptions {
    public let scanMultipleItems:Bool
    public let allowAutoScan:Bool
    public let allowTapToFocus:Bool
    public let defaultColorRenderOption:ScannedItemColorOption
    
    public init(scanMultipleItems:Bool = true, allowAutoScan:Bool = true, allowTapToFocus:Bool = true, defaultColorRenderOption:ScannedItemColorOption = .color) {
        self.scanMultipleItems = scanMultipleItems
        self.allowAutoScan = allowAutoScan
        self.allowTapToFocus = allowTapToFocus
        self.defaultColorRenderOption = defaultColorRenderOption
    }
}

public protocol ImageScannerControllerDelegate: NSObjectProtocol {
    func imageScannerController(_ scanner: ImageScannerController,
                                didFinishWithSession session:MultiPageScanSession)
    
    func imageScannerControllerDidCancel(_ scanner: ImageScannerController)
    
    func imageScannerController(_ scanner: ImageScannerController,
                                didFailWithError error: Error)
}

public final class ImageScannerController: UINavigationController {
    // MARK: - Properties
    weak public var imageScannerDelegate: ImageScannerControllerDelegate?
    
    // MARK: - Life Cycle
    internal let blackFlashView: UIView = {
        let view = UIView()
        view.backgroundColor = UIColor(white: 0.0, alpha: 0.5)
        view.isHidden = true
        view.translatesAutoresizingMaskIntoConstraints = false
        return view
    }()
    
    public required init(options: ImageScannerOptions?, isAddedToPDF: Bool = false) {
        let scannerViewController = ScannerViewController(scanSession: nil, options: options, isAddedToPDF: isAddedToPDF)
        super.init(rootViewController: scannerViewController)
        scannerViewController.delegate = self
        navigationBar.tintColor = .white
        navigationBar.isTranslucent = false
        self.view.addSubview(blackFlashView)
        setupConstraints()
    }
    
    public override init(nibName nibNameOrNil: String?, bundle nibBundleOrNil: Bundle?) {
        super.init(nibName: nibNameOrNil, bundle: nibBundleOrNil)
    }
    
    required public init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    private func setupConstraints() {
        let blackFlashViewConstraints = [
            blackFlashView.topAnchor.constraint(equalTo: view.topAnchor),
            blackFlashView.leadingAnchor.constraint(equalTo: view.leadingAnchor),
            view.bottomAnchor.constraint(equalTo: blackFlashView.bottomAnchor),
            view.trailingAnchor.constraint(equalTo: blackFlashView.trailingAnchor)
        ]
        
        NSLayoutConstraint.activate(blackFlashViewConstraints)
    }
    
    override public var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return .portrait
    }
    
}

extension ImageScannerController: ScannerViewControllerDelegate{
    
    func scannerViewController(_ scannerViewController: ScannerViewController,
                               didFail withError: Error) {
        self.imageScannerDelegate?.imageScannerController(self, didFailWithError: withError)
    }
    
    func scannerViewControllerDidCancel(_ scannerViewController: ScannerViewController) {
        self.imageScannerDelegate?.imageScannerControllerDidCancel(self)
    }
    
    func scannerViewController(_ scannerViewController: ScannerViewController,
                               reviewItems inSession: MultiPageScanSession) {
        
        let multipageScanViewController = MultiPageScanSessionViewController(scanSession: inSession)
        multipageScanViewController.delegate = self
        self.pushViewController(multipageScanViewController, animated: true)
    }
}

extension ImageScannerController: MultiPageScanSessionViewControllerDelegate {
    
    public func multiPageScanSessionViewController(_ multiPageScanSessionViewController: MultiPageScanSessionViewController,
                                                   finished session: MultiPageScanSession) {
        self.imageScannerDelegate?.imageScannerController(self, didFinishWithSession: session)
    }
}
