//
//  MultiPageScanSession.swift
//  WeScan
//
//  Created by Enrique Rodriguez on 07/01/2019.
//  Copyright © 2019 WeTransfer. All rights reserved.
//

import Foundation
import UIKit

public enum ScannedItemColorOption: Codable{
    case color
    case grayscale
}

/// Data structure containing information about one page scanned
public class ScannedItem: Codable, Identifiable {
    
    public var id: UUID = UUID()
    var isSelected: Bool = false
    var type: ScannedItemType = .temp
    
    var folderUrl: URL {
        switch type {
        case .temp:
            URL(fileURLWithPath: NSTemporaryDirectory())
        case .local:
            FileManager.default.urls(for: .documentDirectory, in: .userDomainMask)[0]
        }
    }
    
    /// The original image taken by the user, prior to the cropping applied by WeScan.
    var originalImage:URL?
    
    var newOriginalImage: URL? {
        let name = originalImage?.lastPathComponent ?? ""
        return folderUrl.appendingPathComponent(name)
    }
    
    /// The detected rectangle which was used to generate the `scannedImage`.
    var quad:Quadrilateral?
    
    /// The rotation applied to the resulting image
    var rotation:Double = 0.0
    
    /// The color preference for the output of this image
    var colorOption:ScannedItemColorOption = .color
    
    /// The deskewed and cropped orignal image using the detected rectangle, without any filters.
    var renderedImage:URL? = nil
    
    var newRenderedImage: URL? {
        let name = renderedImage?.lastPathComponent ?? ""
        return folderUrl.appendingPathComponent(name)
    }
    
    var hasSignature: Bool { signtureImage != nil }
    
    var signtureImage: UIImage?
    var signaturePosition: CGPoint?
    var size: CGSize?
    var rotate: CGFloat?
    
    init(originalImage:UIImage, quad:Quadrilateral? = nil, colorOption:ScannedItemColorOption = .color, type: ScannedItemType = .temp) {
        self.type = type
        self.originalImage = filePath().saveImage(image: originalImage.withFixedOrientation())
        self.quad = quad
        self.colorOption = colorOption
    }
    
    init(originalImage:UIImage, renderImage: UIImage, quad:Quadrilateral? = nil, colorOption:ScannedItemColorOption = .color, type: ScannedItemType = .temp) {
        self.type = type
        self.originalImage = filePath().saveImage(image: originalImage.withFixedOrientation())
        self.quad = quad
        self.colorOption = colorOption
        self.renderedImage = filePath().saveImage(image: renderImage.withFixedOrientation())
    }
    
    public func render(completion: @escaping (_ image:URL?)->Void){
        ScannedItemRenderer().render(scannedItem: self) { (image) in
            self.renderedImage = self.filePath().saveImage(image: image?.withFixedOrientation())
            completion(self.renderedImage)
        }
    }
    
    public func getRender(completion: @escaping (_ image:UIImage?)->Void){
        ScannedItemRenderer().render(scannedItem: self) { (image) in
            self.renderedImage = self.filePath().saveImage(image: image?.withFixedOrientation())
            completion(image)
        }
    }
    
    public func addSign(signature: UIImage, position: CGPoint, size: CGSize, rotate: CGFloat) {
        self.signtureImage = signature
        self.signaturePosition = position
        self.size = size
        self.rotate = rotate
    }
    
    public func removeSign() {
        self.signtureImage = nil
        self.signaturePosition = nil
        self.size = nil
        self.rotate = nil
    }
    
    enum CodingKeys: CodingKey {
        case originalImage, quad, rotation, colorOption, renderedImage, signaturePosition, size, rotate
    }
    
    public required init(from decoder: Decoder) throws {
        let container = try decoder.container(keyedBy: CodingKeys.self)
        
        originalImage = try? container.decode(URL.self, forKey: .originalImage)
        rotation = try container.decode(Double.self, forKey: .rotation)
        colorOption = try container.decode(ScannedItemColorOption.self, forKey: .colorOption)
        renderedImage = try? container.decode(URL.self, forKey: .renderedImage)
        signaturePosition = try? container.decode(CGPoint.self, forKey: .signaturePosition)
        size = try? container.decode(CGSize.self, forKey: .size)
        rotate = try? container.decode(CGFloat.self, forKey: .rotate)
        
        // Custom decoding for `quad` here
    }
    
    public func encode(to encoder: Encoder) throws {
        var container = encoder.container(keyedBy: CodingKeys.self)
        
        try container.encode(originalImage, forKey: .originalImage)
        try container.encode(rotation, forKey: .rotation)
        try container.encode(colorOption, forKey: .colorOption)
        try container.encode(renderedImage, forKey: .renderedImage)
        try container.encode(signaturePosition, forKey: .signaturePosition)
        try container.encode(size, forKey: .size)
        try container.encode(rotate, forKey: .rotate)
        
        // Custom encoding for `quad` here
    }
}

extension ScannedItem: Equatable {
    
    public static func == (lhs: ScannedItem, rhs: ScannedItem) -> Bool {
        lhs.id == rhs.id
    }
}

extension ScannedItem {
    
    func createItem(originalImage: UIImage,
                    renderedImage: UIImage,
                    type: ScannedItemType) -> ScannedItem {
        
        let newScannedItem = ScannedItem(originalImage: originalImage,
                                         renderImage: renderedImage,
                                         quad: quad,
                                         colorOption: colorOption,
                                         type: type)
        
        newScannedItem.rotation = rotation
        newScannedItem.signaturePosition = signaturePosition
        newScannedItem.size = size
        newScannedItem.rotate = rotate
        
        return newScannedItem
    }
}

public class MultiPageScanSession: Codable {
    
    public private(set) var scannedItems:Array<ScannedItem> = []
    
    public init(){}
    
    public func add(item:ScannedItem){
        self.scannedItems.append(item)
    }
    
    public func add(items: [ScannedItem]) {
        self.scannedItems.append(contentsOf: items)
    }
    
    public func add(item: ScannedItem, at: Int) {
        self.scannedItems.insert(item, at: at)
    }
    
    public func remove(index:Int){
        self.scannedItems.remove(at: index)
    }
    
    public func replace(with item: ScannedItem, at: Int) {
        if (scannedItems.indices.contains(at)) == true {
            scannedItems[at] = item
        }
    }
    
    public func replaceAndGet(item: ScannedItem, at: Int) -> MultiPageScanSession{
        if (scannedItems.indices.contains(at)) == true {
            scannedItems[at] = item
        }
        return self
    }
    
    public func removeAll() {
        self.scannedItems.removeAll()
    }
    
}

extension ScannedItem {
    private func filePath() -> URL {
        folderUrl.appendingPathComponent("\(UUID().uuidString).jpg")
    }
}

class PDFDocumentObject: Codable {
    let name: String
    let session: MultiPageScanSession
    
    enum CodingKeys: CodingKey {
        case name, session
    }
    
    public required init(from decoder: Decoder) throws {
        let container = try decoder.container(keyedBy: CodingKeys.self)
        name = try! container.decode(String.self, forKey: .name)
        session = try container.decode(MultiPageScanSession.self, forKey: .session)
    }
    
    public init(name: String, session: MultiPageScanSession) {
        self.name = name
        self.session = session
    }
    
    public func encode(to encoder: Encoder) throws {
        var container = encoder.container(keyedBy: CodingKeys.self)
        
        try container.encode(name, forKey: .name)
        try container.encode(session, forKey: .session)
    }
}
