//
//  ScannerViewController.swift
//  WeScan
//
//  Created by Boris Emorine on 2/8/18.
//  Copyright © 2018 WeTransfer. All rights reserved.
//

import UIKit
import AVFoundation
import NBBottomSheet
import ImageViewer_swift

/// An enum used to know if the flashlight was toggled successfully.
enum FlashResult {
    case successful
    case notSuccessful
}

protocol ScannerViewControllerDelegate : NSObjectProtocol {
    func scannerViewController(_ scannerViewController:ScannerViewController, reviewItems inSession:MultiPageScanSession)
    func scannerViewController(_ scannerViewController:ScannerViewController, didFail withError:Error)
    func scannerViewControllerDidCancel(_ scannerViewController:ScannerViewController)
}

protocol ScannerViewControllerPreviewDelegate {
    func addToPDF(_ scannerViewController: ScannerViewController, session: ScanSession)
}

public final class ScannerViewController: UIViewController, ScanSessionSharable, ScanSessionPrintable {
    
    @Inject var addScanSessionUseCase: AddScanSessionUseCase
    @Inject var deleteScanSessionUseCase: DeleteScanSessionUseCase
    
    var captureSessionManager: CaptureSessionManager?
    let videoPreviewLayer = AVCaptureVideoPreviewLayer()
    var focusRectangle: FocusRectangleView!
    let quadView = QuadrilateralView()
    let visualEffectView = UIVisualEffectView(effect: UIBlurEffect(style: .dark))
    var flashEnabled = false
    
    var deviceOrientationHelper = DeviceOrientationHelper()
    var scanSession: ScanSession!
    var options:ImageScannerOptions!
    
    weak var delegate: ScannerViewControllerDelegate?
    var previewDelegate: ScannerViewControllerPreviewDelegate?
    
    var selectedFolder: Folder?
    
    var isAddedToPDF: Bool = false
    
    public override var prefersStatusBarHidden: Bool {
        true
    }
    
    var shareType: ShareType = .pdf
    
    var sharingViewController: UIViewController { self }
    
    private var name: String = ""
    
    /// The original bar style that was set by the host app
    private var originalBarStyle: UIBarStyle?
    
    private var tabbarView: CustomTabBarView?
    
    /// A black UIView, used to quickly display a black screen when the shutter button is presseed.
    internal let blackFlashView: UIView = {
        let view = UIView()
        view.backgroundColor = UIColor(white: 0.0, alpha: 0.5)
        view.isHidden = true
        view.translatesAutoresizingMaskIntoConstraints = false
        return view
    }()
    
    lazy private var autoScanButton: UIBarButtonItem = {
        var title = NSLocalizedString("wescan.scanning.auto", tableName: nil, bundle: Bundle(for: ScannerViewController.self), value: "Auto", comment: "The auto button state")
        if !self.options.allowAutoScan{
            title = NSLocalizedString("wescan.scanning.manual", tableName: nil, bundle: Bundle(for: ScannerViewController.self), value: "Manual", comment: "The manual button state")
        }
        
        let button = UIBarButtonItem(title: title, style: .plain, target: self, action: #selector(toggleAutoScan))
        button.tintColor = .white
        
        return button
    }()
    
    lazy private var flashButton: UIBarButtonItem = {
        let flashImage = UIImage(named: "flash", in: Bundle(for: ScannerViewController.self), compatibleWith: nil)
        let flashButton = UIBarButtonItem(image: flashImage, style: .plain, target: self, action: #selector(toggleFlash))
        flashButton.tintColor = .white
        
        return flashButton
    }()
    
    lazy private var cancelBarButton: UIBarButtonItem = {
        let cancelBarButton = UIBarButtonItem(title: .cancel.localized, style: .done, target: self, action: #selector(cancelTapped))
        cancelBarButton.tintColor = .white
        return cancelBarButton
    }()
    
    lazy var activityIndicator: UIActivityIndicatorView = {
        let activityIndicator = UIActivityIndicatorView(style: .large)
        activityIndicator.color = .gray
        activityIndicator.hidesWhenStopped = true
        activityIndicator.translatesAutoresizingMaskIntoConstraints = false
        return activityIndicator
    }()
    
    // MARK: - Initializers
    
    init(scanSession: ScanSession?, options:ImageScannerOptions?, isAddedToPDF: Bool = false) {
        super.init(nibName: nil, bundle: nil)
        self.scanSession = scanSession ?? .newSession()
        self.options = options ?? ImageScannerOptions()
        tabbarView = CustomTabBarView(frame: .zero, isAddedToPDF: isAddedToPDF)
        tabbarView?.delegate = self
        self.isAddedToPDF = isAddedToPDF
        
    }
    
    init() {
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder aDecoder: NSCoder) {
        self.scanSession = .newSession()
        self.options = ImageScannerOptions()
        super.init(coder: aDecoder)
    }
    
    // MARK: - Life Cycle
    
    public override func viewDidLoad() {
        super.viewDidLoad()
        
        title = nil
        
        setupViews()
        setupNavigationBar()
        setupConstraints()
        
        captureSessionManager = CaptureSessionManager(videoPreviewLayer: videoPreviewLayer)
        captureSessionManager?.delegate = self
        CaptureSession.current.isAutoScanEnabled = self.options.allowAutoScan
        
        originalBarStyle = navigationController?.navigationBar.barStyle
        
        NotificationCenter.default.addObserver(self, selector: #selector(subjectAreaDidChange), name: Notification.Name.AVCaptureDeviceSubjectAreaDidChange, object: nil)
    }
    
    public override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        setNeedsStatusBarAppearanceUpdate()
        
        CaptureSession.current.isEditing = false
        quadView.removeQuadrilateral()
        captureSessionManager?.start()
        UIApplication.shared.isIdleTimerDisabled = true
        navigationController?.navigationBar.isTranslucent = true
        navigationController?.navigationBar.setBackgroundImage(UIImage(), for: .default)
        navigationController?.setToolbarHidden(true, animated: true)
        
        updateCounterButton()
        
        deviceOrientationHelper.startDeviceOrientationNotifier { (deviceOrientation) in
            print("orientation: \(deviceOrientation)")
            self.orientationChanged(deviceOrientation: deviceOrientation)
        }
    }
    
    public override func viewDidDisappear(_ animated: Bool) {
        deviceOrientationHelper.stopDeviceOrientationNotifier()
        captureSessionManager?.stop()
        super.viewDidDisappear(true)
    }
    
    public override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        
        videoPreviewLayer.frame = view.layer.bounds
        
        let statusBarHeight = UIApplication.shared.statusBarFrame.size.height
        let visualEffectRect = self.navigationController?.navigationBar.bounds.insetBy(dx: 0, dy: -(statusBarHeight)).offsetBy(dx: 0, dy: -statusBarHeight)
        
        visualEffectView.frame = visualEffectRect ?? CGRect.zero
    }
    
    public override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        UIApplication.shared.isIdleTimerDisabled = false
        guard let device = AVCaptureDevice.default(for: AVMediaType.video) else { return }
        if device.torchMode == .on {
            toggleFlash()
        }
    }
    
    // MARK: - Setups
    
    private func setupViews() {
        view.layer.addSublayer(videoPreviewLayer)
        quadView.translatesAutoresizingMaskIntoConstraints = false
        quadView.editable = false
        view.addSubview(quadView)
        view.addSubview(activityIndicator)
        view.addSubview(blackFlashView)
        if let tabbar = tabbarView {
            view.addSubview(tabbar)
        }
    }
    
    private func setupNavigationBar() {
        if isAddedToPDF {
            navigationItem.setLeftBarButtonItems([cancelBarButton, flashButton], animated: false)
        } else {
            navigationItem.setLeftBarButton(flashButton, animated: false)
        }
        navigationItem.setRightBarButton(autoScanButton, animated: false)
        
        if UIImagePickerController.isFlashAvailable(for: .rear) == false {
            let flashOffImage = UIImage(named: "flashUnavailable", in: Bundle(for: ScannerViewController.self), compatibleWith: nil)
            flashButton.image = flashOffImage
            flashButton.tintColor = UIColor.lightGray
        }
    }
    
    private func setupConstraints() {
        var quadViewConstraints = [NSLayoutConstraint]()
        var activityIndicatorConstraints = [NSLayoutConstraint]()
        var blackFlashViewConstraints = [NSLayoutConstraint]()
        var tabbarViewConstraints = [NSLayoutConstraint]()
        
        quadViewConstraints = [
            quadView.topAnchor.constraint(equalTo: view.topAnchor),
            view.bottomAnchor.constraint(equalTo: quadView.bottomAnchor),
            view.trailingAnchor.constraint(equalTo: quadView.trailingAnchor),
            quadView.leadingAnchor.constraint(equalTo: view.leadingAnchor)
        ]
        
        if let tabbarView = tabbarView {
            tabbarViewConstraints = [
                tabbarView.leadingAnchor.constraint(equalTo: view.leadingAnchor, constant: 0),
                tabbarView.trailingAnchor.constraint(equalTo: view.trailingAnchor, constant: 0),
                tabbarView.bottomAnchor.constraint(equalTo: view.bottomAnchor, constant: 0),
                tabbarView.heightAnchor.constraint(equalToConstant: 225)
            ]
        }
        
        activityIndicatorConstraints = [
            activityIndicator.centerXAnchor.constraint(equalTo: view.centerXAnchor),
            activityIndicator.centerYAnchor.constraint(equalTo: view.centerYAnchor)
        ]
        
        blackFlashViewConstraints = [
            blackFlashView.topAnchor.constraint(equalTo: view.topAnchor),
            blackFlashView.leadingAnchor.constraint(equalTo: view.leadingAnchor),
            view.bottomAnchor.constraint(equalTo: blackFlashView.bottomAnchor),
            view.trailingAnchor.constraint(equalTo: blackFlashView.trailingAnchor)
        ]
        
        tabbarView?.translatesAutoresizingMaskIntoConstraints = false
        
        NSLayoutConstraint.activate(quadViewConstraints + activityIndicatorConstraints + blackFlashViewConstraints + tabbarViewConstraints)
    }
    
    private func flashToBlack() {
        view.bringSubviewToFront(blackFlashView)
        blackFlashView.isHidden = false
        let flashDuration = DispatchTime.now() + 0.05
        DispatchQueue.main.asyncAfter(deadline: flashDuration) {
            self.blackFlashView.isHidden = true
        }
    }
    
    // MARK: - Tap to Focus
    
    /// Called when the AVCaptureDevice detects that the subject area has changed significantly. When it's called, we reset the focus so the camera is no longer out of focus.
    @objc private func subjectAreaDidChange() {
        /// Reset the focus and exposure back to automatic
        do {
            try CaptureSession.current.resetFocusToAuto()
        } catch {
            let error = ImageScannerControllerError.inputDevice
            guard let captureSessionManager = captureSessionManager else { return }
            captureSessionManager.delegate?.captureSessionManager(captureSessionManager, didFailWithError: error)
            return
        }
        
        /// Remove the focus rectangle if one exists
        CaptureSession.current.removeFocusRectangleIfNeeded(focusRectangle, animated: true)
    }
    
    public override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        super.touchesBegan(touches, with: event)
        
        if self.options.allowTapToFocus{
            guard  let touch = touches.first else { return }
            let touchPoint = touch.location(in: view)
            let convertedTouchPoint: CGPoint = videoPreviewLayer.captureDevicePointConverted(fromLayerPoint: touchPoint)
            
            CaptureSession.current.removeFocusRectangleIfNeeded(focusRectangle, animated: false)
            
            focusRectangle = FocusRectangleView(touchPoint: touchPoint)
            view.addSubview(focusRectangle)
            
            do {
                try CaptureSession.current.setFocusPointToTapPoint(convertedTouchPoint)
            } catch {
                let error = ImageScannerControllerError.inputDevice
                guard let captureSessionManager = captureSessionManager else { return }
                captureSessionManager.delegate?.captureSessionManager(captureSessionManager, didFailWithError: error)
                return
            }
        }
    }
    
    private func updateCounterButton() {
        tabbarView?.updateScannedView(session: scanSession)
    }
    
    // MARK: Rotation methods
    private func orientationChanged(deviceOrientation: UIDeviceOrientation) {
        print("Orientation changed: \(self.deviceOrientationHelper.currentDeviceOrientation.rawValue)")
    }
    
    private func getCurrentRotationAngle()->Double{
        switch self.deviceOrientationHelper.currentDeviceOrientation {
        case .landscapeRight:
            return -90.0
        case .landscapeLeft:
            return 90.0
        case .portrait:
            return 0.0
        case .portraitUpsideDown:
            return 180.0
        default:
            return 0.0
        }
    }
    
    // MARK: - Actions
    
    @objc private func toggleAutoScan() {
        if CaptureSession.current.isAutoScanEnabled {
            CaptureSession.current.isAutoScanEnabled = false
            autoScanButton.title = NSLocalizedString("wescan.scanning.manual", tableName: nil, bundle: Bundle(for: ScannerViewController.self), value: "Manual", comment: "The manual button state")
        } else {
            CaptureSession.current.isAutoScanEnabled = true
            autoScanButton.title = NSLocalizedString("wescan.scanning.auto", tableName: nil, bundle: Bundle(for: ScannerViewController.self), value: "Auto", comment: "The auto button state")
        }
    }
    
    @objc private func toggleFlash() {
        let state = CaptureSession.current.toggleFlash()
        
        let flashImage = UIImage(named: "flash", in: Bundle(for: ScannerViewController.self), compatibleWith: nil)
        let flashOffImage = UIImage(named: "flashUnavailable", in: Bundle(for: ScannerViewController.self), compatibleWith: nil)
        
        switch state {
        case .on:
            flashEnabled = true
            flashButton.image = flashImage
            flashButton.tintColor = .yellow
        case .off:
            flashEnabled = false
            flashButton.image = flashImage
            flashButton.tintColor = .white
        case .unknown, .unavailable:
            flashEnabled = false
            flashButton.image = flashOffImage
            flashButton.tintColor = UIColor.lightGray
        }
    }
    
    @objc private func cancelTapped() {
        discardSessionAndNavigateBack(animated: true)
    }
    
    @objc private func cancelImageScannerController() {
        self.navigationController?.popViewController(animated: true)
    }
    
    private func previewDocument() {
        displayEditViewController()
    }
    
    private func displayEditViewController() {
        let editViewController = EditViewController(scanSession: scanSession)
        editViewController.selectedFolder = selectedFolder
        navigationController?.pushViewController(editViewController, animated: true)
    }
    
    private func saveScanSession(name: String) {
        
        if let selectedFolder {
            
            if selectedFolder != .allDocuments {
                addScanSession(name: name, to: selectedFolder)
            }
            else {
                displayFoldersViewSheet()
            }
        }
    }
    
    private func addScanSession(name: String, to folder: Folder) {
        
        if !name.isEmpty {
            scanSession.name = name
        }
        else {
            scanSession.name = .getScanSessionName()
        }
        
        scanSession.folderId = folder.id
        
        scanSession.creationDate = .now
        
        addScanSessionUseCase.execute(with: scanSession, to: folder)
        
        navigationController?.popViewController(animated: true)
    }
    
    private func displayFoldersViewSheet() {
        
        let foldersViewController = FoldersViewController()
        
        foldersViewController.delegate = self
        
        present(foldersViewController, animated: true)
    }
}

extension ScannerViewController: FoldersViewControllerDelegate {
    
    func dismissed(_ foldersViewController: FoldersViewController, folder: Folder?) {
        
        if let folder {
            addScanSession(name: name, to: folder)
        }
    }
}

extension ScannerViewController: RectangleDetectionDelegateProtocol {
    func captureSessionManager(_ captureSessionManager: CaptureSessionManager, didFailWithError error: Error) {
        
        activityIndicator.stopAnimating()
        tabbarView?.scanButton.isUserInteractionEnabled = true
        self.delegate?.scannerViewController(self, didFail: error)
    }
    
    func didStartCapturingPicture(for captureSessionManager: CaptureSessionManager) {
        activityIndicator.startAnimating()
    }
    
    func captureSessionManager(_ captureSessionManager: CaptureSessionManager, didCapturePicture picture: UIImage, withQuad quad: Quadrilateral?) {
        
        activityIndicator.stopAnimating()
        tabbarView?.scanButton.isUserInteractionEnabled = true
        
        let scannedItem = ScannedItem(originalImage: picture, quad: quad)
        
        scannedItem.rotation = self.getCurrentRotationAngle()
        scannedItem.colorOption = self.options.defaultColorRenderOption
        
        scannedItem.render { [weak self] _ in
            guard let self else { return }
            
            let compressedScannedItem = ImageCompressManager.compress(scannedItem: scannedItem)
            
            scanSession.add(item: compressedScannedItem)
            
            updateCounterButton()
        }
        
        self.captureSessionManager?.start()
    }
    
    func captureSessionManager(_ captureSessionManager: CaptureSessionManager, didDetectQuad quad: Quadrilateral?, _ imageSize: CGSize) {
        guard let quad = quad else {
            // If no quad has been detected, we remove the currently displayed on on the quadView.
            quadView.removeQuadrilateral()
            return
        }
        
        let portraitImageSize = CGSize(width: imageSize.height, height: imageSize.width)
        
        let scaleTransform = CGAffineTransform.scaleTransform(forSize: portraitImageSize, aspectFillInSize: quadView.bounds.size)
        let scaledImageSize = imageSize.applying(scaleTransform)
        
        let rotationTransform = CGAffineTransform(rotationAngle: CGFloat.pi / 2.0)
        
        let imageBounds = CGRect(origin: .zero, size: scaledImageSize).applying(rotationTransform)
        
        let translationTransform = CGAffineTransform.translateTransform(fromCenterOfRect: imageBounds, toCenterOfRect: quadView.bounds)
        
        let transforms = [scaleTransform, rotationTransform, translationTransform]
        
        let transformedQuad = quad.applyTransforms(transforms)
        
        quadView.drawQuadrilateral(quad: transformedQuad, animated: true)
    }
}

// MARK: - CustomTabBarViewDelegate
extension ScannerViewController: CustomTabBarViewDelegate {
    
    func scanTapped() {
        self.flashToBlack()
        tabbarView?.scanButton.isUserInteractionEnabled = false
        captureSessionManager?.capturePhoto()
    }
    
    func fileManagerTapped() {
        if scanSession.scannedItems.isEmpty {
            self.navigationController?.popViewController(animated: false)
        } else {
            let alertController = UIAlertController(title: .alert.localized,
                                                    message: .discardScannerMsg.localized, preferredStyle: .alert)
            
            let okAction = UIAlertAction(title: .discard.localized, style: .destructive, handler: { [weak self] action in
                guard let self = self else { return }
                
                discardSessionAndNavigateBack(animated: false)
            })
            
            alertController.addAction(okAction)
            
            alertController.addAction(UIAlertAction(title: .cancel.localized, style: .cancel, handler: nil))
            present(alertController, animated: true)
        }
    }
    
    func editButtonTapped() {
        if !isAddedToPDF {
            
            displayEditViewController()
        } else {
            
            previewDelegate?.addToPDF(self, session: scanSession)
            navigationController?.popViewController(animated: true)
        }
    }
    
    func shareTapped() {
        
        if !isAddedToPDF {
            
            let popup = ShareSheetViewController()
            
            popup.session = scanSession
            popup.delegate = self
            popup.previewDelegate = self
            
            let configuration = NBBottomSheetConfiguration(animationDuration: 0.4, sheetSize: .fixed(popup.view.frame.height - view.safeAreaInsets.bottom))
            
            let bottomSheetController = NBBottomSheetController(configuration: configuration)
            
            bottomSheetController.present(popup, on: self)
        }
    }
}

extension ScannerViewController {
    
    private func discardSessionAndNavigateBack(animated: Bool) {
        
        deleteScanSessionUseCase.execute(with: scanSession)
        
        navigationController?.popViewController(animated: animated)
    }
}

// MARK: - ShareSheetViewControllerDelegate

extension ScannerViewController: ShareSheetViewControllerDelegate, PreviewDelegate {
    
    func previewTapped() {
        previewDocument()
    }
    
    func shareOptionTapped(name: String, shareOption: ShareOption, shareType: ShareType) {
        
        self.shareType = shareType
        
        if !name.isEmpty {
            scanSession.name = name
        }
        else {
            scanSession.name = .getScanSessionName()
        }
        
        shareScanSession(session: scanSession, shareOption: shareOption)
    }
    
    func printTapped(shareType: ShareType) {
        
        self.shareType = shareType
        
        printScanSession(session: scanSession)
    }
    
    func saveTapped(name: String) {
        self.name = name
        saveScanSession(name: name)
    }
}
